unit AXWActions;

{$B-}
{$H+}
{$R-}
{$I AxCompilers.inc}
{$I AXW.inc}

interface

uses Classes, SysUtils, Types, Contnrs, Vcl.Dialogs, Vcl.Controls, Vcl.StdCtrls, Vcl.ActnList,
{$ifdef DELPHI_XE2_OR_LATER}
     UITypes,
{$endif}
     Vcl.ComCtrls, Vcl.Menus, Vcl.ExtDlgs, Math, Vcl.ImgList,
     AXWActionsBase,
     AXWDefs, AXWUtils, AXWKeys, AXWFramework, AXWDocProps, AXWWriterVcl,
     AXWGraphics, AXWLogParas, AXWLogEditor, AXWEditor,
     AXWFormColorPickerVcl, AXWFormFindReplaceVcl, AXWFormPasteSpecialVcl, AXWFormInserTableVcl,
     AXWFormBordersVcl, AXWFormParaProps, AXWFormStylesVcl, AXWFormTabsVcl, AXWFormHyperlinkVcl,
     AXWFormBookmarksVcl, AXWFormRevisionPaneVcl, AXWFormDocStructVcl, AXWPreviewVcl, AXWFormPreviewVcl,
     AXWFormOptions, AXWFormTextInputVcl, AXWFormSpellCheckVcl, AXWFormPageSetupVcl,
     AXWFormSelectFontVcl;

const AXWALineSpacingValues: array[0..5] of double = (1.0,1.15,1.5,2.0,2.5,3.0);

type TAXWAction = class(TAXWActionBase)
private
     function  GetAXWWriter: TAXWWriter;
protected
     FLargeImageIndex: integer;

     function  CanExecute: boolean;

     procedure DummyExecute(ASender: TObject);
public
     constructor Create(AOwner: TComponent); override;

     property AXWWriter      : TAXWWriter read GetAXWWriter;
published
     property LargeImageIndex: integer read FLargeImageIndex write FLargeImageIndex;
     end;

type TAXWActionFileOpen = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionCmd = class(TAXWAction)
protected
     FCommand: TAXWCommand;

     procedure SetValue(AValue: double); overload; virtual;
     procedure SetValue(AValue: integer); overload; virtual;
     procedure SetValue(AValue: AxUCString);  overload; virtual;
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;

     property Command: TAXWCommand read FCommand;
     end;

type TAXWActionCmdToggle = class(TAXWActionCmd)
protected
public
     destructor Destroy; override;

     procedure Loaded; override;
     end;

type TAXWActionCmdPopup = class(TAXWActionCmd)
protected
     FPop      : TPopupMenu;
     FAutoCheck: boolean;

     procedure MenuItemExecuted(ASender: TObject); virtual;

     procedure AddMenuItem(ACaption: AxUCString; AChecked: boolean = False);
public
     constructor Create(AOwner: TComponent); override;

     procedure Prepare; override;
     function  HasObject(AClass: TClass): TObject; override;
     end;

type TAXWActionFontUnderline = class(TAXWActionCmdPopup)
protected
public
     constructor Create(AOwner: TComponent); override;

     procedure Prepare; override;
     end;

type TAXWActionParaBullet = class(TAXWActionCmdPopup)
protected
public
     constructor Create(AOwner: TComponent); override;

     procedure Prepare; override;
     end;

type TAXWActionParaNumbering = class(TAXWActionCmdPopup)
protected
public
     constructor Create(AOwner: TComponent); override;

     procedure Prepare; override;
     end;

type TAXWActionParaMultilevelList = class(TAXWActionCmdPopup)
protected
     procedure MenuItemExecuted(ASender: TObject); override;
public
     constructor Create(AOwner: TComponent); override;

     procedure Prepare; override;
     end;

type TAXWActionParaLineSpacing = class(TAXWActionCmdPopup)
protected
     procedure MenuItemExecuted(ASender: TObject); override;
public
     constructor Create(AOwner: TComponent); override;

     procedure Prepare; override;
     end;

type TAXWActionColorDialog = class(TAXWActionCmd)
protected
public
     function  Execute: boolean; override;
     end;

type TAXWActionFind = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionFindNext = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionReplace = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionInsertTable = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionInsertPicture = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionBorders = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionParagraphProps = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionStyles = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionTabs = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionHyperlink = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionBookmarks = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;
     end;

type TAXWActionTextBox = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionFontBold = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionFontColor = class(TAXWActionCmd)
protected
     FColor: longword;

     procedure SetValue(AValue: AxUCString); override;
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;

     procedure Loaded; override;
     end;

type TAXWActionParaColor = class(TAXWActionCmd)
protected
     FColor: longword;

     procedure SetValue(AValue: AxUCString); override;
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;

     procedure Loaded; override;
     end;

type TAXWActionFontColorDialog = class(TAXWActionColorDialog)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionParaColorDialog = class(TAXWActionColorDialog)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionFontItalic = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionFontStrikeThrough = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionFontSubscript = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionFontSuperscript = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionEditCopy = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionEditCut = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionEditPaste = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionEditPasteSpecial = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionFontSize = class(TAXWActionCmd)
protected
     FSize: double;

     procedure SetValue(AValue: AxUCString); override;
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;

     procedure Loaded; override;
     end;

type TAXWActionFontName = class(TAXWActionCmd)
protected
     procedure SetValue(AValue: AxUCString); override;
public
     constructor Create(AOwner: TComponent); override;

     function  Execute: boolean; override;

     procedure Loaded; override;
     end;

type TAXWActionFontSizeInc = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionFontSizeDec = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionFontProps = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionParaIndentInc = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionParaIndentDec = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionParaAlignLeft = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionParaAlignCenter = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionParaAlignRight = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionParaAlignJustify = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionRevTrackChanges = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionRevReviewingPane = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionRevAccept = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionRevAcceptAll = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionRevReject = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionRevRejectAll = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionRevNext = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionRevPrevious = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionRevShowMarkup = class(TAXWActionCmdPopup)
protected
     procedure MenuItemExecuted(ASender: TObject); override;
public
     constructor Create(AOwner: TComponent); override;

     procedure Prepare; override;

     function Execute: boolean; override;
     end;

type TAXWActionShowDocStructure = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionPrint = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionPrintPreview = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionOptions = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionEncrypt = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionSpellCheck = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionSpellCheckAll = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionSpellCheckClear = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionPageSetup = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionPageMargins = class(TAXWActionCmdPopup)
protected
     procedure MenuItemExecuted(ASender: TObject); override;
public
     constructor Create(AOwner: TComponent); override;

     procedure Prepare; override;
     end;

type TAXWActionPageOrientation = class(TAXWActionCmdPopup)
protected
     procedure MenuItemExecuted(ASender: TObject); override;
public
     constructor Create(AOwner: TComponent); override;

     procedure Prepare; override;
     end;

type TAXWActionPageSize = class(TAXWActionCmdPopup)
protected
     procedure MenuItemExecuted(ASender: TObject); override;
public
     constructor Create(AOwner: TComponent); override;

     procedure Prepare; override;
     end;

type TAXWActionLayoutPrint = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionLayoutWeb = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionMacroRecord = class(TAXWActionCmdToggle)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionMacroPlay = class(TAXWActionCmd)
protected
public
     constructor Create(AOwner: TComponent); override;
     end;

type TAXWActionZoom = class(TAXWActionCmdPopup)
protected
     procedure MenuItemExecuted(ASender: TObject); override;
public
     constructor Create(AOwner: TComponent); override;

     procedure Prepare; override;
     end;

type TAXWActionZoom100 = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionShowMargins = class(TAXWAction)
protected
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionShowRuler = class(TAXWAction)
protected
     procedure Loaded; override;
public
     constructor Create(AOwner: TComponent); override;

     function Execute: boolean; override;
     end;

type TAXWActionList = class(TAXWActionListBase)
private
     function  GetImages: TCustomImageList;
     procedure Setmages(const Value: TCustomImageList);
     procedure SetLargeImages(const Value: TCustomImageList);
protected
     FAXWWriter  : TAXWWriter;
     FComponents : TObjectList;
     FLargeImages: TCustomImageList;
     FPopup      : TPopupMenu;

     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

     procedure AddComponent(AComponent: TComponent);
     procedure RemoveComponent(AComponent: TComponent);

     procedure DoAXWCommand(ASender: TObject; ACommand: TAXWCommand; var AResult: boolean);
public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;

     procedure SetAXWWriter(AObject: TObject); override;

     property PopupMenu  : TPopupMenu read FPopup;
     property AXWWriter  : TAXWWriter read FAXWWriter;
published
     property SmallImages: TCustomImageList read GetImages write Setmages;
     property LargeImages: TCustomImageList read FLargeImages write SetLargeImages;
     end;

//type TAXWComboBox = class(TComboBox)
//private
//     procedure Set_Action(const Value: TAction);
//     function  Get_Action: TAction;
//protected
//     FAction: TAXWActionCmd;
//
//     procedure Loaded; override;
//     procedure CloseUp; override;
//
//     procedure DoCommand(ACommand: TAXWCommand);
//public
//published
//     property Action: TAction read Get_Action write Set_Action;
//     end;

implementation

{ TAXWActionFileOpen }

constructor TAXWActionFileOpen.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Open';
end;

function TAXWActionFileOpen.Execute: boolean;
var
  Dlg: TOpenDialog;
begin
//  inherited;

  Result := CanExecute;

  if Result then begin
    Dlg := TOpenDialog.Create(Nil);
    try
      Dlg.Filter := AXW_FILES_READ_FILTER;
      Dlg.FileName := AXWWriter.Filename;

      if Dlg.Execute then
        AXWWriter.LoadFromFile(Dlg.FileName);
    finally
      Dlg.Free;
    end;
  end;
end;

{ TAXWActionList }

procedure TAXWActionList.AddComponent(AComponent: TComponent);
begin
  FComponents.Remove(AComponent);

  FComponents.Add(AComponent);
end;

constructor TAXWActionList.Create(AOwner: TComponent);
begin
  inherited;

  FComponents := TObjectList.Create(False);
  FPopup := TPopupMenu.Create(Self);
end;

destructor TAXWActionList.Destroy;
begin
  FComponents.Free;
  FPopup.Free;

  inherited;
end;

procedure TAXWActionList.DoAXWCommand(ASender: TObject; ACommand: TAXWCommand; var AResult: boolean);
var
  i: integer;
  V: double;
begin
  if FAXWWriter = Nil then
    Exit;

  for i := 0 to FComponents.Count - 1 do begin
//    if FComponents[i] is TAXWComboBox then
//      TAXWComboBox(FComponents[i]).DoCommand(ACommand)
//    else
    if FComponents[i] is TAXWActionFontName then
      TAXWActionFontName(FComponents[i]).Caption := FAXWWriter.Editor.CaretPos.CHPX.FontName
    else if FComponents[i] is TAXWActionFontSize then begin
      V := RoundTo(FAXWWriter.Editor.CaretPos.CHPX.Size,-1);
      if Frac(V) > 0 then
        TAXWActionFontSize(FComponents[i]).Caption := Format('%.1f',[V])
      else
        TAXWActionFontSize(FComponents[i]).Caption := FloatToStr(V);
    end
    else if FComponents[i] is TAXWActionCmdToggle then begin
      case TAXWActionCmdToggle(FComponents[i]).Command of
        axcFormatBold           : TAXWActionCmdToggle(FComponents[i]).Checked := FAXWWriter.Editor.CaretPos.CHPX.Bold;
        axcFormatItalic         : TAXWActionCmdToggle(FComponents[i]).Checked := FAXWWriter.Editor.CaretPos.CHPX.Italic;
        axcFormatStrikeThrough  : TAXWActionCmdToggle(FComponents[i]).Checked := FAXWWriter.Editor.CaretPos.CHPX.StrikeThrough;
        axcFormatSubscript      : TAXWActionCmdToggle(FComponents[i]).Checked := FAXWWriter.Editor.CaretPos.CHPX.SubSuperscript = axcssSubscript;
        axcFormatSuperscript    : TAXWActionCmdToggle(FComponents[i]).Checked := FAXWWriter.Editor.CaretPos.CHPX.SubSuperscript = axcssSuperscript;
        axcFormatParaLeft       : TAXWActionCmdToggle(FComponents[i]).Checked := FAXWWriter.Editor.CaretPos.PAPX.Alignment = axptaLeft;
        axcFormatParaCenter     : TAXWActionCmdToggle(FComponents[i]).Checked := FAXWWriter.Editor.CaretPos.PAPX.Alignment = axptaCenter;
        axcFormatParaRight      : TAXWActionCmdToggle(FComponents[i]).Checked := FAXWWriter.Editor.CaretPos.PAPX.Alignment = axptaRight;
        axcFormatParaJustify    : TAXWActionCmdToggle(FComponents[i]).Checked := FAXWWriter.Editor.CaretPos.PAPX.Alignment = axptaJustify;

        axcRevisionTrackChanges : TAXWActionCmdToggle(FComponents[i]).Checked := FAXWWriter.Editor.MainDoc.Revision.TrackChanges;
      end;
    end;
  end;
end;

function TAXWActionList.GetImages: TCustomImageList;
begin
  Result := Images;
end;

procedure TAXWActionList.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;

  if (Operation = opRemove) and (AComponent = FLargeImages) then
    FLargeImages := nil;
end;

procedure TAXWActionList.RemoveComponent(AComponent: TComponent);
begin
  FComponents.Remove(AComponent);
end;

procedure TAXWActionList.SetLargeImages(const Value: TCustomImageList);
begin
  if LargeImages <> nil then
    FLargeImages.RemoveFreeNotification(Self);

  FLargeImages := Value;

  if LargeImages <> nil then begin
    FLargeImages := Value;

    FLargeImages.FreeNotification(Self);
  end;
end;

procedure TAXWActionList.Setmages(const Value: TCustomImageList);
begin
  Images := Value;
end;

procedure TAXWActionList.SetAXWWriter(AObject: TObject);
begin
  if (AObject <> Nil) and not (AObject is TAXWWriter) then
    raise Exception.Create('Object is not TAXWWriter');

  if not (csDesigning in ComponentState) and (FAXWWriter <> Nil) then
    FAXWWriter.Editor.RemoveCmdListener(DoAXWCommand);

  FAXWWriter := TAXWWriter(AObject);

  if not ((csDesigning in ComponentState)) and (FAXWWriter <> Nil) then
    FAXWWriter.Editor.AddCmdListener(DoAXWCommand);
end;

{ TAXWAction }

function TAXWAction.CanExecute: boolean;
begin
  if not ((ActionList is TAXWActionList) and (TAXWActionList(ActionList).AXWWriter <> Nil)) then
    Result := False
  else
    Result := Enabled;
end;

constructor TAXWAction.Create(AOwner: TComponent);
begin
  inherited;

  FLargeImageIndex := -1;

  OnExecute := DummyExecute;
end;

procedure TAXWAction.DummyExecute(ASender: TObject);
begin

end;

function TAXWAction.GetAXWWriter: TAXWWriter;
begin
  Result := TAXWActionList(ActionList).AXWWriter;
end;

{ TAXWActionCmd }

constructor TAXWActionCmd.Create(AOwner: TComponent);
begin
  inherited;

  FCommand := axcNone;
end;

function TAXWActionCmd.Execute: boolean;
begin
//  inherited;

  Result := CanExecute and (FCommand <> axcNone);

  if Result then
    TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(FCommand);
end;

procedure TAXWActionCmd.SetValue(AValue: integer);
begin

end;

procedure TAXWActionCmd.SetValue(AValue: AxUCString);
begin

end;

procedure TAXWActionCmd.SetValue(AValue: double);
begin

end;

{ TAXWActionCmdEditCopy }

constructor TAXWActionEditCopy.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Copy';
  FCommand := axcClipboardCopy;
end;

{ TAXWActionEditCut }

constructor TAXWActionEditCut.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Cut';
  FCommand := axcClipboardCut;
end;

{ TAXWActionEditPaste }

constructor TAXWActionEditPaste.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Paste';
  FCommand := axcClipboardPaste;
end;

{ TAXWActionFontSize }

constructor TAXWActionFontSize.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Font Size';
  FSize := 10;
  FCommand := axcFormatFontSize;
end;

function TAXWActionFontSize.Execute: boolean;
begin
  Result := CanExecute and (FCommand <> axcNone);

  if Result then begin
    SetValue(Caption);

    TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(FCommand,FSize);
  end;
end;

procedure TAXWActionFontSize.Loaded;
begin
  inherited;

  if CanExecute then
    TAXWActionList(ACtionList).AddComponent(Self);
end;

procedure TAXWActionFontSize.SetValue(AValue: AxUCString);
var
  V: double;
begin
  V := StrToFloatDef(AValue,10);

  if (V > 0) and (V <= 1000) then
    FSize := V;
end;

//{ TAXWComboBox }
//
//function TAXWComboBox.Get_Action: TAction;
//begin
//  Result := FAction;
//end;
//
//procedure TAXWComboBox.Loaded;
//var
//  i: integer;
//begin
//  inherited;
//
////  Exit;
//
//  Style := csDropDownList;
//
//  Items.Clear;
//
//  if (FAction = Nil) or (FAction.ActionList = Nil) then
//    Exit;
//
//  case TAXWActionCmd(FAction).Command of
//    axcFormatFontSize: begin
//      for i := Low(AXWStdFontSizes) to High(AXWStdFontSizes) do
//        Items.Add(IntToStr(AXWStdFontSizes[i]));
//
//      TAXWActionList(FAction.ActionList).AddComponent(Self);
//    end;
//    axcFormatFontName: begin
//      AXWGetAvailableFonts(Items);
//
//      TAXWActionList(FAction.ActionList).AddComponent(Self);
//    end;
//  end;
//end;
//
//procedure TAXWComboBox.Set_Action(const Value: TAction);
//begin
//  if (Value <> Nil) and not (Value is TAXWActionCmd) then
//    Exit;
//
//  if (FAction <> Nil) and (FAction.ActionList is TAXWActionList) then
//    TAXWActionList(FAction.ActionList).RemoveComponent(Self);
//
//  FAction := TAXWActionCmd(Value);
//
//  if (FAction <> Nil) and (FAction.ActionList is TAXWActionList) then
//    TAXWActionList(FAction.ActionList).AddComponent(Self);
//end;
//
//procedure TAXWComboBox.CloseUp;
//begin
//  inherited;
//
//  if FAction <> Nil then begin
//    FAction.SetValue(Items[ItemIndex]);
//
//    FAction.Execute;
//  end;
//end;
//
//procedure TAXWComboBox.DoCommand(ACommand: TAXWCommand);
//var
//  n : integer;
//  s : AxUCString;
//  Ed: TAXWEditor;
//begin
//  if (FAction = Nil) or (TAXWActionList(FAction.ActionList).AXWWriter = Nil) then
//    Exit;
//
//  Ed := TAXWActionList(FAction.ActionList).AXWWriter.Editor;
//
//  case ACommand of
//    axcFormat: begin
//      case FAction.Command of
//        axcFormatFontSize: begin
//          if Ed.CaretPos.CHPX <> Nil then
//            n := Round(Ed.CaretPos.CHPX.Size)
//          else
//            n := Round(Ed.MainDoc.MasterCHP.Size);
//
//          ItemIndex := Items.IndexOf(IntToStr(n));
//        end;
//        axcFormatFontName: begin
//          if Ed.CaretPos.CHPX <> Nil then
//            s := Ed.CaretPos.CHPX.FontName
//          else
//            s := Ed.MainDoc.MasterCHP.FontName;
//
//          ItemIndex := Items.IndexOf(s);
//        end;
//      end;
//    end;
//  end;
//end;

{ TAXWActionFontName }

constructor TAXWActionFontName.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Typeface';
  FCommand := axcFormatFontName;
end;

function TAXWActionFontName.Execute: boolean;
begin
  Result := CanExecute and (FCommand <> axcNone);

  if Result then
    TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(FCommand,Caption);
end;

procedure TAXWActionFontName.Loaded;
begin
  inherited;

  if CanExecute then
    TAXWActionList(ACtionList).AddComponent(Self);
end;

procedure TAXWActionFontName.SetValue(AValue: AxUCString);
begin
  Caption := AValue;
end;

{ TAXWActionCmdToggle }

destructor TAXWActionCmdToggle.Destroy;
begin
  if CanExecute then
    TAXWActionList(ACtionList).RemoveComponent(Self);

  inherited;
end;

procedure TAXWActionCmdToggle.Loaded;
begin
  inherited;

  if CanExecute then
    TAXWActionList(ACtionList).AddComponent(Self);
end;

{ TAXWActionFontBoldBold }

constructor TAXWActionFontBold.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Font Bold';
  AutoCheck := True;
  FCommand := axcFormatBold;
end;

{ TAXWActionFontItalic }

constructor TAXWActionFontItalic.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Italic';
  AutoCheck := True;
  FCommand := axcFormatItalic;
end;

{ TAXWActionFontStrikeThrough }

constructor TAXWActionFontStrikeThrough.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Strike Through';
  FCommand := axcFormatStrikeThrough;
end;

{ TAXWActionCmdPopup }

procedure TAXWActionCmdPopup.AddMenuItem(ACaption: AxUCString; AChecked: boolean);
var
  MI: TMenuItem;
begin
  MI := TMenuItem.Create(Owner);
  MI.Caption := ACaption;
  MI.Checked := AChecked;
  MI.AutoCheck := FAutoCheck;

//  Will not remove &
//  MI.AutoHotkeys := maManual;
//  MI.AutoLineReduction := maManual;

  MI.OnClick := MenuItemExecuted;
  FPop.Items.Add(MI);
end;

constructor TAXWActionCmdPopup.Create(AOwner: TComponent);
begin
  inherited;

//  FPop := TPopupMenu.Create(Self);
end;

function TAXWActionCmdPopup.HasObject(AClass: TClass): TObject;
begin
  if AClass = TPopupMenu then
    Result := FPop
  else
    Result := Nil;
end;

procedure TAXWActionCmdPopup.MenuItemExecuted(ASender: TObject);
var
  i: integer;
begin
  if CanExecute then begin
    i := FPop.Items.IndexOf(TMenuItem(ASender));
    if i >= 0 then
      TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(FCommand,i);
  end;
end;

procedure TAXWActionCmdPopup.Prepare;
begin
  inherited Prepare;

  FPop := TAXWActionList(ActionList).FPopup;
  FPop.Items.Clear;

  FAutoCheck := False;
end;

{ TAXWActionFontUnderline }

constructor TAXWActionFontUnderline.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Underline';
  FCommand := axcFormatUnderline;
end;

procedure TAXWActionFontUnderline.Prepare;
begin
  inherited Prepare;

  AddMenuItem('None',False);
  AddMenuItem('Single',False);
  AddMenuItem('Words',False);
  AddMenuItem('Double',False);
  AddMenuItem('Thick',False);
  AddMenuItem('Dotted',False);
  AddMenuItem('Dotted Heavy',False);
  AddMenuItem('Dash',False);
  AddMenuItem('Dashed Heavy',False);
  AddMenuItem('DashLong',False);
  AddMenuItem('Dash Long Heavy',False);
  AddMenuItem('Dot Dash',False);
  AddMenuItem('Dash Dot Heavy',False);
  AddMenuItem('Dot Dot Dash',False);
  AddMenuItem('Dash Dot Dot Heavy',False);
  AddMenuItem('Wave',False);
  AddMenuItem('Wavy Heavy',False);
  AddMenuItem('Wavy Double',False);
end;

{ TAXWActionColorDialog }

function TAXWActionColorDialog.Execute: boolean;
var
  Frm: TfrmAXWColorPicker;
  Cl : longword;
begin
//  inherited Execute;

  Result := CanExecute;

  if Result then begin
    Frm := TfrmAXWColorPicker.Create(Self);
    try
      Result := Frm.Execute(Cl);

      if Result then
        TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(FCommand,Cl);
    finally
      Frm.Free;
    end;
  end;
end;

{ TAXWActionFontColor }

constructor TAXWActionFontColorDialog.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Font Color';
  FCommand := axcFormatColor;
end;

{ TAXWActionFontColor }

constructor TAXWActionFontColor.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Font Color';
  FColor := AXW_COLOR_AUTOMATIC;
  FCommand := axcFormatColor;
end;

function TAXWActionFontColor.Execute: boolean;
begin
  Result := CanExecute and (FCommand <> axcNone);

  if Result then begin
    SetValue(Caption);

    TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(FCommand,FColor);
  end;
end;

procedure TAXWActionFontColor.Loaded;
begin
  inherited;

  if CanExecute then
    TAXWActionList(ACtionList).AddComponent(Self);
end;

procedure TAXWActionFontColor.SetValue(AValue: AxUCString);
begin
  FColor := StrToIntDef(AValue,0);
end;

{ TAXWActionFontSuperscript }

constructor TAXWActionFontSuperscript.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Superscript';
  FCommand := axcFormatSuperscript;
end;

{ TAXWActionFontSubscript }

constructor TAXWActionFontSubscript.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Subscript';
  FCommand := axcFormatSubscript;
end;

{ TAXWActionFontSizeInc }

constructor TAXWActionFontSizeInc.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Increase Font';
  FCommand := axcFormatIncFont;
end;

{ TAXWActionFontSizeDec }

constructor TAXWActionFontSizeDec.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Decrease Font';
  FCommand := axcFormatDecFont;
end;

{ TAXWActionEditPasteSpecial }

constructor TAXWActionEditPasteSpecial.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Paste Special';
end;

function TAXWActionEditPasteSpecial.Execute: boolean;
var
  Frm: TfrmAXWPasteSpecial;
  Fmt: TAXWClipboardFormat;
begin
  Result := CanExecute;

  if Result then begin
    Frm := TfrmAXWPasteSpecial.Create(Self);
    try
       Fmt := Frm.Execute;
       if Fmt <> acfNone then
         TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcClipboardPaste,Integer(Fmt));
    finally
      Frm.Free;
    end;
  end;
end;

{ TAXWActionParaBullet }

constructor TAXWActionParaBullet.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Bullets';
  FCommand := axcFormatParaBullet;
end;

procedure TAXWActionParaBullet.Prepare;
begin
  inherited Prepare;

  AddMenuItem('Default',False);
  AddMenuItem('Disc',False);
  AddMenuItem('Circle',False);
  AddMenuItem('Square',False);
  AddMenuItem('Check Mark',False);
  AddMenuItem('Right Arrow',False);
  AddMenuItem('None',False);
end;

{ TAXWActionParaNumbering }

constructor TAXWActionParaNumbering.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Numbering';
  FCommand := axcFormatParaNumbering;
end;

// anstDecimal,anstDecimalParen,anstLowerRoman,anstUpperRoman,anstLowerLetter,anstLowerLetterParen,anstUpperLetter
procedure TAXWActionParaNumbering.Prepare;
begin
  inherited Prepare;

  AddMenuItem('None',False);
  AddMenuItem('Bullet',False);
  AddMenuItem('1. --------------',False);
  AddMenuItem('1) --------------',False);
  AddMenuItem('i. --------------',False);
  AddMenuItem('I. --------------',False);
  AddMenuItem('a. --------------',False);
  AddMenuItem('a) --------------',False);
  AddMenuItem('A. --------------',False);
end;

{ TAXWActionParaMultilevelList }

constructor TAXWActionParaMultilevelList.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Multilevel List';
  FCommand := axcFormatParaMultilevel;
end;

procedure TAXWActionParaMultilevelList.MenuItemExecuted(ASender: TObject);
begin
  if CanExecute then begin
    case FPop.Items.IndexOf(TMenuItem(ASender)) of
      0: TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcFormatRemoveBulletNumbering);
      1: TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcFormatParaMultilevel,Integer(0));
      2: TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcFormatParaMultilevel,Integer(1));
      4: TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcFormatParaMultilevelLevelInc);
      5: TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcFormatParaMultilevelLevelDec);
    end;
  end;
end;

procedure TAXWActionParaMultilevelList.Prepare;
begin
  inherited Prepare;

  AddMenuItem('None',False);
  AddMenuItem('1.a.1.a ------------',False);
  AddMenuItem('1.1.1.1 ------------',False);
  AddMenuItem('-',False);
  AddMenuItem('Increase level',False);
  AddMenuItem('Decrease level',False);
end;

{ TAXWActionParaIndentDec }

constructor TAXWActionParaIndentDec.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Decrease Indent';
  FCommand := axcFormatParaDecIndentLeft;
end;

{ TAXWActionParaIndentInc }

constructor TAXWActionParaIndentInc.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Increase Indent';
  FCommand := axcFormatParaIncIndentLeft;
end;

{ TAXWActionParaAlignLeft }

constructor TAXWActionParaAlignLeft.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Align Left';
  FCommand := axcFormatParaLeft;
end;

{ TAXWActionParaAlignCenter }

constructor TAXWActionParaAlignCenter.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Center';
  FCommand := axcFormatParaCenter;
end;

{ TAXWActionParaAlignRight }

constructor TAXWActionParaAlignRight.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Align Right';
  FCommand := axcFormatParaRight;
end;

{ TAXWActionParaAlignJustify }

constructor TAXWActionParaAlignJustify.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Justify';
  FCommand := axcFormatParaJustify;
end;

{ TAXWActionParaLineSpacing }

constructor TAXWActionParaLineSpacing.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Line Spacing';
  FCommand := axcFormatParaLineSpacing;
end;

procedure TAXWActionParaLineSpacing.MenuItemExecuted(ASender: TObject);
var
  i: integer;
begin
  if CanExecute then begin
    i := FPop.Items.IndexOf(TMenuItem(ASender));
    if i >= 0 then
      TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(FCommand,AXWALineSpacingValues[i]);
  end;
end;

procedure TAXWActionParaLineSpacing.Prepare;
var
  i: integer;
begin
  inherited Prepare;

  for i := 0 to High(AXWALineSpacingValues) do
    AddMenuItem(FloatToStr(AXWALineSpacingValues[i]),False);
end;

{ TAXWActionParaColor }

constructor TAXWActionParaColorDialog.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Paragraph Color';
  FCommand := axcFormatParaColor;
end;

{ TAXWActionFindReplace }

constructor TAXWActionFind.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Find';
end;

function TAXWActionFind.Execute: boolean;
var
  Frm: TfrmFindReplace;
begin
  Result := CanExecute;

  if Result then begin
    Frm := TfrmFindReplace.Create(Self);
    try
      Frm.Execute(TAXWActionList(ActionList).FAXWWriter.Editor,False);
    finally
      Frm.Free;
    end;
  end;
end;

{ TAXWActionParaColor }

constructor TAXWActionParaColor.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Fill Color';
  FColor := AXW_COLOR_AUTOMATIC;
  FCommand := axcFormatParaColor;
end;

function TAXWActionParaColor.Execute: boolean;
begin
  Result := CanExecute and (FCommand <> axcNone);

  if Result then begin
    SetValue(Caption);

    TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(FCommand,FColor);
  end;
end;

procedure TAXWActionParaColor.Loaded;
begin
  inherited;

  if CanExecute then
    TAXWActionList(ACtionList).AddComponent(Self);
end;

procedure TAXWActionParaColor.SetValue(AValue: AxUCString);
begin
  FColor := StrToIntDef(AValue,0);
end;

{ TAXWActionFindNext }

constructor TAXWActionFindNext.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Find Next';
  FCommand := axcFormatParaColor;
end;

function TAXWActionFindNext.Execute: boolean;
var
  Frm: TfrmFindReplace;
begin
  if TAXWActionList(ActionList).FAXWWriter.Editor.FocusDoc.FindReplace.CurrFind <> '' then
    Result := inherited Execute
  else begin
    Frm := TfrmFindReplace.Create(Self);
    try
      Result := Frm.Execute(TAXWActionList(ActionList).FAXWWriter.Editor,False);
    finally
      Frm.Free;
    end;
  end;
end;

{ TAXWActionReplace }

constructor TAXWActionReplace.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Replace';
end;

function TAXWActionReplace.Execute: boolean;
var
  Frm: TfrmFindReplace;
begin
  Result := CanExecute;

  if Result then begin
    Frm := TfrmFindReplace.Create(Self);
    try
      Frm.Execute(TAXWActionList(ActionList).FAXWWriter.Editor,True);
    finally
      Frm.Free;
    end;
  end;
end;

{ TAXWActionInsertTable }

constructor TAXWActionInsertTable.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Insert Table';
end;

function TAXWActionInsertTable.Execute: boolean;
var
  Frm : TfrmInsertTable;
  Cols: integer;
  Rows: integer;
begin
  Result := CanExecute;

  if Result then begin
    Frm := TfrmInsertTable.Create(Self);
    try
      Cols := 4;
      Rows := 4;

      if Frm.Execute(Cols,Rows) then
        TAXWActionList(ActionList).FAXWWriter.Editor.CommandUI(axcInsertTableAtCaret,Cols,Rows);
    finally
      Frm.Free;
    end;
  end;
end;

{ TAXWActionInsertPicture }

constructor TAXWActionInsertPicture.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Insert Picture';
end;

function TAXWActionInsertPicture.Execute: boolean;
var
  Dlg: TOpenPictureDialog;
begin
  Dlg := TOpenPictureDialog.Create(Self);
  try
    Dlg.Filter := AXWKnownImageFileFormatsFilter;

    Result := Dlg.Execute;

    if Result then
      TAXWActionList(ActionList).FAXWWriter.Editor.CommandUI(axcInsertImageAtCaret,Dlg.FileName);
  finally
    Dlg.Free;
  end;
end;

{ TAXWActionBorders }

constructor TAXWActionBorders.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Borders and Shading';
end;

function TAXWActionBorders.Execute: boolean;
var
  Frm: TfrmAXWBorders;
begin
  Frm := TfrmAXWBorders.Create(Self);
  try
    Result := Frm.Execute(TAXWActionList(ActionList).FAXWWriter.Editor.FocusDoc.Selections);

    if Result then begin
      TAXWActionList(ActionList).FAXWWriter.Editor.Invalidate;
    end;
  finally
    Frm.Free;
  end;
end;

{ TAXWActionParagraphProps }

constructor TAXWActionParagraphProps.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Paragraph';
end;

function TAXWActionParagraphProps.Execute: boolean;
var
  Frm: TfrmAXWParaProps;
begin
  Frm := TfrmAXWParaProps.Create(Self);
  try
    Result := Frm.Execute(TAXWActionList(ActionList).FAXWWriter.Editor.FocusDoc.Selections);

    if Result then begin
      TAXWActionList(ActionList).FAXWWriter.Editor.Invalidate;
    end;
  finally
    Frm.Free;
  end;
end;

{ TAXWActionStyles }

constructor TAXWActionStyles.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Style Manager';
end;

function TAXWActionStyles.Execute: boolean;
var
  Frm: TfrmAXWStyles;
begin
  Frm := TfrmAXWStyles.Create(Self);
  try
    Result := Frm.Execute(TAXWActionList(ActionList).FAXWWriter.Editor.FocusDoc.Styles,TAXWActionList(ActionList).FAXWWriter.Editor.FocusDoc.Selections);

    if Result then begin
      TAXWActionList(ActionList).FAXWWriter.Editor.Invalidate;
    end;
  finally
    Frm.Free;
  end;
end;

{ TAXWActionTabs }

constructor TAXWActionTabs.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Tabs';
end;

function TAXWActionTabs.Execute: boolean;
var
  Frm: TfrmAXWTabs;
begin
  Frm := TfrmAXWTabs.Create(Self);
  try
    Result := Frm.Execute(TAXWActionList(ActionList).FAXWWriter.SysOptions,TAXWActionList(ActionList).FAXWWriter.Editor.DOP,TAXWActionList(ActionList).FAXWWriter.Editor.FocusDoc.Selections);

    if Result then begin
      TAXWActionList(ActionList).FAXWWriter.Editor.Invalidate;
    end;
  finally
    Frm.Free;
  end;
end;

{ TAXWActionHyperlink }

constructor TAXWActionHyperlink.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Hyperlink';
end;

function TAXWActionHyperlink.Execute: boolean;
var
  Frm: TfrmAXWHyperlink;
  Doc: TAXWLogDocEditor;
  Cnt: integer;
  Sel: TAXWLogSelection;
begin
  Frm := TfrmAXWHyperlink.Create(Self);
  try
    Doc := TAXWActionList(ActionList).FAXWWriter.Editor.FocusDoc;

    Cnt := Doc.Selections.Count;

    if Cnt < 1 then begin
      if Doc.CaretPos.Hyperlink <> Nil then begin
        Sel := Doc.Selections.Add;
        Sel.FirstPos.Assign(Doc.CaretPos);
        Sel.SelectHyperlinkAtFirstPos;
      end
      else
        Doc.Selections.SelectWordAtCaret(True);

      TAXWActionList(ActionList).FAXWWriter.Editor.Invalidate;
    end;

    Result := Frm.Execute(Doc);

    if Result then
      Doc.CaretPos.MoveWordEnd;

    if Cnt <= 0 then
      Doc.Selections.Clear;

    TAXWActionList(ActionList).FAXWWriter.Editor.Invalidate;
  finally
    Frm.Free;
  end;
end;

{ TAXWActionBookmarks }

constructor TAXWActionBookmarks.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Bookmarks';
end;

function TAXWActionBookmarks.Execute: boolean;
var
  Frm: TfrmAXWBookmarks;
begin
  Frm := TfrmAXWBookmarks.Create(Self);
  try
    Result := Frm.Execute(TAXWActionList(ActionList).FAXWWriter.Editor.FocusDoc);

    if Result then begin
      TAXWActionList(ActionList).FAXWWriter.Editor.Invalidate;
    end;
  finally
    Frm.Free;
  end;
end;

{ TAXWActionTextBox }

constructor TAXWActionTextBox.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Text box';
  FCommand := axcGraphicInsertTextBox;
end;

{ TAXWActionRevisionTrackChanges }

constructor TAXWActionRevTrackChanges.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Track Changes';
  AutoCheck := True;
  FCommand := axcRevisionTrackChanges;
end;

{ TAXWActionRevisionReviewingPane }

constructor TAXWActionRevReviewingPane.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Reviewing Pane';
end;

function TAXWActionRevReviewingPane.Execute: boolean;
begin
  Result := CanExecute;

  if Result then begin
    if G_FrmRevisionPane = Nil then begin
      G_FrmRevisionPane := TfrmAXWRevisionPane.Create(Self);
      G_FrmRevisionPane.Execute(AXWWriter.SysOptions.IniFile,AXWWriter.Editor.MainDoc);
    end
    else begin
      G_FrmRevisionPane.Close;
    end;
  end;
end;

{ TAXWActionRevAccept }

constructor TAXWActionRevAccept.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Accept';
  FCommand := axcRevisionAccept;
end;

{ TAXWActionRevAcceptAll }

constructor TAXWActionRevAcceptAll.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Accept All';
  FCommand := axcRevisionAcceptAll;
end;

{ TAXWActionRevReject }

constructor TAXWActionRevReject.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Reject';
  FCommand := axcRevisionReject;
end;

{ TAXWActionRevRejectAll }

constructor TAXWActionRevRejectAll.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Reject All';
  FCommand := axcRevisionRejectAll;
end;

{ TAXWActionRevShowMarkup }

constructor TAXWActionRevShowMarkup.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Show Markup';
end;

function TAXWActionRevShowMarkup.Execute: boolean;
begin
  Result := inherited Execute;
end;

procedure TAXWActionRevShowMarkup.MenuItemExecuted(ASender: TObject);
var
  i   : integer;
  S   : AxUCString;
  Item: TMenuItem;
  Doc : TAXWLogDocEditor;
begin
  Doc := TAXWActionList(ActionList).FAXWWriter.Editor.MainDoc;

  Item := TMenuItem(ASender);

  S := Item.Caption;

  repeat
    i := CPos('&',S);
    if i >= 1 then
      System.Delete(S,i,1);
  until i < 1;

  Doc.Revision.EnableAuthor(S,Item.Checked);
//  Doc.Revision.EnableAuthor('John Doe',Item.Checked);

  TAXWActionList(ActionList).FAXWWriter.Invalidate;
end;

procedure TAXWActionRevShowMarkup.Prepare;
var
  i  : integer;
  Doc: TAXWLogDocEditor;
begin
  inherited Prepare;

  FAutoCheck := True;

  Doc := TAXWActionList(ActionList).FAXWWriter.Editor.MainDoc;

  Doc.Revision.Collect;

  if Doc.Revision.Authors.Count > 0 then begin
    for i := 0 to Doc.Revision.Authors.Count - 1 do
      AddMenuItem(Doc.Revision.Authors[i].Name,Doc.Revision.Authors[i].Visible);
  end;
end;

{ TAXWActionRevNext }

constructor TAXWActionRevNext.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Next';
  FCommand := axcSelectRevisionNext;
end;

{ TAXWActionRevPrevious }

constructor TAXWActionRevPrevious.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Previous';
  FCommand := axcSelectRevisionPrev;
end;

{ TAXWActionShowDocStructure }

constructor TAXWActionShowDocStructure.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Document Structure';
end;

function TAXWActionShowDocStructure.Execute: boolean;
begin
  if CanExecute then begin
    TfrmDocStruct.Create(Self).Execute(TAXWActionList(ActionList).FAXWWriter.Editor.MainDoc);

    Result := True;
  end
  else
    Result := False;
end;

{ TAXWActionPrint }

constructor TAXWActionPrint.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Print';
end;

function TAXWActionPrint.Execute: boolean;
var
  AXWPrint: TAXWPrintVcl;
begin
  if CanExecute then begin
    AXWPrint := TAXWPrintVcl.Create(Nil);
    try
      AXWPrint.Document := TAXWActionList(ActionList).FAXWWriter.Editor.MainDoc;
      AXWPrint.Print;

      Result := True;
    finally
      AXWPrint.Free;
    end;
  end
  else
    Result := False;
end;

{ TAXWActionPrintPreview }

constructor TAXWActionPrintPreview.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Print Preview';
end;

function TAXWActionPrintPreview.Execute: boolean;
begin
  if CanExecute then begin
    TFrmPreview.Create(Self).Execute(TAXWActionList(ActionList).FAXWWriter.Editor.MainDoc);

    Result := True;
  end
  else
    Result := False;
end;

{ TAXWActionOptions }

constructor TAXWActionOptions.Create(AOwner: TComponent);
begin
  inherited;

  CAption := 'Options';
end;

function TAXWActionOptions.Execute: boolean;
begin
  Result := CanExecute;
  if Result then begin
    Result := TfrmAXWOptions.Create(Self).Execute(TAXWActionList(ActionList).FAXWWriter.SysOptions);
    if Result then begin
      TAXWActionList(ActionList).FAXWWriter.SysOptions.Save;

      TAXWActionList(ActionList).FAXWWriter.UpdateOptions;

      TAXWActionList(ActionList).FAXWWriter.Invalidate;
    end;
  end;
end;

{ TAXWActionEncrypt }

constructor TAXWActionEncrypt.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Encrypt Document';
end;

function TAXWActionEncrypt.Execute: boolean;
var
  S: string;
  L: TStringList;
begin
  if CanExecute then begin
    S := TAXWActionList(ActionList).FAXWWriter.Password;

    L := TStringList.Create;
    try
      // Most common passwords 2016
      L.Add('123456');
      L.Add('password');
      L.Add('12345');
      L.Add('12345678');
      L.Add('football');
      L.Add('qwerty');
      L.Add('1234567890');
      L.Add('1234567');
      L.Add('princess');

      Result := TfrmTextInput.Create(Self).Execute('Encrypt','Password for Encryption',S,L);

      if Result then
        TAXWActionList(ActionList).FAXWWriter.Password := S;
    finally
      L.Free;
    end;
  end
  else
    Result := False;
end;

{ TAXWActionSpellCheckAll }

constructor TAXWActionSpellCheckAll.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Spell Check All';
  FCommand := axcControlSpellSheckAll;
end;

{ TAXWActionSpellCheck }

constructor TAXWActionSpellCheck.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Spell Check';
end;

function TAXWActionSpellCheck.Execute: boolean;
var
  S   : string;
  Edit: TAXWEditor;
  Res : boolean;
begin
  Result := CanExecute;

  if not Result then
    Exit;

  Edit := TAXWActionList(ActionList).FAXWWriter.Editor;

  Edit.SpellChecker.Checker.Initiate(TAXWActionList(ActionList).FAXWWriter.SysOptions.SpellSelectedLanguage);

  if Edit.SpellChecker.Checker.Ready then begin
    if (Edit.FocusDoc.Selections.Count < 1) or not Edit.FocusDoc.Selections.Last.Valid then begin
      Edit.FocusDoc.Selections.Clear;
      Edit.FocusDoc.Selections.AddCaret;
    end;
    Res := True;
    while Res and Edit.FocusDoc.Selections.Last.MoveWordNext do begin
      if not Edit.SpellChecker.Check(Edit.FocusDoc.Selections.Last.PlainText) then begin
//        Edit.FocusDoc.CaretPos.Assign(Edit.FocusDoc.Selections.Last.FirstPos);
        Edit.MainDoc.Repaint := True;
        Edit.UpdateUI;

        case TfrmSpellCheck.Create(Self).Execute(Edit.FocusDoc.Selections.Last.PlainText,S,Edit.SpellChecker.Suggestions) of
          mrOk: begin
            Edit.FocusDoc.Selections.Last.PlainText := S;

            Edit.MainDoc.Repaint := True;
            Edit.UpdateUI;
          end;
          mrCancel: Break;
        end;
      end;
    end;
    Edit.FocusDoc.Selections.Clear;
    Edit.MainDoc.Repaint := True;
    Edit.UpdateUI;
  end
  else
    raise Exception.Create('Spell checker not ready. Please check dictionaies');
end;

{ TAXWActionSpellCheckClear }

constructor TAXWActionSpellCheckClear.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Clear Spell Check';
  FCommand := axcControlSpellSheckClear;
end;

{ TAXWActionPageSetup }

constructor TAXWActionPageSetup.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Page Setup';
end;

function TAXWActionPageSetup.Execute: boolean;
begin
  Result := CanExecute;

  if Result then
    Result := TfrmAXWPageSetup.Create(Self).Exceute(TAXWActionList(ActionList).FAXWWriter.SysOptions,TAXWActionList(ActionList).FAXWWriter.Editor.MainDoc.SEP);
end;

{ TAXWActionPageMargins }

constructor TAXWActionPageMargins.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Margins';
end;

procedure TAXWActionPageMargins.MenuItemExecuted(ASender: TObject);
var
  L,T,R,B: double;
begin
  if CanExecute then begin
    L := 72;
    T := 72;
    R := 72;
    B := 72;
    case FPop.Items.IndexOf(TMenuItem(ASender)) of
      0: ;
      1: begin
        L := 36;
        T := 36;
        R := 36;
        B := 36;
      end;
      2: begin
        L := 54;
        R := 54;
      end;
      3: begin
        L := 144;
        R := 144;
      end;
    end;

    TAXWActionList(ActionList).AXWWriter.Editor.Command(axcSettingPageMargLeft,L);
    TAXWActionList(ActionList).AXWWriter.Editor.Command(axcSettingPageMargTop,T);
    TAXWActionList(ActionList).AXWWriter.Editor.Command(axcSettingPageMargRight,R);
    TAXWActionList(ActionList).AXWWriter.Editor.Command(axcSettingPageMargBottom,B);

    TAXWActionList(ActionList).AXWWriter.SetupRuler;
    TAXWActionList(ActionList).AXWWriter.Invalidate;
  end;
end;

procedure TAXWActionPageMargins.Prepare;
begin
  inherited Prepare;

  AddMenuItem('Normal',False);
  AddMenuItem('Narrow',False);
  AddMenuItem('Moderate',False);
  AddMenuItem('Wide',False);
end;

{ TAXWActionPageOrientation }

constructor TAXWActionPageOrientation.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Orientation';
end;

procedure TAXWActionPageOrientation.MenuItemExecuted(ASender: TObject);
begin
  if CanExecute then begin
    case FPop.Items.IndexOf(TMenuItem(ASender)) of
      0: TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcSettingPageLandscape,False);
      1: TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcSettingPageLandscape,True);
    end;

    TAXWActionList(ActionList).AXWWriter.Invalidate;
  end;
end;

procedure TAXWActionPageOrientation.Prepare;
begin
  inherited Prepare;

  AddMenuItem('Portrait',False);
  AddMenuItem('Landscape',False);
end;

{ TAXWActionPageSize }

constructor TAXWActionPageSize.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Size';
end;

procedure TAXWActionPageSize.MenuItemExecuted(ASender: TObject);
var
  i  : TAXWPaperSize;
  PSD: TAXWPaperSizeData;
begin
  if CanExecute then begin
    i := TAXWPaperSize(FPop.Items.IndexOf(TMenuItem(ASender)) + 1);
    PSD := AXWDefPaperSizes[i];

    TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcSettingPaperWidth,PSD.Width);
    TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcSettingPaperHeight,PSD.Height);

    TAXWActionList(ActionList).AXWWriter.SetupRuler;
    TAXWActionList(ActionList).AXWWriter.Invalidate;
  end;
end;

procedure TAXWActionPageSize.Prepare;
var
  i  : TAXWPaperSize;
  PSD: TAXWPaperSizeData;
begin
  inherited Prepare;

  for i := Succ(Low(TAXWPaperSize)) to High(TAXWPaperSize) do begin
    PSD := AXWDefPaperSizes[i];

    AddMenuItem(PSD.Name + ' ' + TAXWActionList(ActionList).AXWWriter.SysOptions.ValToUnitStr(PSD.Width) + ' x ' + TAXWActionList(ActionList).AXWWriter.SysOptions.ValToUnitStr(PSD.Height),False);
  end;
end;

{ TAXWActionLayoutPrint }

constructor TAXWActionLayoutPrint.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Print Layout';

  FCommand := axcControlLayoutPrint;
end;

{ TAXWActionLayoutWeb }

constructor TAXWActionLayoutWeb.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Web Layout';

  FCommand := axcControlLayoutWeb;
end;

{ TAXWActionZoom }

constructor TAXWActionZoom.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Zoom';
end;

procedure TAXWActionZoom.MenuItemExecuted(ASender: TObject);
var
  Z: double;
begin
  if CanExecute then begin
    Z := 1.00;

    case FPop.Items.IndexOf(TMenuItem(ASender)) of
      0: Z := 0.25;
      1: Z := 0.50;
      2: Z := 0.75;
      3: Z := 1.00;
      4: Z := 1.50;
      5: Z := 2.00;
    end;

    TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcControlZoom,Z);
  end;
end;

procedure TAXWActionZoom.Prepare;
begin
  inherited Prepare;

  AddMenuItem('25%');
  AddMenuItem('50%');
  AddMenuItem('75%');
  AddMenuItem('100%');
  AddMenuItem('150%');
  AddMenuItem('200%');
end;

{ TAXWActionZoom100 }

constructor TAXWActionZoom100.Create(AOwner: TComponent);
begin
  inherited;

  Caption := '100%';
end;

function TAXWActionZoom100.Execute: boolean;
begin
  Result := CanExecute;

  if Result then
    TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcControlZoom,1.00);
end;

{ TAXWActionShowMargins }

constructor TAXWActionShowMargins.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Margins';

  AutoCheck := True;
end;

function TAXWActionShowMargins.Execute: boolean;
begin
  inherited Execute;

  Result := CanExecute;

  if Result then
    TAXWActionList(ActionList).AXWWriter.Editor.ShowMargins := Checked;
end;

{ TAXWActionShowRuler }

constructor TAXWActionShowRuler.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Ruler';

  AutoCheck := True;
end;

function TAXWActionShowRuler.Execute: boolean;
begin
  inherited Execute;

  Result := CanExecute;

  if Result then
    TAXWActionList(ActionList).AXWWriter.RulersVisible := Checked;
end;

procedure TAXWActionShowRuler.Loaded;
begin
  inherited;

  if CanExecute then
    Checked := TAXWActionList(ActionList).AXWWriter.RulersVisible;
end;

{ TAXWActionMacroRecord }

constructor TAXWActionMacroRecord.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Record';

  FCommand := axcControlMacroRecord;
end;

{ TAXWActionMacroPlay }

constructor TAXWActionMacroPlay.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Play';

  FCommand := axcControlMacroPlay;
end;

{ TAXWActionFontOptions }

constructor TAXWActionFontProps.Create(AOwner: TComponent);
begin
  inherited;

  Caption := 'Font';
end;

function TAXWActionFontProps.Execute: boolean;
var
  Frm: TfrmSelectFont;
begin
  Result := CanExecute;

  if Result then begin
    Frm := TfrmSelectFont.Create(Self);
    try
      TAXWActionList(ActionList).AXWWriter.Editor.MainDoc.TempCHPX.Clear;

      if Frm.Execute(TAXWActionList(ActionList).AXWWriter.Editor.MainDoc.TempCHPX) then
        TAXWActionList(ActionList).AXWWriter.Editor.CommandUI(axcFormatCHPX);
    finally
      Frm.Free;
    end;
  end;
end;

end.
