unit AXWColorPickerVcl;

interface

{$B-}
{$H+}
{$R-}
{$I AxCompilers.inc}

uses
  SysUtils, Classes, Controls, Graphics, Messages, Windows,
{$ifdef DELPHI_XE2_OR_LATER}
  UITypes,
{$endif}
  Math, Dialogs;

const XSS_COMBO_BTN_WIDTH = 18;

type TXc12RGBColor = longword;

type TXc12ColorType = (exctAuto,exctIndexed,exctRgb,exctTheme,exctUnassigned);

type TXc12IndexColor = (
     xc0,xc1,xc2,xc3,xc4,xc5,xc6,xc7,
     xcBlack,xcWhite,xcRed,xcBrightGreen,xcBlue,xcYellow,xcPink,xcTurquoise,
     xcDarkRed,xcGreen,xcDarkBlue,xcBrownGreen,xcViolet,xcBlueGreen,xcGray25,xcGray50,
     xc24,xc25,xc26,xc27,xc28,xc29,xc30,xc31,
     xc32,xc33,xc34,xc35,xc36,xc37,xc38,xc39,
     xcSky,xcPaleTurquois,xcPaleGreen,xcLightYellow,xcPaleSky,xcRose,xcLilac,xcLightBrown,
     xcDarkSky,xcDarkTurquois,xcGrass,xcGold,xcLightOrange,xcOrange,xcDarkBlueGray,xcGray40,
     xcDarkGreenGray,xcEmerald,xcDarkGreen,xcOlive,xcBrown,xcCherry,xcIndigo,xcGray80,
     xcAutomatic);

type TXc12ClrSchemeColor = (cscDk1,cscLt1,cscLt2,cscDk2,cscAccent1,cscAccent2,
                            cscAccent3,cscAccent4,cscAccent5,cscAccent6,
                            cscHLink,cscFolHLink);

type PXc12Color = ^TXc12Color;
     TXc12Color = record
     ARGB: TXc12RGBColor;
     Tint: double;
     case ColorType: TXc12ColorType of
       exctAuto:    (Auto: boolean);
       exctIndexed: (Indexed: TXc12IndexColor);
       exctRgb:     (OrigRGB: TXc12RGBColor);
       exctTheme:   (Theme: TXc12ClrSchemeColor);
     end;

type TColorSwatch = class(TObject)
public
     Col,Row: integer;
     Color: TXc12Color;
     X,Y: integer;
     end;

const SWATCH_BORDERCOLOR = $00C5C5C5;
const SWATCH_MOUSECOLOR1 = $003694F2;
const SWATCH_MOUSECOLOR2 = $0094E2FF;
const SWATCH_SELECTEDCOLOR = $001048EF;

const TXc12DefaultIndexColorPaletteRGB: array[0..65] of integer = (
$000000, $FFFFFF, $FF0000, $00FF00, $0000FF, $FFFF00, $FF00FF, $00FFFF,
$000000, $FFFFFF, $FF0000, $00FF00, $0000FF, $FFFF00, $FF00FF, $00FFFF,
$800000, $008000, $000080, $808000, $800080, $008080, $C0C0C0, $808080,
$9999FF, $993366, $FFFFCC, $CCFFFF, $660066, $FF8080, $0066CC, $CCCCFF,
$000080, $FF00FF, $FFFF00, $00FFFF, $800080, $800000, $008080, $0000FF,
$00CCFF, $CCFFFF, $CCFFCC, $FFFF99, $99CCFF, $FF99CC, $CC99FF, $FFCC99,
$3366FF, $33CCCC, $99CC00, $FFCC00, $FF9900, $FF6600, $666699, $969696,
$003366, $339966, $003300, $333300, $993300, $993366, $333399, $333333,
$0F000000,$0F000000);

const TXc12DefaultIndexColorPalette: array[0..65] of integer = (
$000000, $FFFFFF, $0000FF, $00FF00, $FF0000, $00FFFF, $FF00FF, $FFFF00,
$000000, $FFFFFF, $0000FF, $00FF00, $FF0000, $00FFFF, $FF00FF, $FFFF00,
$000080, $008000, $800000, $008080, $800080, $808000, $C0C0C0, $808080,
$FF9999, $663399, $CCFFFF, $FFFFCC, $660066, $8080FF, $CC6600, $FFCCCC,
$800000, $FF00FF, $00FFFF, $FFFF00, $800080, $000080, $808000, $FF0000,
$FFCC00, $FFFFCC, $CCFFCC, $99FFFF, $FFCC99, $CC99FF, $FF99CC, $99CCFF,
$FF6633, $CCCC33, $00CC99, $00CCFF, $0099FF, $0066FF, $996666, $969696,
$663300, $669933, $003300, $003333, $003399, $663399, $993333, $333333,
$0F000000,$0F000000);

//const Xc12DefColorSchemeValsRGB: array[TXc12ClrSchemeColor] of longword = (
//      $FFFFFF,$000000,$EEECE1,$1F497D,$4F81BD,$C0504D,$9BBB59,
//      $8064A2,$4BACC6,$F79646,$0000FF,$800080);

const Xc12DefColorSchemeValsRGB: array[TXc12ClrSchemeColor] of longword = (
      $FFFFFF,$000000,$44546A,$E7E6E6,$4472C4,$ED7D31,$A5A5A5,
      $FFC000,$5B9BD5,$70AD47,$0563C1,$954F72);

type TAXWMultiColorPicker = class;

     TColorSwatches = class(TObject)
private
     procedure SetCols(const Value: integer);
     procedure SetRows(const Value: integer);
     procedure SetSwatchSize(const Value: integer);
     procedure SetSwatchXSpacing(const Value: integer);
     procedure SetSwatchYSpacing(const Value: integer);
     procedure SetY1(const Value: integer);
     procedure SetX1(const Value: integer);
     function  GetSwatches(Col, Row: integer): TColorSwatch;
protected
     FX1,FY1,FX2,FY2: integer;
     FHorizMarg,FVertMarg: integer;
     FCols,FRows: integer;
     FSwatchSize: integer;
     FSwatchXSpacing: integer;
     FSwatchYSpacing: integer;
     FSwatches: array of array of TColorSwatch;
     FCurrSwatch: TColorSwatch;
     FSelectedSwatch: TColorSwatch;
     FTempSwatch: TColorSwatch;

     procedure Clear;
     procedure InitSwatches; virtual;
     procedure PaintSwatch(Canvas: TCanvas; Swatch: TColorSwatch);
     procedure PaintMouseSwatch(Canvas: TCanvas; Swatch: TColorSwatch);
     function  Hit(X,Y: integer): boolean;
public
     constructor Create(Cols,Rows: integer);
     destructor Destroy; override;

     procedure Paint(Canvas: TCanvas);
     procedure MouseMove(Canvas: TCanvas; X,Y: integer);
     procedure MouseUp(Canvas: TCanvas; X,Y: integer);
     procedure ClearMouse(Canvas: TCanvas);
     function  ClientHit(X,Y: integer): boolean;
     procedure ClearSelected(Canvas: TCanvas);
     function  SetSelected(Canvas: TCanvas; Color: TXc12Color): boolean;

     property X1: integer read FX1 write SetX1;
     property Y1: integer read FY1 write SetY1;
     property X2: integer read FX2;
     property Y2: integer read FY2;
     property Cols: integer read FCols write SetCols;
     property Rows: integer read FRows write SetRows;
     property SwatchSize: integer read FSwatchSize write SetSwatchSize;
     property SwatchXSpacing: integer read FSwatchXSpacing write SetSwatchXSpacing;
     property SwatchYSpacing: integer read FSwatchYSpacing write SetSwatchYSpacing;
     property Swatches[Col,Row: integer]: TColorSwatch read GetSwatches; default;
     end;

     TSwatchContainer = class(TObject)
private
     procedure SetX1(const Value: integer);
     procedure SetY1(const Value: integer);
protected
     FX1,FY1,FX2,FY2: integer;
     FSwatchList: array of TColorSwatches;
     FSelectedColor: TXc12Color;

     procedure Initiate; virtual;
     procedure _ClearSelected;
public
     constructor Create;
     destructor Destroy; override;

     procedure Paint(Canvas: TCanvas);
     procedure MouseMove(Canvas: TCanvas; X,Y: integer);
     procedure MouseUp(Canvas: TCanvas; X,Y: integer);
     procedure ClearMouse(Canvas: TCanvas);
     procedure ClearSelected(Canvas: TCanvas);
     function  ClientHit(X,Y: integer): boolean;
     function  SwatchHit: boolean;
     function  SetSelected(Canvas: TCanvas; Color: TXc12Color): boolean;

     property X1: integer read FX1 write SetX1;
     property Y1: integer read FY1 write SetY1;
     property X2: integer read FX2;
     property Y2: integer read FY2;
     end;

     TEx97Swatches = class(TSwatchContainer)
private
protected
     procedure Initiate; override;
public
     constructor Create;
     end;

     TEx2007ThemeSwatches = class(TSwatchContainer)
protected
     FAdjust: array[0..9,0..4] of double;

     procedure Initiate; override;
public
     constructor Create(Compact: boolean);
     end;

     TEx2007StandardSwatches = class(TSwatchContainer)
private
protected
     procedure Initiate; override;
public
     constructor Create;
     end;

     TExcelColorPickerMode = (ecpmNone,ecpmExcel2007ThemeCompact,ecpmExcel2007Theme,ecpmExcel2007Standard,ecpmExcel97);

     TAXWColorPicker = class(TGraphicControl)
private
     procedure SetColorMode(const Value: TExcelColorPickerMode);
     function  GetExcelColor: TXc12Color;
     procedure SetExcelColor(const Value: TXc12Color);
     procedure SetLinkedPicker(const Value: TAXWColorPicker);
protected
     FColorMode    : TExcelColorPickerMode;
     FColorSwatches: TSwatchContainer;
     FClickEvent   : TNotifyEvent;
     FLinkedPicker : TAXWColorPicker;
     FParentPicker : TAXWColorPicker;

     procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
     procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
     procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
     procedure WMMouseLeave(var Message: TMessage); message WM_MOUSELEAVE;
public
     constructor Create(AOwner: TComponent); override;

     destructor Destroy; override;
     procedure Paint; override;
     procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;

     function  FindAndSelect(AColor: TXc12Color): boolean;

     property ExcelColor: TXc12Color read GetExcelColor write SetExcelColor;
published
     property ColorMode: TExcelColorPickerMode read FColorMode write SetColorMode;
     property LinkedPicker: TAXWColorPicker read FLinkedPicker write SetLinkedPicker;

     property OnClick: TNotifyEvent read FClickEvent write FClickEvent;
     end;

     TAXWMultiColorPicker = class(TCustomControl)
private
     procedure SetColor(const Value: TXc12Color);
     function  GetCompact: boolean;
     procedure SetCompact(const Value: boolean);
protected
     FCPTheme    : TAXWColorPicker;
     FCPStandard : TAXWColorPicker;
     FTitleHeight: integer;
     FSelColor   : TXc12Color;
     FSelectEvent: TNotifyEvent;

     procedure ThemeColorClicked(ASender: TObject);
     procedure StandardColorClicked(ASender: TObject);
public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;

     procedure Paint; override;

     procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;

     property Color: TXc12Color write SetColor;
     property SelectedColor: TXc12Color read FSelColor;
published
     property Compact : boolean read GetCompact write SetCompact;
     property OnSelect: TNotifyEvent read FSelectEvent write FSelectEvent;
     end;

function Xc12ColorToRGB(const Color: TXc12Color; AAutoColor: TXc12RGBColor = $00000000): longword; overload;
function Xc12ColorToRGB(const Color: PXc12Color; AAutoColor: TXc12RGBColor = $00000000): longword; overload;
function RGBColorToXc12(const ARGB: TXc12RGBColor): TXc12Color; overload;

implementation

function MakeXc12ColorAuto(const ADefault: longword = $000000): TXc12Color;
begin
  Result.ColorType := exctAuto;
  Result.Auto := True;
  Result.ARGB := ADefault;
end;

type TRGBRec = packed record
    case Integer of
      0: (Value: longword);
      1: (Red, Green, Blue: Byte);
      2: (R, G, B, Flag: Byte);
      3: (Index: Word);
  end;

type THLSRec = record
     H,L,S: byte;
     end;

const Ex12StandardColors: array[0..9] of longword =
($00C00000,$00FF0000,$00FFC000,$00FFFF00,$0092D050,$0000B050,$0000B0F0,$000070C0,$00002060,$007030A0);

var Xc12DefColorSchemeRGB: array[TXc12ClrSchemeColor] of longword;

var Xc12IndexColorPalette: array[0..High(TXc12DefaultIndexColorPalette)] of integer;

var Xc12IndexColorPaletteRGB: array[0..High(TXc12DefaultIndexColorPalette)] of integer;

function RevRGB(RGB: longword): longword; {$ifdef D2006PLUS} inline; {$endif}
begin
  Result := ((RGB and $FF0000) shr 16) + (RGB and $00FF00) + ((RGB and $0000FF) shl 16);
end;

function Xc12ColorEqual(const C1,C2: TXc12Color): boolean;
begin
  Result := C1.ColorType = C2.ColorType;
  if Result then begin
    case C1.ColorType of
      exctAuto:       Result := True;
      exctIndexed:    Result := C1.Indexed = C2.Indexed;
      exctRgb:        Result := C1.OrigRGB = C2.OrigRGB;
      exctTheme:      Result := (C1.Theme = C2.Theme) and SameValue(C1.Tint,C2.Tint,0.0001);
      exctUnassigned: Result := False;
    end;
  end;
end;

function Max3(P1,P2,P3: byte): byte;
begin
  if (P1 > P2) then begin
    if (P1 > P3) then begin
      Result := P1;
    end else begin
      Result := P3;
    end;
  end else if P2 > P3 then begin
    result := P2;
  end else result := P3;
end;

function Min3(P1,P2,P3: byte): byte;
begin
  if (P1 < P2) then begin
    if (P1 < P3) then begin
      Result := P1;
    end else begin
      Result := P3;
    end;
  end else if P2 < P3 then begin
    result := P2;
  end else result := P3;
end;

function Lighten(Lum: integer; Value: double): integer;
begin
// Lum = Lum * (1.0-tint) + (HLSMAX  HLSMAX * (1.0-tint))
  Result := Round(Lum * (1.0 - Value) + (255 - 255 * (1.0 - Value)));
end;

function Darken(Lum: integer; Value: double): integer;
begin
// Lum = Lum * (1.0 + tint)
  Result := Round(Lum * (1.0 + Value));
end;

function RGBToHLS(const PRGB: TRGBRec): THLSRec;
Var
  LR,LG,LB,LMin,LMax,LDif : byte;
  LH,LL,LS,LSum : integer;
begin
  LR := PRGB.R;
  LG := PRGB.G;
  LB := PRGB.B;
  LMin := min3(LR,LG,LB);
  LMax := max3(LR,LG,LB);
  LDif := LMax - LMin;
  LSum := LMax + LMin;
  LL := LSum shr 1;
  if LMin = LMax then begin
    LH := 0;
    LS := 0;
    Result.H := LH;
    Result.L := LL;
    Result.S := LS;
    exit;
  end;
  If LL < 128 then LS := LDif shl 8 div LSum
              else LS := LDif shl 8 div (512 - LSum);
  if LS > 255 then LS := 255;
  If LR = LMax then LH := (LG - LB) shl 8 div LDif
  else If LG = LMax then LH := 512 + (LB - LR) shl 8 div LDif
  else LH := 1024 + (LR - LG) shl 8 div LDif;
  Result.H := LH div 6;
  Result.L := LL;
  Result.S := LS;
end;

function HLSToRGB(const PHLS: THLSRec): TRGBRec;
Var
  LH,LL,LS : byte;
  LR,LG,LB,L1,L2,LDif,L6Dif : integer;
begin
  LH := PHLS.H;
  LL := PHLS.L;
  LS := PHLS.S;
  if LS = 0 then begin
    Result.R := LL;
    Result.G := LL;
    Result.B := LL;
    Exit;
  end;
  If LL < 128 then L2 := LL * (256 + LS) shr 8
              else L2 := LL + LS - LL * LS shr 8;
  L1 := LL shl 1 - L2;
  LDif := L2 - L1;
  L6Dif := LDif * 6;
  LR := LH + 85;
  if LR < 0 then Inc(LR, 256);
  if LR > 256 then Dec(LR, 256);
  If LR < 43 then LR := L1 + L6Dif * LR shr 8
  Else if LR < 128 then LR := L2
  Else if LR < 171 then LR := L1 + L6Dif * (170 - LR) shr 8
  Else LR := L1;
  if LR > 255 then LR := 255;
  LG := LH;
  if LG < 0 then Inc(LG, 256);
  if LG > 256 then Dec(LG, 256);
  If LG < 43 then LG := L1 + L6Dif * LG shr 8
  Else if LG < 128 then LG := L2
  Else if LG < 171 then LG := L1 + L6Dif * (170 - LG) shr 8
  Else LG := L1;
  if LG > 255 then LG := 255;
  LB := LH - 85;
  if LB < 0 then Inc(LB, 256);
  if LB > 256 then Dec(LB, 256);
  If LB < 43 then LB := L1 + L6Dif * LB shr 8
  Else if LB < 128 then LB := L2
  Else if LB < 171 then LB := L1 + L6Dif * (170 - LB) shr 8
  Else LB := L1;
  if LB > 255 then LB := 255;
  Result.R := LR;
  Result.G := LG;
  Result.B := LB;
end;

function Xc12ColorToRGB(const Color: PXc12Color; AAutoColor: TXc12RGBColor = $00000000): longword; overload;
var
  RGB: TRGBRec;
  HLS: THLSRec;
//  RRec: TRGBRec;
//  T: Byte;
begin
  case Color.ColorType of
    exctAuto   : Result := AAutoColor;
    exctIndexed: begin
      if Integer(Color.Indexed) > High(Xc12IndexColorPalette) then
        Result := AAutoColor
      else
        Result := Xc12IndexColorPalette[Integer(Color.Indexed)];
    end;
    exctRgb    : begin
      Result := Color.OrigRGB and $00FFFFFF;
// Don't reverse colors. This function shall return RGB.
//      RRec.Value := Result;
//      T := RRec.R;
//      RRec.R := RRec.B;
//      RRec.B := T;
//      Result := RRec.Value;
    end;
    exctTheme  : begin
      if Color.Tint = 0 then
        Result := Xc12DefColorSchemeRGB[Color.Theme]
      else begin
        RGB.Value := Xc12DefColorSchemeRGB[Color.Theme];
        HLS := RGBToHLS(RGB);
        if Color.Tint > 0 then
          HLS.L := Lighten(HLS.L,Color.Tint)
        else if Color.Tint < 0 then
          HLS.L := Darken(HLS.L,Color.Tint);
        RGB := HLSToRGB(HLS);
        Result := RGB.Value;
      end;
    end
    else       Result := $000000;
  end;
end;

function Xc12ColorToRGB(const Color: TXc12Color; AAutoColor: TXc12RGBColor = $00000000): longword; overload;
begin
  Result := Xc12ColorToRGB(@Color,AAutoColor);
end;

function RGBColorToXc12(const ARGB: longword): TXc12Color;
begin
  Result.ColorType := exctRgb;
  Result.OrigRGB := ARGB;
  Result.ARGB := ARGB;
  Result.Tint := 0;
end;

{ TExcelColorPicker }

constructor TAXWColorPicker.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  Color := clWhite;

  SetColorMode(ecpmExcel2007Theme);

  Width := 100;
  Height := 100;

//  OnMouseLeave := MouseLeave;
end;

destructor TAXWColorPicker.Destroy;
begin
  FColorSwatches.Free;
  inherited;
end;

function TAXWColorPicker.FindAndSelect(AColor: TXc12Color): boolean;
var
  ECP: TAXWColorPicker;
begin
  ECP := Self;

  while ECP <> Nil do begin
    Result := FColorSwatches.SetSelected(Canvas,AColor);
    if Result then
      Exit;
    ECP := ECP.LinkedPicker;
  end;
  Result := False;
end;

function TAXWColorPicker.GetExcelColor: TXc12Color;
begin
  Result := FColorSwatches.FSelectedColor;
end;

procedure TAXWColorPicker.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;

//  SetFocus;
end;

procedure TAXWColorPicker.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited;

   if FParentPicker <> Nil then
     FParentPicker.FColorSwatches.ClearMouse(FParentPicker.Canvas);

   if FColorSwatches.ClientHit(X,Y) then
     FColorSwatches.MouseMove(Canvas,X,Y)
   else
     FColorSwatches.ClearMouse(Canvas);
end;

procedure TAXWColorPicker.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;

  if FColorSwatches.ClientHit(X,Y) and FColorSwatches.SwatchHit then begin
    FColorSwatches.MouseUp(Canvas,X,Y);
    if Assigned(FLinkedPicker) then begin
      FLinkedPicker.FColorSwatches._ClearSelected;
      FLinkedPicker.FColorSwatches.FSelectedColor := FColorSwatches.FSelectedColor;
      FLinkedPicker.Invalidate;
      if (FParentPicker <> NIl) and Assigned(FParentPicker.FClickEvent) then
        FParentPicker.FClickEvent(Self)
      else if Assigned(FClickEvent) then
        FClickEvent(Self);
    end
    else if Assigned(FClickEvent) then
      FClickEvent(Self);
  end;
end;

procedure TAXWColorPicker.Paint;
begin
  inherited;

  Canvas.Brush.Color := Color;
  Canvas.Pen.Color := Canvas.Brush.Color;
  Canvas.Rectangle(0,0,Width,Height);

  FColorSwatches.Paint(Canvas);
end;

procedure TAXWColorPicker.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  if (AWidth <= 0) or (AHeight <= 0) then begin
    inherited SetBounds(ALeft, ATop, AWidth, AHeight);

    Exit;
  end;

  AWidth := FColorSwatches.X2 - FColorSwatches.X1;
  AHeight := FColorSwatches.Y2 - FColorSwatches.Y1;
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TAXWColorPicker.SetColorMode(const Value: TExcelColorPickerMode);
begin
  if (Value <> ecpmNone) and (Value <> FColorMode) then begin
    FColorMode := Value;

    if FColorSwatches <> Nil then
      FColorSwatches.Free;

    case FColorMode of
      ecpmExcel2007Theme:
        FColorSwatches := TEx2007ThemeSwatches.Create(False);
      ecpmExcel2007ThemeCompact:
        FColorSwatches := TEx2007ThemeSwatches.Create(True);
      ecpmExcel2007Standard:
        FColorSwatches := TEx2007StandardSwatches.Create;
      ecpmExcel97:
        FColorSwatches := TEx97Swatches.Create;
    end;
    if Parent <> Nil then begin
      Parent.Invalidate;
      UpdateBoundsRect(Rect(Left,Top,Left + (FColorSwatches.X2 - FColorSwatches.X1),Top + (FColorSwatches.Y2 - FColorSwatches.Y1))); ;
      if Canvas.HandleAllocated then
        Paint;
    end;
  end;
end;

procedure TAXWColorPicker.SetExcelColor(const Value: TXc12Color);
begin
  case Value.ColorType of
    exctAuto      : begin
      SetColorMode(ecpmExcel2007Theme);
      FColorSwatches.ClearSelected(Canvas);
      if FLinkedPicker <> Nil then
        FLinkedPicker.FColorSwatches.ClearSelected(FLinkedPicker.Canvas);
    end;
    exctIndexed   : SetColorMode(ecpmExcel97);
    exctRgb       : SetColorMode(ecpmExcel2007Theme);
    exctTheme     : SetColorMode(ecpmExcel2007Theme);
    exctUnassigned: SetColorMode(ecpmExcel2007Theme);
  end;
  if not FColorSwatches.SetSelected(Canvas,Value) and (FLinkedPicker <> Nil) then
    FLinkedPicker.FColorSwatches.SetSelected(FLinkedPicker.Canvas,Value);
end;

procedure TAXWColorPicker.SetLinkedPicker(const Value: TAXWColorPicker);
begin
  FLinkedPicker := Value;
  if Value <> Nil then begin
    Value.FLinkedPicker := Self;
    Value.FParentPicker := Self;
  end;
end;

procedure TAXWColorPicker.WMMouseLeave(var Message: TMessage);
begin
  FColorSwatches.ClearMouse(Canvas);
end;

{ TColorSwatches }

procedure TColorSwatches.InitSwatches;
var
  Col,Row: integer;
begin
  Clear;
  SetLength(FSwatches,FCols,FRows);
  for Row := 0 to FRows - 1 do begin
    for Col := 0 to FCols - 1 do begin
      FSwatches[Col,Row] := TColorSwatch.Create;
      FSwatches[Col,Row].Col := Col;
      FSwatches[Col,Row].Row := Row;
      FSwatches[Col,Row].Color.ARGB := RevRGB(clGray);
      FSwatches[Col,Row].X := FX1 + FHorizMarg + Col * (FSwatchSize + FSwatchXSpacing);
      FSwatches[Col,Row].Y := FY1 + FVertMarg + Row * (FSwatchSize + FSwatchYSpacing);
    end;
  end;
  FX2 := FSwatches[FCols - 1,0].X + FSwatchSize + FHorizMarg;
  FY2 := FSwatches[0,FRows - 1].Y + FSwatchSize + FVertMarg;
end;

procedure TColorSwatches.Clear;
var
  Col,Row: integer;
begin
  for Col := 0 to High(FSwatches) do begin
    for Row := 0 to High(FSwatches[Col]) do
      FSwatches[Col,Row].Free;
  end;
end;

procedure TColorSwatches.ClearMouse(Canvas: TCanvas);
var
  Tmp: TColorSwatch;
begin
  FTempSwatch := Nil;
  if FCurrSwatch <> Nil then begin
    Tmp := FCurrSwatch;
    FCurrSwatch := Nil;
    PaintSwatch(Canvas,Tmp);
  end;
end;

procedure TColorSwatches.ClearSelected(Canvas: TCanvas);
var
  Tmp: TColorSwatch;
begin
  FTempSwatch := Nil;
  if FSelectedSwatch <> Nil then begin
    Tmp := FSelectedSwatch;
    FSelectedSwatch := Nil;
    PaintSwatch(Canvas,Tmp);
  end;
end;

function TColorSwatches.ClientHit(X, Y: integer): boolean;
begin
  Result := (X >= FX1) and (X <= FX2) and (Y >= FY1) and (Y <= FY2);
end;

constructor TColorSwatches.Create(Cols, Rows: integer);
begin
  FCols := Cols;
  FRows := Rows;
  FSwatchSize := 14;
  FSwatchXSpacing := 4;
  FSwatchYSpacing := 4;
  FHorizMarg := Max(FSwatchXSpacing div 2,FSwatchYSpacing div 2);
  FVertMarg := FHorizMarg;
  InitSwatches;
end;

destructor TColorSwatches.Destroy;
begin
  Clear;
  inherited;
end;

function TColorSwatches.GetSwatches(Col, Row: integer): TColorSwatch;
begin
  Result := FSwatches[Col,Row];
end;

function TColorSwatches.Hit(X, Y: integer): boolean;
var
  Col,Row: integer;
  WX,WY: integer;
begin
  SetLength(FSwatches,FCols * FRows);
  if SwatchXSpacing > 0 then
    WX := SwatchXSpacing div 2
  else
    WX := 0;
  if SwatchYSpacing > 0 then
    WY := SwatchYSpacing div 2
  else
    WY := 0;
  for Row := 0 to FRows - 1 do begin
    for Col := 0 to FCols - 1 do begin
      FTempSwatch := FSwatches[Col,Row];
      Result := (X >= (FTempSwatch.X - WX)) and (X <= (FTempSwatch.X + SwatchSize + WX)) and (Y >= (FTempSwatch.Y - WY)) and (Y <= (FTempSwatch.Y + SwatchSize + WY));
      if Result then
        Exit;
    end;
  end;
  FTempSwatch := Nil;
  Result := False;
end;

procedure TColorSwatches.MouseMove(Canvas: TCanvas; X, Y: integer);
begin
  if Hit(X,Y) then begin
    if FCurrSwatch <> FTempSwatch then begin
      if FCurrSwatch <> Nil then
        PaintSwatch(Canvas,FCurrSwatch);
      FCurrSwatch := FTempSwatch;
      PaintMouseSwatch(Canvas,FCurrSwatch);
    end;
  end;
end;

procedure TColorSwatches.MouseUp(Canvas: TCanvas; X, Y: integer);
var
  Tmp: TColorSwatch;
begin
  if Hit(X,Y) and (FTempSwatch <> FSelectedSwatch) then begin
    if FSelectedSwatch <> Nil then begin
      Tmp := FSelectedSwatch;
      FSelectedSwatch := Nil;
      PaintSwatch(Canvas,Tmp);
    end;
    FSelectedSwatch := FTempSwatch;

//    if FMultiPicker <> Nil then
//      FMultiPicker.SwatchClicked(FSelectedSwatch);

    PaintSwatch(Canvas,FSelectedSwatch);
  end;
end;

procedure TColorSwatches.Paint(Canvas: TCanvas);
var
  Col,Row: integer;
begin

  Canvas.Brush.Color := clWhite;
  Canvas.Pen.Color := clWhite;
  Canvas.Rectangle(FX1,FY1,FX2,FY2);
  Canvas.Brush.Style := bsSolid;

  for Row := 0 to FRows - 1 do begin
    for Col := 0 to FCols - 1 do
      PaintSwatch(Canvas,FSwatches[Col,Row]);
  end;
end;

procedure TColorSwatches.PaintMouseSwatch(Canvas: TCanvas; Swatch: TColorSwatch);
begin
  Canvas.Brush.Color := RevRGB(Swatch.Color.ARGB);
  Canvas.Pen.Color := SWATCH_MOUSECOLOR1;
  Canvas.Rectangle(Swatch.X,Swatch.Y,Swatch.X + SwatchSize,Swatch.Y + SwatchSize);
  Canvas.Pen.Color := SWATCH_MOUSECOLOR2;
  Canvas.Rectangle(Swatch.X + 1,Swatch.Y + 1,Swatch.X + SwatchSize - 1,Swatch.Y + SwatchSize - 1);
end;

procedure TColorSwatches.PaintSwatch(Canvas: TCanvas; Swatch: TColorSwatch);
begin
  if Swatch = FSelectedSwatch then
    Canvas.Pen.Color := SWATCH_SELECTEDCOLOR
  else
    Canvas.Pen.Color := SWATCH_BORDERCOLOR;
  Canvas.Brush.Color := RevRGB(Swatch.Color.ARGB);
  Canvas.Rectangle(Swatch.X,Swatch.Y,Swatch.X + SwatchSize,Swatch.Y + SwatchSize);
  if Swatch = FSelectedSwatch then begin
    Canvas.Pen.Color := SWATCH_MOUSECOLOR2;
    Canvas.Brush.Style := bsClear;
    Canvas.Rectangle(Swatch.X + 1,Swatch.Y + 1,Swatch.X + SwatchSize - 1,Swatch.Y + SwatchSize - 1);
    Canvas.Brush.Style := bsSolid;
  end
  else if SwatchYSpacing = 0 then begin
    Canvas.Pen.Color := RevRGB(Swatch.Color.ARGB);
    if Swatch.Row > 0 then begin
      Canvas.MoveTo(Swatch.X + 1,Swatch.Y);
      Canvas.LineTo(Swatch.X + SwatchSize - 1,Swatch.Y);
    end;
    if Swatch.Row < (FRows - 1) then begin
      Canvas.MoveTo(Swatch.X + 1,Swatch.Y + SwatchSize - 1);
      Canvas.LineTo(Swatch.X + SwatchSize - 1,Swatch.Y + SwatchSize - 1);
    end;
  end;
end;

procedure TColorSwatches.SetCols(const Value: integer);
begin
  FCols := Value;
  InitSwatches;
end;

procedure TColorSwatches.SetY1(const Value: integer);
begin
  FY1 := Value;
  InitSwatches;
end;

procedure TColorSwatches.SetRows(const Value: integer);
begin
  FRows := Value;
  InitSwatches;
end;

function TColorSwatches.SetSelected(Canvas: TCanvas; Color: TXc12Color): boolean;
var
  Col,Row: integer;
begin
  Result := False;
  for Row := 0 to FRows - 1 do begin
    for Col := 0 to FCols - 1 do begin

      if Xc12ColorEqual(FSwatches[Col,Row].Color,Color) then begin
        if FSwatches[Col,Row] <> FSelectedSwatch then begin
          ClearSelected(Canvas);
          FSelectedSwatch := FSwatches[Col,Row];
          PaintSwatch(Canvas,FSwatches[Col,Row]);
        end;
        Result := True;
        Exit;
      end;

    end;
  end;
end;

procedure TColorSwatches.SetSwatchSize(const Value: integer);
begin
  FSwatchSize := Value;
  InitSwatches;
end;

procedure TColorSwatches.SetSwatchXSpacing(const Value: integer);
begin
  FSwatchXSpacing := Value;
  InitSwatches;
end;

procedure TColorSwatches.SetSwatchYSpacing(const Value: integer);
begin
  FSwatchYSpacing := Value;
  InitSwatches;
end;

procedure TColorSwatches.SetX1(const Value: integer);
begin
  FX1 := Value;
  InitSwatches;
end;

{ TEx2007ThemeSwatches }

constructor TEx2007ThemeSwatches.Create(Compact: boolean);
var
  Col: integer;
begin
  FAdjust[0,0] := -0.05;
  FAdjust[0,1] := -0.15;
  FAdjust[0,2] := -0.25;
  FAdjust[0,3] := -0.35;
  FAdjust[0,4] := -0.50;

  FAdjust[1,0] :=  0.50;
  FAdjust[1,1] :=  0.35;
  FAdjust[1,2] :=  0.25;
  FAdjust[1,3] :=  0.15;
  FAdjust[1,4] :=  0.05;

  FAdjust[2,0] := -0.10;
  FAdjust[2,1] := -0.25;
  FAdjust[2,2] := -0.50;
  FAdjust[2,3] := -0.75;
  FAdjust[2,4] := -0.90;

  for Col := 3 to 9 do begin
    FAdjust[Col,0] :=  0.80;
    FAdjust[Col,1] :=  0.60;
    FAdjust[Col,2] :=  0.40;
    FAdjust[Col,3] := -0.25;
    FAdjust[Col,4] := -0.50;
  end;

  SetLength(FSwatchList,2);

  FSwatchList[0] := TColorSwatches.Create(10,1);
  FSwatchList[1] := TColorSwatches.Create(10,5);
  FSwatchList[1].FY1 := 25;
  if Compact then
    FSwatchList[1].FSwatchYSpacing := 0;

  inherited Create;

  Initiate;
end;

procedure TEx2007ThemeSwatches.Initiate;
const
  ColOrder: array[0..9] of integer = (0,1,3,2,4,5,6,7,8,9);
var
  Col,Row: integer;
begin
  for Col := 0 to 9 do begin
    FSwatchList[0][Col,0].Color.ColorType := exctTheme;
    FSwatchList[0][Col,0].Color.Theme := TXc12ClrSchemeColor(ColOrder[Col]);
    FSwatchList[0][Col,0].Color.Tint := 0;
    FSwatchList[0][Col,0].Color.ARGB := Xc12DefColorSchemeRGB[TXc12ClrSchemeColor(ColOrder[Col])];
  end;

  for Col := 0 to FSwatchList[1].Cols - 1 do begin
    for Row := 0 to FSwatchList[1].Rows - 1 do begin
      FSwatchList[1][Col,Row].Color.ColorType := exctTheme;
      FSwatchList[1][Col,Row].Color.Theme := TXc12ClrSchemeColor(ColOrder[Col]);
      FSwatchList[1][Col,Row].Color.Tint := FAdjust[Col,Row];

      FSwatchList[1][Col,Row].Color.ARGB := Xc12ColorToRGB(FSwatchList[1][Col,Row].Color);
    end;
  end;

  inherited Initiate;
end;

{ TEx97Swatches }

constructor TEx97Swatches.Create;
var
  i: integer;
  PaletteInitiated: boolean;
begin
  for i := 0 to High(Xc12IndexColorPaletteRGB) do begin
    PaletteInitiated := Xc12IndexColorPaletteRGB[i] <> 0;
    if PaletteInitiated then
      Break;
  end;
  if not PaletteInitiated then
    Move(TXc12DefaultIndexColorPalette[0],Xc12IndexColorPaletteRGB[0],SizeOf(integer) * Length(Xc12IndexColorPaletteRGB));

  SetLength(FSwatchList,1);

  FSwatchList[0] := TColorSwatches.Create(8,7);

  inherited Create;

  Initiate;
end;

procedure TEx97Swatches.Initiate;
var
  Col,Row: integer;
begin
  for Row := 0 to FSwatchList[0].Rows - 1 do begin
    for Col := 0 to FSwatchList[0].Cols - 1 do begin
      FSwatchList[0][Col,Row].Color.ColorType := exctIndexed;
      FSwatchList[0][Col,Row].Color.Indexed := TXc12IndexColor(Col + ((Row + 1) * 8));
      FSwatchList[0][Col,Row].Color.ARGB := Xc12IndexColorPaletteRGB[Col + ((Row + 1) * 8)]
    end;
  end;

  inherited Initiate;
end;

{ TSwatchContainer }

procedure TSwatchContainer.ClearMouse(Canvas: TCanvas);
var
  i: integer;
begin
  for i := 0 to High(FSwatchList) do
    FSwatchList[i].ClearMouse(Canvas);
end;

procedure TSwatchContainer.ClearSelected(Canvas: TCanvas);
var
  i: integer;
begin
  for i := 0 to High(FSwatchList) do
    FSwatchList[i].ClearSelected(Canvas);
  FSelectedColor.ColorType := exctUnassigned;
end;

function TSwatchContainer.ClientHit(X, Y: integer): boolean;
begin
  Result := (X >= FX1) and (X <= FX2) and (Y >= FY1) and (Y <= FY2);
end;

constructor TSwatchContainer.Create;
var
  i: integer;
begin
  for i := 0 to High(FSwatchList) do
    FSwatchList[i].InitSwatches;
end;

destructor TSwatchContainer.Destroy;
var
  i: integer;
begin
   for i := 0 to High(FSwatchList) do
     FSwatchList[i].Free;

  inherited;
end;

procedure TSwatchContainer.Initiate;
var
  i: integer;
begin
  FX1 := MAXINT;
  FY1 := MAXINT;
  FX2 := 0;
  FY2 := 0;
  for i := 0 to High(FSwatchList) do begin
    FX1 := Min(FX1,FSwatchList[i].X1);
    FY1 := Min(FY1,FSwatchList[i].Y1);
    FX2 := Max(FX2,FSwatchList[i].X2);
    FY2 := Max(FY2,FSwatchList[i].Y2);
  end;
end;

procedure TSwatchContainer.MouseMove(Canvas: TCanvas; X, Y: integer);
var
  i: integer;
begin
  for i := 0 to High(FSwatchList) do begin
    if FSwatchList[i].ClientHit(X,Y) then
      FSwatchList[i].MouseMove(Canvas,X,Y)
    else
      FSwatchList[i].ClearMouse(Canvas);
  end;
end;

procedure TSwatchContainer.MouseUp(Canvas: TCanvas; X, Y: integer);
var
  i,j: integer;
begin
  for i := 0 to High(FSwatchList) do begin
    if FSwatchList[i].ClientHit(X,Y) then begin
      FSwatchList[i].MouseUp(Canvas,X,Y);
      for j := 0 to High(FSwatchList) do begin
        if j <> i then
          FSwatchList[j].ClearSelected(Canvas);
      end;
      if FSwatchList[i].FSelectedSwatch <> Nil then
        FSelectedColor := FSwatchList[i].FSelectedSwatch.Color;
      Break;
    end;
  end;
end;

procedure TSwatchContainer.Paint(Canvas: TCanvas);
var
  i: integer;
begin
  for i := 0 to High(FSwatchList) do
    FSwatchList[i].Paint(Canvas);
end;

function TSwatchContainer.SetSelected(Canvas: TCanvas; Color: TXc12Color): boolean;
var
  i,j: integer;
begin
  for i := 0 to High(FSwatchList) do begin
    if FSwatchList[i].SetSelected(Canvas,Color) then begin
      for j := 0 to High(FSwatchList) do begin
        if j <> i then
          FSwatchList[j].ClearSelected(Canvas);
      end;
      FSelectedColor := Color;
      Result := True;
      Exit;
    end;
  end;
  Result := False;
end;

procedure TSwatchContainer.SetX1(const Value: integer);
begin
  FX1 := Value;
end;

procedure TSwatchContainer.SetY1(const Value: integer);
begin
  FY1 := Value;
end;

function TSwatchContainer.SwatchHit: boolean;
var
  i: integer;
begin
  for i := 0 to High(FSwatchList) do begin
    Result := FSwatchList[i].FTempSwatch <> Nil;
    if Result then
      Exit;
  end;
  Result := False;
end;

procedure TSwatchContainer._ClearSelected;
var
  i: integer;
begin
  for i := 0 to High(FSwatchList) do
    FSwatchList[i].FSelectedSwatch := Nil;
  FSelectedColor.ColorType := exctUnassigned;
end;

{ TEx2007StandardSwatches }

constructor TEx2007StandardSwatches.Create;
begin
  SetLength(FSwatchList,1);

  FSwatchList[0] := TColorSwatches.Create(10,1);

  inherited Create;

  Initiate;
end;

procedure TEx2007StandardSwatches.Initiate;
var
  Col: integer;
begin
  for Col := 0 to FSwatchList[0].Cols - 1 do begin
    FSwatchList[0][Col,0].Color.ColorType := exctRgb;
    FSwatchList[0][Col,0].Color.OrigRGB := Ex12StandardColors[Col];
    FSwatchList[0][Col,0].Color.ARGB := Ex12StandardColors[Col];
  end;

  inherited Initiate;
end;

procedure FillDefaultColors;
var
  i: integer;
  j: TXc12ClrSchemeColor;
begin
  for i := 9 to High(TXc12DefaultIndexColorPalette) do
    Xc12IndexColorPalette[i] := TXc12DefaultIndexColorPalette[i];

  for i := 9 to High(TXc12DefaultIndexColorPalette) do
    Xc12IndexColorPaletteRGB[i] := TXc12DefaultIndexColorPaletteRGB[i];

  for j := Low(TXc12ClrSchemeColor) to High(TXc12ClrSchemeColor) do
    Xc12DefColorSchemeRGB[j] := Xc12DefColorSchemeValsRGB[j];
end;

{ TAXWMultiColorPicker }

constructor TAXWMultiColorPicker.Create(AOwner: TComponent);
begin
  inherited;

  FSelColor.Auto := True;

  FTitleHeight := 20;

  FCPTheme := TAXWColorPicker.Create(Owner);
  FCPTheme.Parent := Self;
  FCPTheme.Left := 1;
  FCPTheme.Top := FTitleHeight * 2;
  FCPTheme.ColorMode := ecpmExcel2007ThemeCompact;
  FCPTheme.OnClick := ThemeColorClicked;

  FCPStandard := TAXWColorPicker.Create(Owner);
  FCPStandard.Parent := Self;
  FCPStandard.Left := 1;
  FCPStandard.Top := FCPTheme.Top + FCPTheme.Height + FTitleHeight;
  FCPStandard.ColorMode := ecpmExcel2007Standard;
  FCPStandard.LinkedPicker := FCPTheme;
  FCPStandard.OnClick := StandardColorClicked;

  Width := FCPStandard.Width + 5;
  Height := FCPStandard.Top + FCPStandard.Height + 5;
end;

destructor TAXWMultiColorPicker.Destroy;
begin
  FCPTheme.Free;
  FCPStandard.Free;

  inherited;
end;

function TAXWMultiColorPicker.GetCompact: boolean;
begin
  Result := FCPTheme.ColorMode = ecpmExcel2007ThemeCompact;
end;

procedure TAXWMultiColorPicker.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;

  if Y < FTitleHeight then begin
    FSelColor := MakeXc12ColorAuto;

    if Assigned(FSelectEvent) then
      FSelectEvent(Self);

    Visible := False;
  end;
end;

procedure TAXWMultiColorPicker.Paint;
var
  R: TRect;
begin
  inherited;

  SetRect(R,0,0,Width,Height);

  Canvas.Brush.Color := clWhite;
  Canvas.Pen.Color := clGray;
  Canvas.Rectangle(R);

  Canvas.Pen.Color := RGB(219,229,241);
  Canvas.Brush.Style := bsSolid;
  Canvas.Brush.Color := RGB(219,229,241);
  Canvas.Font.Color := clNavy;
  Canvas.Font.Style := [];

  Canvas.Rectangle(1,FTitleHeight,Width - 2,FTitleHeight * 2);
  Canvas.Rectangle(1,FCPStandard.Top - FTitleHeight,Width - 2,FCPStandard.Top - 2);

  Canvas.Brush.Color := clBlack;
  Canvas.Rectangle(4,4,FTitleHeight - 4,FTitleHeight - 4);
  if FSelColor.Auto then begin
    Canvas.Brush.Style := bsClear;
    Canvas.Pen.Color := SWATCH_MOUSECOLOR1;
    Canvas.Pen.Width := 2;
    Canvas.Rectangle(3,3,FTitleHeight - 2,FTitleHeight - 2);
    Canvas.Pen.Width := 1;
  end;

  Canvas.Brush.Style := bsClear;
  Canvas.TextOut(24,4,'Automatic');

  Canvas.Font.Style := [fsBold];

  Canvas.TextOut(8,FTitleHeight + 2,'Theme colors');
  Canvas.TextOut(8,FCPStandard.Top - FTitleHeight + 2,'Standard colors');
end;

procedure TAXWMultiColorPicker.SetColor(const Value: TXc12Color);
begin
  FCPTheme.FindAndSelect(Value);
end;

procedure TAXWMultiColorPicker.SetCompact(const Value: boolean);
begin
  if Value <> GetCompact then begin
    if Value then
      FCPTheme.ColorMode := ecpmExcel2007ThemeCompact
    else
      FCPTheme.ColorMode := ecpmExcel2007Theme;
  end;
end;

procedure TAXWMultiColorPicker.StandardColorClicked(ASender: TObject);
begin
  FSelColor.Auto := False;

  FSelColor := FCPStandard.ExcelColor;

  if Assigned(FSelectEvent) then
    FSelectEvent(Self);
end;

procedure TAXWMultiColorPicker.ThemeColorClicked(ASender: TObject);
begin
  FSelColor := FCPTheme.ExcelColor;

  FSelColor.Auto := False;

  if Assigned(FSelectEvent) then
    FSelectEvent(Self);
end;

initialization

FillDefaultColors;

end.
