unit AXWRibbon;

{$B-}
{$H+}
{$R-}
{$I AxCompilers.inc}

{
  MIT License -  http://www.opensource.org/licenses/mit-license.php
  Copyright (c) 2018 Axolot Data

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.

}

interface

uses Classes, SysUtils, Math, Messages, Windows, Graphics, Controls, StdCtrls,
     ComCtrls, ExtCtrls, Forms, ActnList, Contnrs, Menus, Themes, ImgList,
{$ifdef DELPHI_XE2_OR_LATER}
     UITypes,
{$endif}
     AXWActionsBase,
{$ifndef AXWRIBBON}
     AXWActions,
{$endif}
     AXWColorPickerVcl;

const WM_AXWR_CLOSEWINDOW              = WM_USER + 1;

const AXWR_COLOR                       = $00F1F1F1;
const AXWR_COLOR_LINE                  = $00D5D5D5;

const AXWR_COLOR_TAB                   = $002A579A;
const AXWR_COLOR_TAB_HOOVER            = $003E6DB6;
const AXWR_COLOR_TAB_SEL               = $00F1F1F1;
const AXWR_COLOR_TAB_FONT              = $00FFFFFF;
const AXWR_COLOR_TAB_FONT_SEL          = $002A579A;

const AXWR_COLOR_GROUP                 = $00F1F1F1;
const AXWR_COLOR_GROUP_FONT            = $00666666;
const AXWR_COLOR_GROUP_CONT            = $00F1F1F1;
const AXWR_COLOR_GROUP_CONT_BTN        = $00F1F1F1;
const AXWR_COLOR_GROUP_CONT_BTN_HOOVER = $00D1D1D1;
const AXWR_COLOR_GROUP_CONT_BTN_DOWN   = $00A9A9A9;
const AXWR_COLOR_GROUP_CONT_IMAGE      = $00666666;

const AXWR_COLOR_FONT_BUTTON           = $00000000;
const AXWR_COLOR_FONT_BUTTON_DISABLED  = $00B5B2B5;
const AXWR_COLOR_BTN                   = $00F1F1F1;
const AXWR_COLOR_CHECKBOX              = $00FFFFFF;
const AXWR_COLOR_BTN_HOOVER            = $00C5C5C5;
const AXWR_COLOR_BTN_DOWN              = $00AEAEAE;
const AXWR_COLOR_BTN_DOWN_HOOVER       = $00808080;
const AXWR_COLOR_BTN_SPLIT             = $00F1F1F1;
const AXWR_COLOR_BTN_SPLIT_HOOVER      = AXWR_COLOR_GROUP;
const AXWR_COLOR_BTN_SPLIT_BORDER      = AXWR_COLOR_BTN_HOOVER;

const AXWR_COLOR_COMBO                 =  $00FFFFFF;
const AXWR_COLOR_COMBO_LINE            =  $00C5C5C5;
const AXWR_COLOR_COMBO_FONT            =  $00000000;

const AXWR_SIZE_CONTENT_BTN            = 7;
const AXWR_SIZE_DOWN_BTN               = 5;
const AXWR_SIZE_LARGE_BUTTON           = 44;
const AXWR_SIZE_BTN_BORDER             = 1;

const AXWR_LARGE_IMAGE_MARG            = 2;
const AXWR_SMALL_IMAGE_MARG            = 1;

{$ifndef DELPHI_2007_OR_LATER}
type TTextFormats = (tfBottom, tfCalcRect, tfCenter, tfEditControl, tfEndEllipsis,
    tfPathEllipsis, tfExpandTabs, tfExternalLeading, tfLeft, tfModifyString,
    tfNoClip, tfNoPrefix, tfRight, tfRtlReading, tfSingleLine, tfTop,
    tfVerticalCenter, tfWordBreak, tfHidePrefix, tfNoFullWidthCharBreak,
    tfPrefixOnly, tfTabStop, tfWordEllipsis, tfComposited);

    TTextFormat = set of TTextFormats;
{$endif}

type TAXWRibbonPaintEvent = procedure (ACanvas: TCanvas) of object;

type TAXWRMouseState = (amsNormal,amsHoover,amsSplitHoover,amsDown,amsHooverDown);

type TAXWRibbon = class;
     TAXWRCollectionItem = class;
     TAXWRibbonButton = class;
     TAXWRibbonGroup = class;
     TAXWRibbonPage = class;
     TAXWRibbonAppMenuItem = class;
     TAXWRibbonAppMenuItems = class;
     TAXWRibbonAppMenu = class;

     TAXWRHooverListBoxStyle = (ahlbsNormal,ahlbsMenu,ahlbsMenuPage,ahlbsFonts);

     TAXWRibbonButtonEvent = procedure (ASender: TAXWRibbonButton) of object;

     TAXWRHooverListBox = class(TListBox)
protected
     FRibbon     : TAXWRibbon;

     FClickEvent : TNotifyEvent;

     FAXWStyle   : TAXWRHooverListBoxStyle;

     FHooverItem : integer;
     FHooverSplit: boolean;

     FTimer      : TTimer;

     procedure WndProc(var Message: TMessage); override;

     procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;

     procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;

     procedure MeasureItem(Index: Integer; var Height: Integer); override;

     procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
     procedure DrawItemNormal(Index: Integer; Rect: TRect; State: TOwnerDrawState);
     procedure DrawItemFonts(Index: Integer; Rect: TRect; State: TOwnerDrawState);
     procedure DrawItemMenu(Index: Integer; Rect: TRect; State: TOwnerDrawState);
     procedure DrawItemMenuPage(Index: Integer; Rect: TRect; State: TOwnerDrawState);

     procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
     procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;

     procedure HandleTimer(ASender: TObject);
public
     constructor Create(AOwner: TComponent; ARibbon: TAXWRibbon); reintroduce; virtual;
     destructor Destroy; override;

     property AXWStyle: TAXWRHooverListBoxStyle read FAXWStyle write FAXWStyle;

     property OnClick: TNotifyEvent read FClickEvent write FClickEvent;
     end;

     TAXWRListBox = class(TAXWRHooverListBox)
protected
     FButton    : TAXWRibbonButton;
     FDestroying: boolean;

     procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;

     procedure WndProc(var Message: TMessage); override;

     procedure Hide;

     procedure DoClick(ASender: TObject);
public
     constructor Create(AOwner: TComponent; ARibbon: TAXWRibbon); override;
     end;

     TAXWRMultiColorPicker = class(TAXWMultiColorPicker)
protected
     FButton    : TAXWRibbonButton;

     procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;

     procedure WndProc(var Message: TMessage); override;

     procedure Hide;
public
     property Button: TAXWRibbonButton read FButton write FButton;
     end;

     TAXWRCollection = class(TCollection)
protected
     FRibbon: TAXWRibbon;
     FRect  : TRect;
     FState : TAXWRMouseState;

     procedure Update(Item: TCollectionItem); override;

     function  Hit(AX,AY: integer): TAXWRCollectionItem; virtual;
     procedure Calc(ACanvas: TCanvas; var AX,AY: integer); virtual;
     procedure Paint(ACanvas: TCanvas); virtual;

     property State: TAXWRMouseState read FState write FState;
public
     end;

     TAXWRCollectionItem = class(TCollectionItem)
protected
     FRect   : TRect;
     FState  : TAXWRMouseState;
     FCapture: boolean;
     FEnabled: boolean;

     function  Hit(AX,AY: integer): TAXWRCollectionItem; virtual;
     procedure MouseEnter; virtual;
     procedure MouseLeave; virtual;
     procedure Calc(ACanvas: TCanvas; var AX,AY: integer); virtual;
     procedure Paint(ACanvas: TCanvas); virtual;

     procedure DoPaintButton(ACanvas: TCanvas; ARect: TRect; ASplitState,ADown: boolean);
     procedure DoPaintButtonTheme(ACanvas: TCanvas; ARect: TRect; ASplitState,ADown: boolean);

     procedure PaintButton(ACanvas: TCanvas; ARect: TRect; ASplitState,ADown: boolean);

     procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;
     procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;
     procedure CaptureChange(AStart: boolean); virtual;

     property State  : TAXWRMouseState read FState write FState;
     property Capture: boolean read FCapture write FCapture;
public
     constructor Create(Collection: TCollection); override;
     end;

     TAXWRibbonSubButton = class(TAXWRCollectionItem)
protected
     FPaintEvent: TAXWRibbonPaintEvent;
     FAction    : TAction;
     FButton    : TAXWRibbonButton;

     procedure Paint(ACanvas: TCanvas); override;

     procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
     procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
public
     constructor Create(AButton: TAXWRibbonButton); reintroduce;

     property Action : TAction read FAction write FAction;
     end;

     TAXWRibbonButtonStyle = (arbsNormal,arbsDropDown,arbsCheckBox,arbsSplit,arbsSplitThemeColors,arbsComboBox,arbsFontComboBox,arbsSeparator,arbsBreak);

     TAXWRibbonButton = class(TAXWRCollectionItem)
private
     procedure SetCaption(const Value: string);
     procedure SetStyle(const Value: TAXWRibbonButtonStyle);
     procedure SetImageIndex(const Value: TImageIndex);
     procedure SetEnabled(const Value: boolean);
     procedure SetCheked(const Value: boolean);
     procedure SetComboWidth(const Value: integer);
     function  GetAction: TAction;
     procedure SetAction(const Value: TAction);
     procedure SetComboItems(const Value: TStringList);
     procedure SetGroupIndex(const Value: integer);
     procedure SetShowCaption(const Value: boolean);
     function  GetSplitAction: TAction;
     procedure SetSplitAction(const Value: TAction);
protected
     FRibbon        : TAXWRibbon;
     FName          : string;
     FStyle         : TAXWRibbonButtonStyle;
     FCaption       : string;
     FShowCaption   : boolean;
     FImageIndex    : TImageIndex;
     FChecked       : boolean;
     FAutoCheck     : boolean;
     // This is the part with the down arrow.
     FSplitBtn      : TAXWRibbonSubButton;
     FSplitHit      : boolean;
     FAction        : TAction;
     FGroupIndex    : integer;
     FComboWidth    : integer;
     FComboItems    : TStringList;
     FComboItemIndex: integer;

     FDropDownMenu: TPopupMenu;

     procedure Calc(ACanvas: TCanvas; var AX,AY: integer); override;
     procedure CalcSmall(ACanvas: TCanvas; var AX,AY: integer);
     procedure CalcLarge(ACanvas: TCanvas; var AX,AY: integer);
     function  Hit(AX,AY: integer): TAXWRCollectionItem; override;
     procedure Paint(ACanvas: TCanvas); override;
     function  DoDropDown: boolean;

     procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
     procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;

     procedure SplitClicked;

     procedure ListBoxClicked;
     procedure ColorPickerClicked;

     procedure SetBtnColor(ACanvas: TCanvas);

     procedure PaintSplit(ACanvas: TCanvas);
     procedure PaintBtnRect(ACanvas: TCanvas; AItem: TAXWRCollectionItem);

     procedure DoPaintText(ACanvas: TCanvas; ARect: TRect; AFormat: TTextFormat);

     procedure PaintSmallNormal(ACanvas: TCanvas);
     procedure PaintSmallCheckBox(ACanvas: TCanvas);
     procedure PaintSmallSplit(ACanvas: TCanvas);
     procedure PaintSmallImage(ACanvas: TCanvas; var AX: integer);
     procedure PaintComboBox(ACanvas: TCanvas);
     procedure PaintLargeNormal(ACanvas: TCanvas);
     procedure PaintLargeSplit(ACanvas: TCanvas);
     procedure PaintLargeCaption(ACanvas: TCanvas);

     procedure HandleActionChange(ASender: TObject);
public
     constructor Create(Collection: TCollection); override;
     destructor Destroy; override;

     procedure AssignTo(Dest: TPersistent); override;
published
     property Name        : string read FName write FName;
     property Action      : TAction read GetAction write SetAction;
     property Caption     : string read FCaption write SetCaption;
     property ShowCaption : boolean read FShowCaption write SetShowCaption;
     property Style       : TAXWRibbonButtonStyle read FStyle write SetStyle;
     property ImageIndex  : TImageIndex read FImageIndex write SetImageIndex;
     property Enabled     : boolean read FEnabled write SetEnabled;
     property Checked     : boolean read FChecked write SetCheked;
     property AutoCheck   : boolean read FAutoCheck write FAutoCheck;
     property GroupIndex  : integer read FGroupIndex write SetGroupIndex;
     property ComboWidth  : integer read FComboWidth write SetComboWidth;
     property ComboItems  : TStringList read FComboItems write SetComboItems;
     property DropDownMenu: TPopupMenu read FDropDownMenu write FDropDownMenu;
     property SplitAction : TAction read GetSplitAction write SetSplitAction;
     end;

     TAXWRibbonButtons = class(TAXWRCollection)
private
     function  GetItems(Index: Integer): TAXWRibbonButton;
     procedure SetItems(Index: Integer; const Value: TAXWRibbonButton);
protected
     FGroup     : TAXWRibbonGroup;
     FLarge     : boolean;
     FBreakCount: integer;

     function  GetOwner: TPersistent; override;

     procedure Calc(ACanvas: TCanvas; var AX, AY: integer); override;
     procedure CalcSmall(ACanvas: TCanvas; var AX, AY: integer);
     procedure CalcLarge(ACanvas: TCanvas; var AX, AY: integer);
     procedure Paint(ACanvas: TCanvas); override;

     procedure DoGroup(AButton: TAXWRibbonButton);
public
     constructor Create(ARibbon: TAXWRibbon; AGroup: TAXWRibbonGroup; ALarge: boolean);
     destructor Destroy; override;

     function  Add: TAXWRibbonButton; overload;
     function  Add(ACaption: string; AImageIndex: integer; AAction: TAction): TAXWRibbonButton; overload;

     property Items[Index: Integer]: TAXWRibbonButton read GetItems write SetItems; default;
     end;

     TAXWRibbonGroup = class(TAXWRCollectionItem)
private
     procedure SetCaption(const Value: string);
     procedure SetContent(const Value: boolean);
     function  GetContent: boolean;
     function  GetContentAction: TAction;
     procedure SetContentAction(const Value: TAction);
     procedure SetLarge(const Value: TAXWRibbonButtons);
     procedure SetSmall(const Value: TAXWRibbonButtons);
protected
     FCaption: string;
     FContent: TAXWRibbonSubButton;
     FLarge  : TAXWRibbonButtons;
     FSmall  : TAXWRibbonButtons;

     procedure Calc(ACanvas: TCanvas; var AX,AY: integer); override;
     function  Hit(AX,AY: integer): TAXWRCollectionItem; override;

     procedure Paint(ACanvas: TCanvas); override;
     procedure DoPaint(ACanvas: TCanvas);
     procedure DoPaintTheme(ACanvas: TCanvas);

     procedure PaintContent(ACanvas: TCanvas);

     procedure CalcContent;
public
     constructor Create(Collection: TCollection); override;
     destructor Destroy; override;

     procedure AssignTo(Dest: TPersistent); override;
published
     property Caption       : string read FCaption write SetCaption;
     property ContentButton : boolean read GetContent write SetContent;
     property LargeButtons  : TAXWRibbonButtons read FLarge write SetLarge;
     property SmallButtons  : TAXWRibbonButtons read FSmall write SetSmall;
     property ContentAction : TAction read GetContentAction write SetContentAction;
     end;

     TAXWRibbonGroups = class(TAXWRCollection)
private
     function  GetItems(Index: Integer): TAXWRibbonGroup;
     procedure SetItems(Index: Integer; const Value: TAXWRibbonGroup);
protected
     FRibbon: TAXWRibbon;
     FPage  : TAXWRibbonPage;

     function  GetOwner: TPersistent; override;

     procedure Calc(ACanvas: TCanvas; var AX, AY: integer); override;
     procedure Paint(ACanvas: TCanvas); override;
public
     constructor Create(ARibbon: TAXWRibbon; APage: TAXWRibbonPage);
     destructor Destroy; override;

     function  Add: TAXWRibbonGroup;

     property Items[Index: Integer]: TAXWRibbonGroup read GetItems write SetItems; default;
     end;

     TAXWRibbonTab = class(TAXWRCollectionItem)
protected
     FCaption    : string;
     FClickEvent : TNotifyEvent;

     procedure Calc(ACanvas: TCanvas; var AX, AY: integer); override;
     procedure Paint(ACanvas: TCanvas); override;

     procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
public
     procedure AssignTo(Dest: TPersistent); override;

     property Caption: string read FCaption write FCaption;
     end;

     TAXWRibbonTabs = class(TAXWRCollection)
private
     function  GetTab(Index: Integer): TAXWRibbonTab;
     procedure SetTab(Index: Integer; const Value: TAXWRibbonTab);
protected
     FRibbon  : TAXWRibbon;
     FSelected: TAXWRibbonTab;

     procedure Calc(ACanvas: TCanvas; var AX, AY: integer); override;
     procedure Paint(ACanvas: TCanvas); override;
public
     constructor Create(ARibbon: TAXWRibbon);

     function  Add: TAXWRibbonTab;

     property Items[Index: Integer]: TAXWRibbonTab read GetTab write SetTab; default;
     end;

     TAXWRibbonPage = class(TAXWRCollectionItem)
private
     procedure SetCaption(const Value: string);
     function  GetCaption: string;
     procedure SetGroups(const Value: TAXWRibbonGroups);
protected
     FTab    : TAXWRibbonTab;
     FGroups : TAXWRibbonGroups;

     procedure Calc(ACanvas: TCanvas; var AX, AY: integer); override;
     function  Hit(AX,AY: integer): TAXWRCollectionItem; override;
     procedure Paint(ACanvas: TCanvas); override;
public
     constructor Create(Collection: TCollection); override;
     destructor Destroy; override;

     procedure AssignTo(Dest: TPersistent); override;
published
     property Caption: string read GetCaption write SetCaption;
     property Groups : TAXWRibbonGroups read FGroups write SetGroups;
     end;

     TAXWRibbonPages = class(TAXWRCollection)
private
     function  GetPage(Index: Integer): TAXWRibbonPage;
     procedure SetPage(Index: Integer; const Value: TAXWRibbonPage);
     procedure SetSelectedPage(const Value: TAXWRibbonPage);
protected
     FTabs    : TAXWRibbonTabs;
     FSelected: TAXWRibbonPage;

     function  GetOwner: TPersistent; override;
     procedure Notify(Item: TCollectionItem; Action: TCollectionNotification); override;

     function  Hit(AX,AY: integer): TAXWRCollectionItem; override;
     procedure Calc(ACanvas: TCanvas; var AX, AY: integer); override;
     procedure Paint(ACanvas: TCanvas); override;

     procedure TabClick(ASender: TObject);
public
     constructor Create(ARibbon: TAXWRibbon);
     destructor Destroy; override;

     function  Add: TAXWRibbonPage;

     procedure Delete(AIndex: integer);

     property SelectedPage: TAXWRibbonPage read FSelected write SetSelectedPage;

     property Items[Index: Integer]: TAXWRibbonPage read GetPage write SetPage; default;
published
     end;

     TAXWRibbonAppSubMenuItem = class(TCollectionItem)
private
     procedure SetAction(const Value: TAction);
protected
     FCaption    : string;
     FDescription: string;
     FImageIndex : TImageIndex;
     FAction     : TAction;

     procedure HandleActionChange(ASender: TObject);
public
     constructor Create(Collection: TCollection); override;
     destructor Destroy; override;

     procedure AssignTo(Dest: TPersistent); override;
published
     property Caption    : string read FCaption write FCaption;
     property Description: string read FDescription write FDescription;
     property ImageIndex : TImageIndex read FImageIndex write FImageIndex;
     property Action     : TAction read FAction write SetAction;
     end;

     TAXWRibbonAppSubMenuItems = class(TCollection)
private
     function  GetItem(Index: Integer): TAXWRibbonAppSubMenuItem;
     procedure SetItem(Index: Integer; const Value: TAXWRibbonAppSubMenuItem);
protected
     FAppMenuItem: TAXWRibbonAppMenuItem;

     function  GetOwner: TPersistent; override;
public
     constructor Create(AAppMenuItem: TAXWRibbonAppMenuItem); overload;
     destructor Destroy; override;

     function  Add: TAXWRibbonAppSubMenuItem;

     property Items[Index: Integer]: TAXWRibbonAppSubMenuItem read GetItem write SetItem; default;
published
     end;

     TAXWRibbonAppMenuItem = class(TCollectionItem)
private
     procedure SetSubMenu(const Value: TAXWRibbonAppSubMenuItems);
     procedure SetAction(const Value: TAction);
protected
     FCaption    : string;
     FImageIndex : TImageIndex;
     FAction     : TAction;
     FSubMenu    : TAXWRibbonAppSubMenuItems;
     FShowMRU    : boolean;
     FIsSplit    : boolean;
     FSeparator  : boolean;

     procedure HandleActionChange(ASender: TObject);
public
     constructor Create(Collection: TCollection); override;
     destructor Destroy; override;

     procedure AssignTo(Dest: TPersistent); override;
published
     property Caption    : string read FCaption write FCaption;
     property ImageIndex : TImageIndex read FImageIndex write FImageIndex;
     property Action     : TAction read FAction write SetAction;
     property SubMenu    : TAXWRibbonAppSubMenuItems read FSubMenu write SetSubMenu;
     property ShowMRU    : boolean read FShowMRU write FShowMRU;
     property IsSplit    : boolean read FIsSplit write FIsSplit;
     property Separator  : boolean read FSeparator write FSeparator;
     end;

     TAXWRibbonAppMenuItems = class(TCollection)
private
     function  GetItem(Index: Integer): TAXWRibbonAppMenuItem;
     procedure SetItem(Index: Integer; const Value: TAXWRibbonAppMenuItem);
protected
     FRibbon  : TAXWRibbon;

     FMRU     : TStringList;
     FMRUCount: integer;

     function  GetOwner: TPersistent; override;
public
     constructor Create(ARibbon: TAXWRibbon); overload;
     destructor Destroy; override;

     function  Add: TAXWRibbonAppMenuItem;
     procedure AddMRU(AText: string);

     property MRU     : TStringList read FMRU;
     property MRUCount: integer read FMRUCount write FMRUCount;

     property Items[Index: Integer]: TAXWRibbonAppMenuItem read GetItem write SetItem; default;
     end;

     TAXWRibbonAppMenu = class(TPanel)
protected
     FRibbon   : TAXWRibbon;

     FLbMenu   : TAXWRHooverListBox;
     FLbPage   : TAXWRHooverListBox;

     FMRUIndex : integer;

     FCurrTab  : TAXWRibbonTab;

     FActive   : boolean;

     procedure HandleSelectPage(ASender: TObject);
     procedure HandleMenuClick(ASender: TObject);
     procedure HandlePageClick(ASender: TObject);
public
     constructor Create(AOwner: TComponent; ARibbon: TAXWRibbon); reintroduce;
     destructor Destroy; override;

     property Active   : boolean read FActive write FActive;
     property MRUIndex : integer read FMRUIndex;
     end;

     TAXWRibbon = class(TCustomControl)
private
     procedure SetSelectedPage(const Value: TAXWRibbonPage);
     function  GetSelectedPage: TAXWRibbonPage;
     procedure SetPages(const Value: TAXWRibbonPages);
     procedure SetAppMenu(const Value: TAXWRibbonAppMenuItems);
     procedure SetOffice2016(const Value: boolean);
     function  GetMRUString: string;
     function  GetSelectedPageIndex: integer;
     procedure SetSelectedPageIndex(const Value: integer);
{$ifndef AXWRIBBON}
     procedure SetAXWActions(const Value: TAXWActionList);
{$endif}
     function  GetLargeImages: TCustomImageList;
     function  GetSmallImages: TCustomImageList;
protected
     FSetupDone    : boolean;

     FOffice2016   : boolean;

     FTextHeight   : integer;
     FTextMargin   : integer;
     FCharWidth    : integer;
     FRowHeight    : integer;
     FTabHeight    : integer;
     FTabMargin    : integer;
     FPageHeight   : integer;
     FContentSize  : integer;
     FGrpCapSize   : integer;
     FLargeWidth   : integer;
     FLineWidth    : integer;
     FSmallSplitSz : integer;
     FSmallImgSz   : integer;
     FLargeImgSz   : integer;
     FCheckBoxSz   : integer;

{$ifndef AXWRIBBON}
     FAXWActions   : TAXWActionList;
{$endif}

     FAppMenu      : TAXWRibbonAppMenu;
     FAppMenuItems : TAXWRibbonAppMenuItems;

     FMouseIsDown  : boolean;

     FPages        : TAXWRibbonPages;
     FCurrItem     : TAXWRCollectionItem;

     FToDestroy    : TObjectList;

     FSmallImages  : TCustomImageList;
     FLargeImages  : TCustomImageList;

     FListBox      : TAXWRListBox;
     FColorPicker  : TAXWRMultiColorPicker;

     FMRUAction    : TAction;

     FShowListBoxEv: TAXWRibbonButtonEvent;

     FDebug        : TStrings;

     procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
     procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;

     procedure WndProc(var Message: TMessage); override;

     procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
     procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
     procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;

     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

     procedure Loaded; override;

     procedure Setup;

     procedure DrawContentBmp(ACanvas: TCanvas; AX,AY: integer);
     procedure DrawDownBmp(ACanvas: TCanvas; AColor: TColor; AX,AY: integer);
     procedure DrawRightBmp(ACanvas: TCanvas; AColor: TColor; ARect: TRect);

     procedure ShowAppMenu;
     procedure HideAppMenu;

     procedure ShowListBox(AButton: TAXWRibbonButton; AX,AY: integer; AIsFonts: boolean);
     procedure ShowColorPicker(AButton: TAXWRibbonButton; AX,AY: integer);

     procedure ColorPickerClicked(ASender: TObject);

     function  CloseThisWindow(AFocusWin: HWND): boolean; overload;
     function  CloseThisWindow(AControl: TWinControl): boolean; overload;
public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;

     procedure Paint; override;

     property MRUString      : string read GetMRUString;
     property Office2016Style: boolean read FOffice2016 write SetOffice2016;

     property Debug: TStrings read FDebug write FDebug;
     property SelectedPage: TAXWRibbonPage read GetSelectedPage write SetSelectedPage;
published
{$ifndef AXWRIBBON}
     property AXWActions       : TAXWActionList read FAXWActions write SetAXWActions;
{$endif}
     property Pages            : TAXWRibbonPages read FPages write SetPages;
     property SmallImages      : TCustomImageList read GetSmallImages write FSmallImages;
     property LargeImages      : TCustomImageList read GetLargeImages write FLargeImages;
     property ApplicationMenu  : TAXWRibbonAppMenuItems read FAppMenuItems write SetAppMenu;
     property MRUAction        : TAction read FMRUAction write FMRUAction;
     property SelectedPageIndex: integer read GetSelectedPageIndex write SetSelectedPageIndex;

     property OnBeforeShowListBox: TAXWRibbonButtonEvent read FShowListBoxEv write FShowListBoxEv;

     property Align;
     property Font;
     end;

implementation

type TAXWRAction = class(TAction)
     end;

{$ifndef DELPHI_2007_OR_LATER}
procedure TextRect(ADC: HDC; var Rect: TRect; var Text: string; TextFormat: TTextFormat = []);
const
  CFlags: array[TTextFormats] of Cardinal = (
    DT_BOTTOM, DT_CALCRECT, DT_CENTER, DT_EDITCONTROL, DT_END_ELLIPSIS,
    DT_PATH_ELLIPSIS, DT_EXPANDTABS, DT_EXTERNALLEADING, DT_LEFT,
    DT_MODIFYSTRING, DT_NOCLIP, DT_NOPREFIX, DT_RIGHT, DT_RTLREADING,
    DT_SINGLELINE, DT_TOP, DT_VCENTER, DT_WORDBREAK, DT_HIDEPREFIX,
    0, DT_PREFIXONLY, DT_TABSTOP, DT_WORD_ELLIPSIS,
    0);
var
  Format: cardinal;
  LDrawTextFlag: TTextFormats;
begin
  Format := 0;
  for LDrawTextFlag := Low(TTextFormats) to High(TTextFormats) do
    if (LDrawTextFlag in TextFormat) then
      Format := Format or CFlags[LDrawTextFlag];

  DrawTextEx(ADC, PChar(Text), Length(Text), Rect, Format, nil);

  if tfModifyString in TextFormat then
    SetLength(Text, StrLen(PChar(Text)));
end;
{$endif}

procedure DrawImageListImage(AImageList: TCustomImageList; ACanvas: TCanvas; AX,AY,AIndex: integer; AEnabled: boolean);
{$ifndef DELPHI_XE_OR_LATER}
var
  BMP: TBitMap;
{$endif}
begin
{$ifdef DELPHI_XE_OR_LATER}
  AImageList.Draw(ACanvas,AX,AY,AIndex,AEnabled);
{$else}
  BMP := TBitMap.Create;
  try
    AImageList.GetBitmap(AIndex,BMP);
    BMP.Transparent := True;
    ACanvas.Draw (AX, AY, BMP);
  finally
    BMP.Free;
  end
{$endif}
end;

function EnumFontsProc(var LogFont: TLogFont; var TextMetric: TTextMetric; FontType: Integer; Data: Pointer): Integer; stdcall;
var
  S   : TStrings;
  Temp: string;
begin
  S := TStrings(Data);
  Temp := LogFont.lfFaceName;
  // @ = Exclude vertical fonts.
  if ((S.Count = 0) or (AnsiCompareText(S[S.Count - 1], Temp) <> 0)) and (Temp[1] <> '@') then
    S.AddObject(Temp,TObject(FontType));
  Result := 1;
end;

procedure GetAvailableFonts(AList: TStrings);
var
  DC: HDC;
  LF: TLogFont;
begin
  DC := GetDC(0);
  FillChar(LF,SizeOf(TLogFont),0);
  LF.lfCharset := DEFAULT_CHARSET;
  EnumFontFamiliesEx(DC, LF, @EnumFontsProc, Windows.LPARAM(AList), 0);
  ReleaseDC(0, DC);
end;

function RectHit(AX,AY: integer; ARect: TRect): boolean; {$ifdef D2006PLUS} inline; {$endif}
begin
  Result := (AX >= ARect.Left) and (AY >= ARect.Top) and (AX <= ARect.Right) and (AY <= ARect.Bottom);
end;

function RectCenterX(ARect: TRect; AWidth: integer): integer; {$ifdef D2006PLUS} inline; {$endif}
begin
  Result := ARect.Left + ((ARect.Right - ARect.Left) - AWidth) div 2;
end;

function RectCenterY(ARect: TRect; AHeight: integer): integer; {$ifdef D2006PLUS} inline; {$endif}
begin
  Result := ARect.Top + ((ARect.Bottom - ARect.Top) - AHeight) div 2;
end;

function RevRGB(RGB: longword): longword; {$ifdef D2006PLUS} inline; {$endif}
begin
  Result := ((RGB and $FF0000) shr 16) + (RGB and $00FF00) + ((RGB and $0000FF) shl 16);
end;

function CPos(const C: Char; const S: string): integer;
begin
  for Result := 1 to Length(S) do begin
    if S[Result] = C then
      Exit;
  end;
  Result := -1;
end;

function RCPos(const C: Char; const S: string): integer;
begin
  for Result := Length(S) downto 1 do begin
    if S[Result] = C then
      Exit;
  end;
  Result := -1;
end;

function SplitAtChar(const C: Char; var S: string): string;
var
  p: integer;
begin
  p := CPos(C,S);
  if p > 0 then begin
    Result := Copy(S,1,p - 1);
    S := Copy(S,p + 1,MAXINT);
  end
  else begin
    Result := S;
    S := '';
  end;
end;

procedure PaintText(ACanvas: TCanvas; AText: string; ARect: TRect; AButtonState: TAXWRMouseState; AFormat: TTextFormat);
begin
  Windows.SetBkMode(ACanvas.Handle,TRANSPARENT);

{$ifdef DELPHI_2007_OR_LATER}
  ACanvas.TextRect(ARect,AText,[tfSingleLine,tfVerticalCenter] + AFormat)
{$else}
  TextRect(ACanvas.Handle,ARect,AText,[tfSingleLine,tfVerticalCenter] + AFormat)
{$endif}
end;


{ TAXWRibbon }

function TAXWRibbon.CloseThisWindow(AFocusWin: HWND): boolean;
begin
  Result := FAppMenu <> Nil;
  if Result then
    Result := not ((AFocusWin = FAppMenu.FLbMenu.Handle) or (AFocusWin = FAppMenu.FLbPage.Handle));
end;

function TAXWRibbon.CloseThisWindow(AControl: TWinControl): boolean;
begin
  Result := FAppMenu <> Nil;
  if Result then
    Result := not ((AControl = FAppMenu.FLbMenu) or (AControl = FAppMenu.FLbPage));
end;

procedure TAXWRibbon.CMMouseEnter(var Message: TMessage);
begin

end;

procedure TAXWRibbon.CMMouseLeave(var Message: TMessage);
begin
  if FCurrItem <> Nil then begin
    FCurrItem.State := amsNormal;
    FCurrItem.Paint(Canvas);

    FCurrItem := Nil;
  end;
end;

procedure TAXWRibbon.ColorPickerClicked(ASender: TObject);
begin
  FColorPicker.FButton.ColorPickerClicked;
end;

constructor TAXWRibbon.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FOffice2016 := True;

  FToDestroy := TObjectList.Create;

  DoubleBuffered := True;

  Font.Name := 'Calibri';
  Font.Size := 10;

  Width := 100;
  Height := 100;

  Align := alTop;

  FPages := TAXWRibbonPages.Create(Self);
  FAppMenuItems := TAXWRibbonAppMenuItems.Create(Self);
end;

destructor TAXWRibbon.Destroy;
begin
  FPages.Free;
  FAppMenuItems.Free;

  FToDestroy.Free;

  inherited;
end;

procedure TAXWRibbon.DrawContentBmp(ACanvas: TCanvas; AX, AY: integer);
begin
  ACanvas.MoveTo(AX,AY + 5);
  ACanvas.LineTo(AX,AY);
  ACanvas.LineTo(AX + 6,AY);

  ACanvas.MoveTo(AX + 3,AY + 6);
  ACanvas.LineTo(AX + 6,AY + 6);
  ACanvas.LineTo(AX + 6,AY + 2);

  ACanvas.MoveTo(AX + 4,AY + 5);
  ACanvas.LineTo(AX + 5,AY + 5);
  ACanvas.LineTo(AX + 5,AY + 3);

  ACanvas.MoveTo(AX + 4,AY + 4);
  ACanvas.LineTo(AX + 2,AY + 2);
end;

procedure TAXWRibbon.DrawDownBmp(ACanvas: TCanvas; AColor: TColor; AX, AY: integer);
var
  Pts: array[0..3] of TPoint;
begin
  ACanvas.Pen.Color := AColor;
  ACanvas.Brush.Color := AColor;

  Pts[0].X := AX - 3;
  Pts[0].Y := AY;

  Pts[1].X := AX + 3;
  Pts[1].Y := AY;

  Pts[2].X := AX;
  Pts[2].Y := AY + 3;

  Pts[3] := Pts[0];

  ACanvas.Polygon(Pts);
end;

procedure TAXWRibbon.DrawRightBmp(ACanvas: TCanvas; AColor: TColor; ARect: TRect);
var
  X,Y: integer;
  Pts: array[0..3] of TPoint;
begin
  ACanvas.Pen.Color := AColor;
  ACanvas.Brush.Color := AColor;

  X := ARect.Left + (ARect.Right - ARect.Left) div 2;
  Y := ARect.Top + (ARect.Bottom - ARect.Top) div 2;

  Pts[0].X := X;
  Pts[0].Y := Y - 3;

  Pts[1].X := X + 3;
  Pts[1].Y := Y;

  Pts[2].X := X;
  Pts[2].Y := Y + 3;

  Pts[3] := Pts[0];

  ACanvas.Polygon(Pts);
end;

function TAXWRibbon.GetLargeImages: TCustomImageList;
begin
{$ifndef AXWRIBBON}
  if (FAXWActions <> Nil) and (FAXWActions.LargeImages <> Nil) then
    Result := FAXWActions.LargeImages
  else
{$endif}
    Result := FLargeImages;
end;

function TAXWRibbon.GetMRUString: string;
begin
  if (FAppMenu <> Nil) and (FAppMenu.FMRUIndex >= 0) and (FAppMenu.FMRUIndex < FAppMenuItems.FMRU.Count) then
    Result := FAppMenuItems.FMRU[FAppMenu.FMRUIndex]
  else
    Result := '';
end;

function TAXWRibbon.GetSelectedPage: TAXWRibbonPage;
begin
  Result := FPages.FSelected;
end;

function TAXWRibbon.GetSelectedPageIndex: integer;
begin
  if FPages.Count > 0 then
    Result := FPages.SelectedPage.Index
  else
    Result := 0;
end;

function TAXWRibbon.GetSmallImages: TCustomImageList;
begin
{$ifndef AXWRIBBON}
  if (FAXWActions <> Nil) and (FAXWActions.SmallImages <> Nil) then
    Result := FAXWActions.SmallImages
  else
{$endif}
    Result := FSmallImages;
end;

procedure TAXWRibbon.HideAppMenu;
begin
  if FAppMenu <> Nil then begin
    FPages.FTabs.FSelected := FAppMenu.FCurrTab;

    FPages.FTabs.Paint(Canvas);

    TWinControl(Owner).RemoveControl(FAppMenu);

    FAppMenu := Nil;
  end;
end;

procedure TAXWRibbon.Loaded;
begin
  inherited;

end;

procedure TAXWRibbon.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;

  FMouseIsDown := True;

  FToDestroy.Clear;

  if FCurrItem <> Nil then begin
    if (FCurrItem is TAXWRibbonTab) and (TAXWRibbonTab(FCurrItem).Index = 0) then
      ShowAppMenu
    else begin
      HideAppMenu;

      FCurrItem.MouseDown(Button,Shift,X,Y);

      if FCurrItem <> Nil then
        FCurrItem.Paint(Canvas);
    end;
  end;
end;

procedure TAXWRibbon.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  Item: TAXWRCollectionItem;
begin
//  Canvas.Font.Color := clWhite;
//  Canvas.Brush.Color := clNavy;
//  Canvas.TextOut(500,50,Format('X: %d  Y: %d',[X,Y]));

  Item := FPages.Hit(X,Y);
  if Item <> FCurrItem then begin
    if FCurrItem <> Nil then begin
      FCurrItem.State := amsNormal;
      FCurrItem.Paint(Canvas);
    end;

    if (Item is TAXWRibbonButton) or (Item is TAXWRibbonSubButton) or (Item is TAXWRibbonTab) then begin
      FCurrItem := Item;

      if FCurrItem <> Nil then begin
        FCurrItem.State := amsHoover;
        FCurrItem.Paint(Canvas);
      end;
    end
    else
      FCurrItem := Nil;
  end;
end;

procedure TAXWRibbon.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;

  if not FMouseIsDown then
    Exit;

  if FCurrItem <> Nil then begin
    FCurrItem.MouseUp(Button,Shift,X,Y);

    if FCurrItem <> Nil then
      FCurrItem.Paint(Canvas);
  end;

  FMouseIsDown := False;
end;

procedure TAXWRibbon.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;

{$ifndef AXWRIBBON}
  if (Operation = opRemove) and (AComponent = FAXWActions) then begin
    if FAXWActions.SmallImages = FSmallImages then
      FSmallImages := Nil;

    if FAXWActions.LargeImages = FLargeImages then
      FLargeImages := Nil;
  end;
{$endif}
end;

procedure TAXWRibbon.Paint;
var
  X,Y: integer;
begin
  if not FSetupDone then begin
    Setup;

    FSetupDone := True;
  end;

  X := 0;
  Y := 0;
  FPages.Calc(Canvas,X,Y);

  Canvas.Brush.Color := RevRGB(AXWR_COLOR);
  Canvas.FillRect(Rect(0,0,Width,Height));

  Canvas.Brush.Color := RevRGB(AXWR_COLOR_TAB);
  Canvas.FillRect(Rect(0,0,Width,FTabHeight));

  Canvas.Pen.Color := RevRGB(AXWR_COLOR_LINE);
  Canvas.MoveTo(0,Height - 1);
  Canvas.LineTo(Width,Height - 1);

  FPages.Paint(Canvas);
end;

procedure TAXWRibbon.SetAppMenu(const Value: TAXWRibbonAppMenuItems);
begin
  FAppMenuItems.Assign(Value);
end;

{$ifndef AXWRIBBON}
procedure TAXWRibbon.SetAXWActions(const Value: TAXWActionList);
begin
  if FAXWActions <> Nil then
    FAXWActions.RemoveFreeNotification(Self);

  FAXWActions := Value;
  if FAXWActions <> Nil then begin
    if FSmallImages = Nil then
      FSmallImages := TImageList(FAXWActions.Images);
    if FLargeImages = Nil then
      FLargeImages := FAXWActions.LargeImages;

    FAXWActions.FreeNotification(Self);
  end
  else begin
    FSmallImages := Nil;
    FLargeImages := Nil;
  end;
end;
{$endif}

procedure TAXWRibbon.SetOffice2016(const Value: boolean);
begin
//  FOffice2016 := Value;
//
//  Invalidate;
end;

procedure TAXWRibbon.SetPages(const Value: TAXWRibbonPages);
begin
  FPages.Assign(Value);
end;

procedure TAXWRibbon.SetSelectedPage(const Value: TAXWRibbonPage);
begin
  FPages.SetSelectedPage(Value);
end;

procedure TAXWRibbon.SetSelectedPageIndex(const Value: integer);
begin
  if (Value >= 0) and (Value < FPages.Count) then begin
    SetSelectedPage(FPages[Value]);

    Invalidate;
  end;
end;

procedure TAXWRibbon.Setup;
var
  TM: TTEXTMETRIC;
begin
  Canvas.Font.Assign(Font);

  GetTextMetrics(Canvas.Handle,TM);

  FTextHeight := TM.tmHeight;
  FRowHeight := Round(FTextHeight * 1.6);
  FTextMargin := TM.tmAveCharWidth;
  FCharWidth := TM.tmAveCharWidth;
  FTabHeight := FTextHeight * 2;
  FTabMargin := TM.tmAveCharWidth * 2;
  FGrpCapSize := Round(FTextHeight * 1.2);
  FContentSize := FTextHeight;
  FPageHeight := FRowHeight * 3 + FGrpCapSize;
  FLineWidth := 1;
  FLargeWidth := AXWR_SIZE_LARGE_BUTTON;
  FSmallImgSz := 16;
  FLargeImgSz := 32;
  FSmallSplitSz := FCharWidth * 2;
  FCheckBoxSz := FTextHeight;

  Width := 200;
  Height := FTabHeight + FPageHeight + 1;
end;

procedure TAXWRibbon.ShowAppMenu;
var
  i : integer;
begin
  if FAppMenu = Nil then begin
    FAppMenu := TAXWRibbonAppMenu.Create(Owner,Self);
    FAppMenu.Visible := False;
    FAppMenu.Parent := TWinControl(Owner);

    FAppMenu.FCurrTab := FPages.FTabs.FSelected;

    for i := 0 to FAppMenuItems.Count - 1 do
      FAppMenu.FLbMenu.Items.AddObject(FAppMenuItems[i].Caption,FAppMenuItems[i]);

//    if (FAppMenuItems.Count > 0) and FAppMenuItems[0].ShowMRU then
//      FAppMenu.FLbPage.Items.Assign(FAppMenuItems.MRU);

    FAppMenu.FLbMenu.FHooverItem := 0;
    FAppMenu.HandleSelectPage(Nil);
    FAppMenu.FLbMenu.FHooverItem := -1;

    FAppMenu.Height := Max(4 * ((FRowHeight * 2) + 2),FAppMenuItems.Count * ((FRowHeight * 2) + 1));

    FPages.FTabs.FSelected := FPages.FTabs[0];
    FPages.FTabs.Paint(Canvas);

    FAppMenu.Left := 0;
    FAppMenu.Top := TAXWRibbonTab(FCurrItem).FRect.Bottom;
    FAppMenu.BringToFront;

    FAppMenu.Visible := True;

    FAppMenu.FLbMenu.SetFocus;
  end;
end;

procedure TAXWRibbon.ShowColorPicker(AButton: TAXWRibbonButton; AX, AY: integer);
begin
  if FColorPicker = Nil then begin
    FColorPicker := TAXWRMultiColorPicker.Create(Owner);
    FColorPicker.Visible := False;
    FColorPicker.Parent := TWinControl(Owner);

    FColorPicker.OnSelect := ColorPickerClicked;
  end;

  FColorPicker.Button := AButton;

  FColorPicker.Left := AX;
  FColorPicker.Top := AY;

  FColorPicker.Visible := True;
  FColorPicker.SetFocus;
end;

procedure TAXWRibbon.ShowListBox(AButton: TAXWRibbonButton; AX, AY: integer; AIsFonts: boolean);
begin
  if FListBox = Nil then begin
    FListBox := TAXWRListBox.Create(Owner,Self);
    // Must be False to avoid flickering.
    FListBox.Visible := False;
    FListBox.Parent := TWinControl(Owner);
  end;

  FListBox.FButton := AButton;

  if AIsFonts then
    FListBox.AXWStyle := ahlbsFonts
  else
    FListBox.AXWStyle := ahlbsNormal;

  FListBox.Visible := False;

  FListBox.Left := AX;
  FListBox.Top := AY;

  FListBox.Width := FCharWidth * AButton.FComboWidth * 3;

  case AButton.FStyle of
    arbsComboBox    : begin
      FListBox.Style := lbOwnerDrawFixed;
      FListBox.Height := FTextHeight * 10;
      FListBox.Font.Size := 8;
    end;
    arbsFontComboBox: begin
      FListBox.Style := lbOwnerDrawVariable;
      FListBox.Height := FTextHeight * 24;
      FListBox.Font.Size := 12;
    end;
  end;

  FListBox.Items.BeginUpdate;

  if AButton.FStyle = arbsFontComboBox then begin
    AButton.FComboItems.Clear;

    GetAvailableFonts(AButton.FComboItems);

    AButton.FComboItems.Sort;
  end;

  FListBox.Items.Assign(AButton.FComboItems);

  FListBox.ItemIndex := AButton.FComboItems.IndexOf(AButton.FCaption);

  FListBox.Items.EndUpdate;

  FListBox.Visible := True;

  FListBox.SetFocus;
end;

procedure TAXWRibbon.WndProc(var Message: TMessage);
begin
  inherited;

  if Message.Msg = WM_AXWR_CLOSEWINDOW then begin
    FCurrItem := Nil;

    HideAppMenu;
  end;
end;

{ TAXWRibbonTab }

procedure TAXWRibbonPage.AssignTo(Dest: TPersistent);
begin
  if Dest is TAXWRibbonPage then begin
  end
  else
    inherited;
end;

procedure TAXWRibbonPage.Calc(ACanvas: TCanvas; var AX, AY: integer);
var
  X,Y: integer;
begin
  X := AX;
  Y := AY;

  FGroups.Calc(ACanvas,X,Y);

  FRect.Left := AX;
  FRect.Top := AY;
  FRect.Right := X;
  FRect.Bottom := FRect.Top + TAXWRibbonPages(Collection).FRibbon.FPageHeight;
end;

constructor TAXWRibbonPage.Create(Collection: TCollection);
begin
  inherited Create(Collection);

  FGroups := TAXWRibbonGroups.Create(TAXWRibbonPages(Collection).FRibbon,Self);

  TAXWRibbonPages(Collection).SetSelectedPage(Self);
end;

destructor TAXWRibbonPage.Destroy;
begin
  FGroups.Free;

  inherited;
end;

function TAXWRibbonPage.GetCaption: string;
begin
  Result := FTab.Caption;
end;

function TAXWRibbonPage.Hit(AX, AY: integer): TAXWRCollectionItem;
begin
  Result := FGroups.Hit(AX,AY);
end;

procedure TAXWRibbonPage.Paint(ACanvas: TCanvas);
begin
  FGroups.Paint(ACanvas);
end;

procedure TAXWRibbonPage.SetCaption(const Value: string);
begin
  FTab.Caption := Value;

  TAXWRibbonPages(Collection).FRibbon.Invalidate;
end;

procedure TAXWRibbonPage.SetGroups(const Value: TAXWRibbonGroups);
begin
  FGroups.Assign(Value);
end;

{ TAXWRibbonTabs }

function TAXWRibbonPages.Add: TAXWRibbonPage;
begin
  Result := TAXWRibbonPage(inherited Add);

  FSelected := Result;

  FRibbon.Invalidate;
end;

procedure TAXWRibbonPages.Calc(ACanvas: TCanvas; var AX, AY: integer);
var
  i  : integer;
  X,Y: integer;
begin
  X := AX;
  Y := AY;

  FTabs.Calc(ACanvas,AX,AY);

  X := AX;
  Y := AY + FRibbon.FTabHeight;
  for i := 0 to Count - 1 do
    Items[i].Calc(ACanvas,X,Y);
end;

constructor TAXWRibbonPages.Create(ARibbon: TAXWRibbon);
begin
  inherited Create(TAXWRibbonPage);

  FRibbon := ARibbon;

  FTabs := TAXWRibbonTabs.Create(FRibbon);
end;

procedure TAXWRibbonPages.Delete(AIndex: integer);
begin
  if (FSelected <> Nil) and (AIndex = FSelected.Index) then begin
    if Count > 2 then
      SetSelectedPage(Items[1])
    else if Count > 1 then
      SetSelectedPage(Items[0])
    else
      SetSelectedPage(Nil);
  end;

  inherited Delete(AIndex);
end;

destructor TAXWRibbonPages.Destroy;
begin
  FTabs.Free;

  inherited;
end;

function TAXWRibbonPages.GetOwner: TPersistent;
begin
  Result := FRibbon;
end;

function TAXWRibbonPages.GetPage(Index: Integer): TAXWRibbonPage;
begin
  Result := TAXWRibbonPage(inherited Items[Index]);
end;

function TAXWRibbonPages.Hit(AX, AY: integer): TAXWRCollectionItem;
begin
  if FSelected <> Nil then
    Result := FSelected.Hit(AX,AY)
  else
    Result := Nil;

  if Result = Nil then
    Result := FTabs.Hit(AX,AY);
end;

procedure TAXWRibbonPages.Notify(Item: TCollectionItem; Action: TCollectionNotification);
begin
  inherited;

  case Action of
    cnAdded   : begin
      TAXWRibbonPage(Item).FTab := FTabs.Add;
      TAXWRibbonPage(Item).FTab.FClickEvent := TabClick;
    end;
    cnDeleting: FTabs.Delete(Item.Index);
  end;
end;

procedure TAXWRibbonPages.Paint(ACanvas: TCanvas);
begin
  FTabs.Paint(ACanvas);

  if FSelected <> Nil then
    FSelected.Paint(ACanvas);
end;

procedure TAXWRibbonPages.SetPage(Index: Integer; const Value: TAXWRibbonPage);
begin
  inherited SetItem(Index, Value);
end;

procedure TAXWRibbonPages.SetSelectedPage(const Value: TAXWRibbonPage);
begin
//  if Value.Index > 0 then
    FSelected := Value;

  if FSelected <> Nil then
    FTabs.FSelected := FTabs[Value.Index]
  else
    FTabs.FSelected := Nil;
end;

procedure TAXWRibbonPages.TabClick(ASender: TObject);
begin
  SetSelectedPage(Items[TAXWRibbonTab(ASender).Index]);
end;

{ TAXWRibbonTabs }

function TAXWRibbonTabs.Add: TAXWRibbonTab;
begin
  Result := TAXWRibbonTab(inherited Add);
  Result.FCaption := 'Ribbon Tab';

  FSelected := Result;
end;

procedure TAXWRibbonTabs.Calc(ACanvas: TCanvas; var AX, AY: integer);
var
  i   : integer;
  X,Y: integer;
begin
  X := AX;
  Y := AY;

  FRect.Left := X;
  FRect.Top := Y;
  FRect.Bottom := Y + FRibbon.FTabHeight;
  for i := 0 to Count - 1 do begin
    Y := AY;
    Items[i].Calc(ACanvas,X,Y);
  end;

  FRect.Left := X;
end;

constructor TAXWRibbonTabs.Create(ARibbon: TAXWRibbon);
begin
  inherited Create(TAXWRibbonTab);

  FRibbon := ARibbon;
end;

function TAXWRibbonTabs.GetTab(Index: Integer): TAXWRibbonTab;
begin
  Result := TAXWRibbonTab(inherited Items[Index]);
end;

procedure TAXWRibbonTabs.Paint(ACanvas: TCanvas);
var
  i: integer;
begin
  ACanvas.Font.Assign(FRibbon.Font);

  for i := 0 to Count - 1 do
    Items[i].Paint(ACanvas);
end;

procedure TAXWRibbonTabs.SetTab(Index: Integer; const Value: TAXWRibbonTab);
begin
  inherited SetItem(Index, Value);
end;

{ TAXWRibbonTab }

procedure TAXWRibbonTab.AssignTo(Dest: TPersistent);
begin
  inherited;

end;

procedure TAXWRibbonTab.Calc(ACanvas: TCanvas; var AX, AY: integer);
begin
  FRect.Left := AX;
  FRect.Top := AY;
  FRect.Right := FRect.Left + ACanvas.TextWidth(FCaption) + TAXWRibbonTabs(Collection).FRibbon.FTabMargin * 2;
  FRect.Bottom := AY + TAXWRibbonTabs(Collection).FRibbon.FTabHeight;

  AX := FRect.Right;
  AY := FRect.Bottom;
end;

procedure TAXWRibbonTab.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(FClickEvent) then
    FClickEvent(Self);

  TAXWRibbonTabs(Collection).FRibbon.Invalidate;
end;

procedure TAXWRibbonTab.Paint(ACanvas: TCanvas);
begin
  if Self = TAXWRibbonTabs(Collection).FSelected then begin
    ACanvas.Brush.Color := RevRGB(AXWR_COLOR_TAB_SEL);
    ACanvas.Font.Color := RevRGB(AXWR_COLOR_TAB_FONT_SEL);
  end
  else if FState = amsHoover then begin
    ACanvas.Brush.Color := RevRGB(AXWR_COLOR_TAB_HOOVER);
    ACanvas.Font.Color := RevRGB(AXWR_COLOR_TAB_FONT);
  end
  else begin
    ACanvas.Brush.Color := RevRGB(AXWR_COLOR_TAB);
    ACanvas.Font.Color := RevRGB(AXWR_COLOR_TAB_FONT);
  end;

  ACanvas.FillRect(FRect);

  PaintText(ACanvas,FCaption,FRect,FState,[tfCenter]);
end;

{ TAXWRibbonGroup }

procedure TAXWRibbonGroup.AssignTo(Dest: TPersistent);
begin
  if Dest is TAXWRibbonGroup then begin
  end
  else
    inherited;
end;

procedure TAXWRibbonGroup.Calc(ACanvas: TCanvas; var AX, AY: integer);
var
  X,Y: integer;
  W  : integer;
begin
  X := AX;
  Y := AY;

  W := 0;

  FLarge.Calc(ACanvas,X,Y);

  Y := AY;

  FSmall.Calc(ACanvas,X,Y);

  Inc(W,X - AX);

  FRect.Left := AX;
  FRect.Top := AY;
  FRect.Right := FRect.Left + MAX(W,ACanvas.TextWidth(FCaption) + TAXWRibbonGroups(Collection).FRibbon.FTextMargin * 2 + TAXWRibbonGroups(Collection).FRibbon.FContentSize);
  if FRect.Right <= FRect.Left then
    FRect.Right := FRect.Left + 150;

  FRect.Bottom := FRect.Top + TAXWRibbonGroups(Collection).FRibbon.FPageHeight;

  AX := FRect.Right + TAXWRibbonGroups(Collection).FRibbon.FLineWidth;

  CalcContent;
end;

procedure TAXWRibbonGroup.CalcContent;
begin
  if FContent <> Nil then begin
    FContent.FRect.Left := FRect.Right - TAXWRibbonGroups(Collection).FRibbon.FContentSize;
    FContent.FRect.Top := FRect.Bottom - TAXWRibbonGroups(Collection).FRibbon.FContentSize;
    FContent.FRect.Right := FRect.Right;
    FContent.FRect.Bottom := FRect.Bottom;
  end;
end;

constructor TAXWRibbonGroup.Create(Collection: TCollection);
begin
  inherited Create(Collection);

  FCaption := 'Ribbon Group';

  FLarge := TAXWRibbonButtons.Create(TAXWRibbonGroups(Collection).FRibbon,Self,True);
  FSmall := TAXWRibbonButtons.Create(TAXWRibbonGroups(Collection).FRibbon,Self,False);
end;

destructor TAXWRibbonGroup.Destroy;
begin
  FLarge.Free;
  FSmall.Free;

  if FContent <> Nil then
    FContent.Free;

  inherited;
end;

procedure TAXWRibbonGroup.DoPaint(ACanvas: TCanvas);
var
  R: TRect;
begin
  ACanvas.Brush.Color := RevRGB(AXWR_COLOR_GROUP);

  R := FRect;
  R.Bottom := R.Bottom - TAXWRibbonGroups(Collection).FRibbon.FRowHeight;

  ACanvas.FillRect(R);

  ACanvas.Pen.Color := RevRGB(AXWR_COLOR_LINE);
  ACanvas.MoveTo(FRect.Right,FRect.Top);
  ACanvas.LineTo(FRect.Right,FRect.Bottom);

  FLarge.Paint(ACanvas);
  FSmall.Paint(ACanvas);

  ACanvas.Brush.Color := RevRGB(AXWR_COLOR_GROUP);
  ACanvas.Font.Color := RevRGB(AXWR_COLOR_GROUP_FONT);

  R := FRect;
  R.Top := R.Bottom - TAXWRibbonGroups(Collection).FRibbon.FGrpCapSize;

  if FContent <> Nil then
    R.Right := R.Right - TAXWRibbonGroups(Collection).FRibbon.FContentSize;

  ACanvas.FillRect(R);

  PaintText(ACanvas,FCaption,R,FState,[tfCenter]);

  if FContent <> Nil then
    PaintContent(ACanvas);
end;

procedure TAXWRibbonGroup.DoPaintTheme(ACanvas: TCanvas);
{$ifdef DELPHI_XE2_OR_LATER}
var
  R   : TRect;
  Elem: TThemedElementDetails;
{$endif}
begin
{$ifdef DELPHI_XE2_OR_LATER}
  R := FRect;
  R.Bottom := R.Bottom - TAXWRibbonGroups(Collection).FRibbon.FRowHeight;

  Elem := StyleServices.GetElementDetails(tmPopupBackground);

  StyleServices.DrawElement(ACanvas.Handle,Elem,R);

//  ACanvas.Pen.Color := RevRGB(AXWR_COLOR_LINE);
//  ACanvas.MoveTo(FRect.Right,FRect.Top);
//  ACanvas.LineTo(FRect.Right,FRect.Bottom);
//
  FLarge.Paint(ACanvas);
  FSmall.Paint(ACanvas);
//
//  ACanvas.Brush.Color := RevRGB(AXWR_COLOR_GROUP);
//  ACanvas.Font.Color := RevRGB(AXWR_COLOR_GROUP_FONT);
//
//  R := FRect;
//  R.Top := R.Bottom - TAXWRibbonGroups(Collection).FRibbon.FGrpCapSize;
//
//  if FContent <> Nil then
//    R.Right := R.Right - TAXWRibbonGroups(Collection).FRibbon.FContentSize;
//
//  ACanvas.FillRect(R);
//
//  PaintText(ACanvas,FCaption,R,FState,False);
//
//  if FContent <> Nil then
//    PaintContent(ACanvas);
{$endif}
end;

function TAXWRibbonGroup.GetContent: boolean;
begin
  Result := FContent <> Nil;
end;

function TAXWRibbonGroup.GetContentAction: TAction;
begin
  if FContent <> Nil then
    Result := FContent.Action
  else
    Result := Nil;
end;

function TAXWRibbonGroup.Hit(AX, AY: integer): TAXWRCollectionItem;
begin
  if (FContent <> Nil) and RectHit(AX,AY,FContent.FRect) then
    Result := FContent
  else begin
    Result := FLarge.Hit(AX,AY);
    if Result = Nil then begin
      Result := FSmall.Hit(AX,AY);
      if Result = Nil then
        Result := inherited Hit(AX,AY);
    end;
  end;
end;

procedure TAXWRibbonGroup.Paint(ACanvas: TCanvas);
begin
  if (Collection <> Nil) and not TAXWRibbonGroups(Collection).FRibbon.FOffice2016 then
    DoPaintTheme(ACanvas)
  else
    DoPaint(ACanvas);
end;

procedure TAXWRibbonGroup.PaintContent(ACanvas: TCanvas);
var
  Sz: integer;
begin
  case FContent.FState of
    amsNormal: ACanvas.Brush.Color := RevRGB(AXWR_COLOR_GROUP_CONT_BTN);
    amsHoover: ACanvas.Brush.Color := RevRGB(AXWR_COLOR_GROUP_CONT_BTN_HOOVER);
    amsDown  : ACanvas.Brush.Color := RevRGB(AXWR_COLOR_GROUP_CONT_BTN_DOWN);
  end;

  ACanvas.FillRect(FContent.FRect);

  Sz := (TAXWRibbonGroups(Collection).FRibbon.FGrpCapSize - AXWR_SIZE_CONTENT_BTN) div 2;

  ACanvas.Pen.Color := RevRGB(AXWR_COLOR_GROUP_CONT_IMAGE);
  TAXWRibbonGroups(Collection).FRibbon.DrawContentBmp(ACanvas,FRect.Right - AXWR_SIZE_CONTENT_BTN - Sz,FRect.Bottom - AXWR_SIZE_CONTENT_BTN - Sz);
end;

procedure TAXWRibbonGroup.SetCaption(const Value: string);
begin
  FCaption := Value;

  TAXWRibbonGroups(Collection).FRibbon.Invalidate;
end;

procedure TAXWRibbonGroup.SetContent(const Value: boolean);
begin
  if Value and (FContent = Nil) then begin
    FContent := TAXWRibbonSubButton.Create(Nil);
    FContent.FPaintEvent := PaintContent;

    CalcContent;
  end
  else if not Value and (TAXWRCollectionItem <> Nil) then begin
    FContent.Free;
    FContent := Nil;
  end;
end;

procedure TAXWRibbonGroup.SetContentAction(const Value: TAction);
begin
  if FContent <> Nil then
    FContent.Action := Value;
end;

procedure TAXWRibbonGroup.SetLarge(const Value: TAXWRibbonButtons);
begin
  FLarge.Assign(Value);
end;

procedure TAXWRibbonGroup.SetSmall(const Value: TAXWRibbonButtons);
begin
  FSmall.Assign(Value);
end;

{ TAXWRibbonGroups }

function TAXWRibbonGroups.Add: TAXWRibbonGroup;
begin
  Result := TAXWRibbonGroup(inherited Add);
end;

procedure TAXWRibbonGroups.Calc(ACanvas: TCanvas; var AX, AY: integer);
var
  i: integer;
  X: integer;
begin
  X := AX;

  for i := 0 to Count - 1 do
    Items[i].Calc(ACanvas,X,AY);
end;

constructor TAXWRibbonGroups.Create(ARibbon: TAXWRibbon; APage: TAXWRibbonPage);
begin
  inherited Create(TAXWRibbonGroup);

  FRibbon := ARibbon;
  FPage := APage;
end;

destructor TAXWRibbonGroups.Destroy;
begin

  inherited;
end;

function TAXWRibbonGroups.GetItems(Index: Integer): TAXWRibbonGroup;
begin
  Result := TAXWRibbonGroup(inherited Items[Index]);
end;

function TAXWRibbonGroups.GetOwner: TPersistent;
begin
  Result := FPage;
end;

procedure TAXWRibbonGroups.Paint(ACanvas: TCanvas);
var
  i  : integer;
begin
  for i := 0 to Count - 1 do
    Items[i].Paint(ACanvas);
end;

procedure TAXWRibbonGroups.SetItems(Index: Integer; const Value: TAXWRibbonGroup);
begin
  inherited SetItem(Index, Value);
end;

{ TAXWRCollectionItem }

procedure TAXWRCollectionItem.Calc(ACanvas: TCanvas; var AX, AY: integer);
begin

end;

procedure TAXWRCollectionItem.CaptureChange(AStart: boolean);
begin

end;

constructor TAXWRCollectionItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);

  FEnabled := True;
end;

procedure TAXWRCollectionItem.DoPaintButton(ACanvas: TCanvas; ARect: TRect; ASplitState, ADown: boolean);
var
  Cl  : longword;
  S   : TAXWRMouseState;
begin
  if ACanvas = Nil then
    raise Exception.Create('Canvas is Nil');

  if ADown and (FState = amsHoover) then begin
    ACanvas.Pen.Color := RevRGB(AXWR_COLOR_BTN_DOWN_HOOVER);
    ACanvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_DOWN);
    ACanvas.Rectangle(ARect);

    Exit;
  end
  else if ADown then
    Cl := AXWR_COLOR_BTN_DOWN
  else begin
    Cl := clBlue;

    if ASplitState and (FState = amsNormal) then
      S := amsSplitHoover
    else
      S := FState;

    case S of
      amsNormal     : Cl := AXWR_COLOR_BTN;
      amsHoover     : Cl := AXWR_COLOR_BTN_HOOVER;
      amsSplitHoover: Cl := AXWR_COLOR_BTN_SPLIT_HOOVER;
      amsDown       : Cl := AXWR_COLOR_BTN_DOWN;
      amsHooverDown : ;
      else            Cl := clBlue;
    end;
  end;

  ACanvas.Brush.Color := RevRGB(Cl);
  ACanvas.FillRect(ARect);
end;

procedure TAXWRCollectionItem.DoPaintButtonTheme(ACanvas: TCanvas; ARect: TRect; ASplitState, ADown: boolean);
{$ifdef DELPHI_XE2_OR_LATER}
var
  S   : TAXWRMouseState;
  Elem: TThemedElementDetails;
{$endif}
begin
{$ifdef DELPHI_XE2_OR_LATER}
  if ADown and (FState = amsHoover) then begin
    Elem := StyleServices.GetElementDetails(ttbButtonHot);

    StyleServices.DrawElement(ACanvas.Handle,Elem,ARect);

    Exit;
  end
  else if ADown then
    Elem := StyleServices.GetElementDetails(ttbButtonChecked)
  else begin
    if ASplitState and (FState = amsNormal) then
      S := amsSplitHoover
    else
      S := FState;

    case S of
      amsNormal     : Elem := StyleServices.GetElementDetails(ttbButtonNormal);
      amsHoover     : Elem := StyleServices.GetElementDetails(ttbButtonHot);
      amsSplitHoover: Elem := StyleServices.GetElementDetails(ttbButtonHot);
      amsDown       : Elem := StyleServices.GetElementDetails(ttbButtonChecked);
      amsHooverDown : ;
      else            Elem := StyleServices.GetElementDetails(ttbButtonNormal);
    end;
  end;

  StyleServices.DrawElement(ACanvas.Handle,Elem,ARect);
{$endif}
end;

function TAXWRCollectionItem.Hit(AX, AY: integer): TAXWRCollectionItem;
begin
  if FEnabled and RectHit(AX,AY,FRect) then
    Result := Self
  else
    Result := Nil;
end;

procedure TAXWRCollectionItem.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FState := amsDown;
end;

procedure TAXWRCollectionItem.MouseEnter;
begin
  if FEnabled then
    FState := amsHoover;
end;

procedure TAXWRCollectionItem.MouseLeave;
begin
  if FEnabled then
    FState := amsNormal;
end;

procedure TAXWRCollectionItem.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Hit(X,Y) <> Nil then
    FState := amsHoover
  else
    FState := amsNormal;
end;

procedure TAXWRCollectionItem.Paint;
begin

end;

procedure TAXWRCollectionItem.PaintButton(ACanvas: TCanvas; ARect: TRect; ASplitState,ADown: boolean);
begin
  if (Collection <> Nil) and not TAXWRCollection(Collection).FRibbon.FOffice2016 then
    DoPaintButtonTheme(ACanvas,ARect,ASplitState,ADown)
  else
    DoPaintButton(ACanvas,ARect,ASplitState,ADown)
end;

{ TAXWRCollection }

procedure TAXWRCollection.Calc(ACanvas: TCanvas; var AX, AY: integer);
begin

end;

function TAXWRCollection.Hit(AX, AY: integer): TAXWRCollectionItem;
var
  i: integer;
begin
  for i := 0 to Count - 1 do begin
    Result := TAXWRCollectionItem(Items[i]).Hit(AX,AY);
    if Result <> Nil then
      Exit;
  end;

  Result := Nil;
end;

procedure TAXWRCollection.Paint(ACanvas: TCanvas);
begin

end;

procedure TAXWRCollection.Update(Item: TCollectionItem);
begin
  inherited;

end;

{ TAXWRibbonContentButton }

constructor TAXWRibbonSubButton.Create(AButton: TAXWRibbonButton);
begin
  inherited Create(Nil);

  FButton := AButton;
end;

procedure TAXWRibbonSubButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;

  if FButton <> Nil then
    FButton.SplitClicked;
end;

procedure TAXWRibbonSubButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;

//  if FButton <> Nil then
//    FButton.FDown := False;

  if FAction <> Nil then
    FAction.Execute;
end;

procedure TAXWRibbonSubButton.Paint(ACanvas: TCanvas);
begin
  if Assigned(FPaintEvent) then
    FPaintEvent(ACanvas);
end;

{ TAXWRibbonButton }

procedure TAXWRibbonButton.AssignTo(Dest: TPersistent);
begin
  inherited;

end;

procedure TAXWRibbonButton.Calc(ACanvas: TCanvas; var AX, AY: integer);
begin
  if TAXWRibbonButtons(Collection).FLarge then
    CalcLarge(ACanvas,AX,AY)
  else
    CalcSmall(ACanvas,AX,AY);
end;

procedure TAXWRibbonButton.CalcLarge(ACanvas: TCanvas; var AX, AY: integer);
var
  i : integer;
  S1: string;
  S2: string;
  W: integer;
begin
  case FStyle of
    arbsNormal,
    arbsDropDown,
    arbsSplit,
    arbsSplitThemeColors: begin
      W := 0;

      if FCaption <> '' then begin
        i := RCPos(' ',FCaption);
        if i > 1 then begin
          S1 := Copy(FCaption,1,i - 1);
          S2 := Copy(FCaption,i + 1,MAXINT);

          if FStyle in [arbsSplit,arbsSplitThemeColors,arbsDropDown] then
            W := Max(ACanvas.TextWidth(S1),ACanvas.TextWidth(S2) + AXWR_SIZE_DOWN_BTN * 2)
          else
            W := Max(ACanvas.TextWidth(S1),ACanvas.TextWidth(S2));
        end
        else
          W := ACanvas.TextWidth(FCaption);

        Inc(W,FRibbon.FTextMargin * 2);
      end;

      FRect.Left := AX + AXWR_LARGE_IMAGE_MARG;
      FRect.Top := AY + AXWR_LARGE_IMAGE_MARG;
      FRect.Right := FRect.Left + Max(FRibbon.FLargeWidth,W) - AXWR_LARGE_IMAGE_MARG;
      FRect.Bottom := FRect.Top + FRibbon.FRowHeight * 3 - AXWR_LARGE_IMAGE_MARG;

      AX := FRect.Right + 1;

      if FStyle in [arbsSplit,arbsSplitThemeColors] then begin
        FSplitBtn.FRect := FRect;
        FSplitBtn.FRect.Top := FRect.Bottom - TAXWRibbonButtons(Collection).FRibbon.FTextHeight * 2
      end;
    end;
    arbsComboBox        : ;
    arbsFontComboBox    : ;
    arbsSeparator       : begin
      FRect.Left := AX + AXWR_LARGE_IMAGE_MARG;
      FRect.Top := AY + AXWR_LARGE_IMAGE_MARG;
      FRect.Right := FRect.Left + FRibbon.FCharWidth - AXWR_LARGE_IMAGE_MARG;
      FRect.Bottom := FRect.Top + FRibbon.FRowHeight * 3 - AXWR_LARGE_IMAGE_MARG;

      AX := FRect.Right + 1;
    end;
    arbsBreak: ;
  end;
end;

procedure TAXWRibbonButton.CalcSmall(ACanvas: TCanvas; var AX, AY: integer);
begin
  FRect.Left := AX + AXWR_SMALL_IMAGE_MARG;
  FRect.Top := AY + AXWR_SMALL_IMAGE_MARG;
  FRect.Right := AX - AXWR_SMALL_IMAGE_MARG;
  FRect.Bottom := FRect.Top + FRibbon.FRowHeight - AXWR_SMALL_IMAGE_MARG;

  if FImageIndex >= 0 then
    FRect.Right := FRect.Right + FRibbon.FRowHeight + AXWR_SMALL_IMAGE_MARG * 2;

  case FStyle of
    arbsNormal  : begin
      if FShowCaption and (FCaption <> '') then
        FRect.Right := FRect.Right + ACanvas.TextWidth(FCaption) + FRibbon.FTextMargin * 2;
    end;
    arbsCheckBox: FRect.Right := FRect.Right + FRibbon.FSmallImgSz + ACanvas.TextWidth(FCaption) + FRibbon.FTextMargin * 2;
    arbsDropDown: begin
      if FShowCaption and (FCaption <> '') then
        FRect.Right := FRect.Right + ACanvas.TextWidth(FCaption) + FRibbon.FTextMargin * 2;

      FRect.Right := FRect.Right + FRibbon.FCharWidth;
    end;
    arbsSplit,
    arbsSplitThemeColors: begin
      if FShowCaption and (FCaption <> '') then
        FRect.Right := FRect.Right + ACanvas.TextWidth(FCaption) + FRibbon.FTextMargin;

      FRect.Right := FRect.Right + FRibbon.FSmallSplitSz;

      FSplitBtn.FRect := FRect;
      FSplitBtn.FRect.Left := FRect.Right - FRibbon.FSmallSplitSz;
    end;
    arbsComboBox,
    arbsFontComboBox: begin
      FRect.Right := FRect.Right + FRibbon.FCharWidth * FComboWidth + FRibbon.FTextMargin * 2;

      FRect.Right := FRect.Right + FRibbon.FSmallSplitSz + AXWR_SMALL_IMAGE_MARG * 2;

      FSplitBtn.FRect := FRect;
      FSplitBtn.FRect.Left := FRect.Right - FRibbon.FSmallSplitSz - AXWR_SMALL_IMAGE_MARG * 2;
    end;
    arbsSeparator: FRect.Right := FRect.Left + (FRibbon.FSmallImgSz div 4);
  end;

  if (FStyle <> arbsSeparator) and ((FRect.Right - FRect.Left) < FRibbon.FRowHeight) then
    FRect.Right := FRect.Left + FRibbon.FRowHeight;

  AX := FRect.Right + 1;
end;

procedure TAXWRibbonButton.ColorPickerClicked;
begin
  if FAction <> Nil then begin
    if FAction is TAction then
      TAction(FAction).Caption := IntToStr(Xc12ColorToRGB(FRibbon.FColorPicker.SelectedColor));

    FAction.Execute;
  end;
end;

constructor TAXWRibbonButton.Create(Collection: TCollection);
begin
  inherited Create(Collection);

  FRibbon := TAXWRibbonButtons(Collection).FRibbon;

  FCaption := 'Button';
  FEnabled := True;
  FImageIndex := -1;
  FGroupIndex := -1;
  FComboWidth := 8;

  FComboItems := TStringList.Create;
end;

destructor TAXWRibbonButton.Destroy;
begin
  if FSplitBtn <> Nil then
    FSplitBtn.Free;

  FComboItems.Free;

  inherited;
end;

procedure TAXWRibbonButton.HandleActionChange(ASender: TObject);
begin
  FCaption := TCustomAction(ASender).Caption;
  FAutoCheck := TCustomAction(ASender).AutoCheck;
  FChecked := TCustomAction(ASender).Checked;

  if not TAXWRibbonButtons(Collection).FLarge then
    FImageIndex := TCustomAction(ASender).ImageIndex;

  if TCustomAction(ASender).GroupIndex > 0 then
    FGroupIndex := TCustomAction(ASender).GroupIndex;

  FRibbon.Invalidate;
end;

function TAXWRibbonButton.GetAction: TAction;
begin
  Result := FAction;
end;

function TAXWRibbonButton.GetSplitAction: TAction;
begin
  if FSplitBtn <> Nil then
    Result := FSplitBtn.Action
  else
    Result := Nil;
end;

function TAXWRibbonButton.Hit(AX, AY: integer): TAXWRCollectionItem;
begin
  Result := inherited Hit(AX,AY);

  FSplitHit := False;

  if (Result <> Nil) and (FStyle in [arbsSplit,arbsSplitThemeColors]) and (FSplitBtn.Hit(AX,AY) <> Nil) then begin
    Result := FSplitBtn;

    FSplitHit := True;
  end;
end;

procedure TAXWRibbonButton.ListBoxClicked;
begin
  if FRibbon.FListBox.ItemIndex < 0 then
    Exit;

  FCaption := FRibbon.FListBox.Items[FRibbon.FListBox.ItemIndex];

  Paint(FRibbon.Canvas);

  if FAction <> Nil then begin
    if FAction is TAction then
      TAction(FAction).Caption := FCaption;

    if FAction is TAXWActionBase then begin
      TAXWActionBase(FAction).ValInteger := FRibbon.FListBox.ItemIndex;
      TAXWActionBase(FAction).ValString := FCaption;
    end;

    FAction.Execute;
  end;
end;

procedure TAXWRibbonButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if not FEnabled then
    Exit;

  inherited;

  case FStyle of
    arbsNormal,
    arbsCheckBox    : ;
    arbsDropDown    : DoDropDown;
    arbsSplit       : ;
    arbsSplitThemeColors: ;
    arbsComboBox,
    arbsFontComboBox: begin
      FChecked := True;

      FRibbon.ShowListBox(Self,FRect.Left,FRect.Bottom + 1,FStyle = arbsFontComboBox);
    end;
    arbsSeparator   : ;
    arbsBreak       : ;
  end;
end;

procedure TAXWRibbonButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if not FEnabled then
    Exit;

  inherited;

  if FGroupIndex > 0 then
    TAXWRibbonButtons(Collection).DoGroup(Self);

  if FAutoCheck then
    FChecked := not FChecked;

  if (Action <> Nil) and not (FStyle in [arbsComboBox,arbsFontComboBox]) then
    Action.Execute;
end;

procedure TAXWRibbonButton.Paint(ACanvas: TCanvas);
begin
  if TAXWRibbonButtons(Collection).FLarge then begin
    case FStyle of
      arbsNormal,
      arbsDropDown        : PaintLargeNormal(ACanvas);
      arbsSplit,
      arbsSplitThemeColors: PaintLargeSplit(ACanvas);
      arbsSeparator       : begin
        ACanvas.Pen.Color := RevRGB(AXWR_COLOR_LINE);
        ACanvas.MoveTo(FRect.Left + (FRect.Right - FRect.Left) div 2,FRect.Top);
        ACanvas.LineTo(FRect.Left + (FRect.Right - FRect.Left) div 2,FRect.Bottom);
      end;
      arbsBreak           : ;
    end;
  end
  else begin
    case FStyle of
      arbsNormal,
      arbsDropDown        : PaintSmallNormal(ACanvas);
      arbsCheckBox        : PaintSmallCheckBox(ACanvas);
      arbsSplit,
      arbsSplitThemeColors: PaintSmallSplit(ACanvas);
      arbsFontComboBox,
      arbsComboBox        : PaintComboBox(ACanvas);
      arbsSeparator       : begin
        ACanvas.Pen.Color := RevRGB(AXWR_COLOR_LINE);
        ACanvas.MoveTo(FRect.Left + (FRect.Right - FRect.Left) div 2,FRect.Top);
        ACanvas.LineTo(FRect.Left + (FRect.Right - FRect.Left) div 2,FRect.Bottom);
      end;
      arbsBreak           : ;
    end;
  end;
end;

procedure TAXWRibbonButton.PaintComboBox(ACanvas: TCanvas);
var
  R: TRect;
begin
  if FState = amsHoover then
    ACanvas.Pen.Color := RevRGB(AXWR_COLOR_BTN_DOWN)
  else
    ACanvas.Pen.Color := RevRGB(AXWR_COLOR_COMBO_LINE);
  ACanvas.Brush.Color := RevRGB(AXWR_COLOR_COMBO);
  ACanvas.Rectangle(FRect);

  if FChecked then
    ACanvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_DOWN)
  else begin
    case FState of
      amsNormal: ACanvas.Brush.Color := RevRGB(AXWR_COLOR_COMBO);
      amsHoover: ACanvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_HOOVER);
      amsDown  : ACanvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_DOWN);
    end;
  end;

  R := FSplitBtn.FRect;
  InflateRect(R,-1,-1);
  ACanvas.FillRect(R);

  FRibbon.DrawDownBmp(ACanvas,RevRGB(AXWR_COLOR_GROUP_CONT_IMAGE),FSplitBtn.FRect.Left + (FSplitBtn.FRect.Right - FSplitBtn.FRect.Left) div 2 ,FSplitBtn.FRect.Top + (FSplitBtn.FRect.Bottom - FSplitBtn.FRect.Top) div 2);

  R := FRect;
  InflateRect(R,-1,-1);
  R.Left := R.Left + FRibbon.FTextMargin div 2;
  R.Right := FSplitBtn.FRect.Left - 1;

  DoPaintText(ACanvas,R,[tfLeft]);
end;

procedure TAXWRibbonButton.PaintLargeCaption(ACanvas: TCanvas);
var
  i  : integer;
  R  : TRect;
  S1 : string;
  S2 : string;
begin
  if (FImageIndex >= 0) and (FRibbon.LargeImages <> Nil) and (FImageIndex < FRibbon.LargeImages.Count) then
    DrawImageListImage(FRibbon.LargeImages,ACanvas,RectCenterX(FRect,FRibbon.FLargeImgSz),FRect.Top + AXWR_LARGE_IMAGE_MARG,FImageIndex,FEnabled);

  if FCaption = '' then
    Exit;

  Windows.SetBkMode(ACanvas.Handle,TRANSPARENT);

  // FLargeImages.Draw may change the font color without notify ACanvas.Font
  if FEnabled then
    Windows.SetTextColor(ACanvas.Handle, RevRGB(AXWR_COLOR_FONT_BUTTON))
  else
    Windows.SetTextColor(ACanvas.Handle, RevRGB(AXWR_COLOR_FONT_BUTTON_DISABLED));

//  ACanvas.Font.Color := RevRGB(AXWR_COLOR_FONT_BUTTON);

  R := FRect;
  R.Top := R.Bottom - FRibbon.FTextHeight * 2;

  i := RCPos(' ',FCaption);
  if i > 1 then begin
    S1 := Copy(FCaption,1,i - 1);
    S2 := Copy(FCaption,i + 1,MAXINT);

{$ifdef DELPHI_2007_OR_LATER}
    ACanvas.TextRect(R,S1,[tfSingleline,tfCenter]);
{$else}
    TextRect(ACanvas.Handle,R,S1,[tfSingleline,tfCenter]);
{$endif}

    R.Top := R.Top + FRibbon.FTextHeight;

    if FStyle in [arbsSplit,arbsSplitThemeColors,arbsDropDown] then
      R.Right := R.Right - AXWR_SIZE_DOWN_BTN * 2;

{$ifdef DELPHI_2007_OR_LATER}
    ACanvas.TextRect(R,S2,[tfSingleline,tfCenter]);
{$else}
    TextRect(ACanvas.Handle,R,S2,[tfSingleline,tfCenter]);
{$endif}

    if FStyle in [arbsSplit,arbsSplitThemeColors,arbsDropDown] then
      FRibbon.DrawDownBmp(ACanvas,RevRGB(AXWR_COLOR_GROUP_CONT_IMAGE),R.Right,R.Top + AXWR_SIZE_DOWN_BTN);
  end
  else begin
{$ifdef DELPHI_2007_OR_LATER}
    ACanvas.TextRect(R,FCaption,[tfSingleline,tfCenter]);
{$else}
    TextRect(ACanvas.Handle,R,FCaption,[tfSingleline,tfCenter]);
{$endif}

    if FStyle in [arbsSplit,arbsSplitThemeColors,arbsDropDown] then
      FRibbon.DrawDownBmp(ACanvas,RevRGB(AXWR_COLOR_GROUP_CONT_IMAGE),R.Left + (R.Right - R.Left) div 2,R.Top + FRibbon.FTextHeight + AXWR_SIZE_DOWN_BTN);
  end;
end;

procedure TAXWRibbonButton.PaintLargeNormal(ACanvas: TCanvas);
begin
  PaintButton(ACanvas,FRect,False,FChecked);

  PaintLargeCaption(ACanvas);
end;

procedure TAXWRibbonButton.PaintLargeSplit(ACanvas: TCanvas);
begin
  PaintSplit(ACanvas);

  PaintLargeCaption(ACanvas);
end;

procedure TAXWRibbonButton.PaintBtnRect(ACanvas: TCanvas; AItem: TAXWRCollectionItem);
var
  R: TRect;
begin
  case AItem.FState of
    amsNormal     : begin
      ACanvas.Brush.Color := clGray;
      ACanvas.FillRect(AItem.FRect);
    end;
    amsHoover     : begin
      ACanvas.Brush.Color := clRed;
      ACanvas.Pen.Width := AXWR_SIZE_BTN_BORDER;
      ACanvas.FrameRect(AItem.FRect);
      ACanvas.Pen.Width := 1;

      R := AItem.FRect;
      InflateRect(R,-AXWR_SIZE_BTN_BORDER,-AXWR_SIZE_BTN_BORDER);
      ACanvas.FillRect(AItem.FRect);
    end;
    amsDown       : begin
      ACanvas.Brush.Color := clYellow;
      ACanvas.FillRect(AItem.FRect);
    end;
  end;
end;

procedure TAXWRibbonButton.PaintSmallCheckBox(ACanvas: TCanvas);
var
  R : TRect;
  H : integer;
  Cl: longword;
begin
  ACanvas.Brush.Color := RevRGB(AXWR_COLOR_BTN);
  ACanvas.FillRect(FRect);

  R := FRect;

  R.Left := R.Left + FRibbon.FTextMargin div 2;
  R.Right := R.Left + FRibbon.FCheckBoxSz;

  H := ((R.Bottom - R.Top) - FRibbon.FCheckBoxSz) div 2;

  R.Top := R.Top + H;
  R.Bottom := R.Bottom - H;

  Cl := clBlue;

  case FState of
    amsNormal     : Cl := AXWR_COLOR_CHECKBOX;
    amsHoover     : Cl := AXWR_COLOR_BTN_HOOVER;
    amsSplitHoover: Cl := AXWR_COLOR_BTN_SPLIT_HOOVER;
    amsDown       : Cl := AXWR_COLOR_BTN_DOWN;
    amsHooverDown : ;
  end;

  ACanvas.Pen.Color := clBlack;
  ACanvas.Brush.Color := Cl;
  ACanvas.Rectangle(R);

  if FChecked then begin
    ACanvas.Pen.Width := 2;

    InflateRect(R,-3,-3);

    ACanvas.MoveTo(R.Left,R.Top);
    ACanvas.LineTo(R.Right - 1,R.Bottom - 1);

    ACanvas.MoveTo(R.Right - 1,R.Top);
    ACanvas.LineTo(R.Left,R.Bottom - 1);

    ACanvas.Pen.Width := 1;
  end;

  R := FRect;

  R.Left := R.Left + FRibbon.FSmallImgSz;

  DoPaintText(ACanvas,R,[tfCenter]);
end;

procedure TAXWRibbonButton.PaintSmallImage(ACanvas: TCanvas; var AX: integer);
begin
  if (FImageIndex >= 0) and (FRibbon.SmallImages <> Nil) then begin
    if FImageIndex < FRibbon.SmallImages.Count then
      DrawImageListImage(FRibbon.SmallImages,ACanvas,AX + ((FRect.Bottom - FRect.Top) - FRibbon.FSmallImgSz) div 2 + 1,RectCenterY(FRect,FRibbon.FSmallImgSz),FImageIndex,True);
    Inc(AX,FRibbon.FSmallImgSz);
  end;
end;

procedure TAXWRibbonButton.PaintSmallNormal(ACanvas: TCanvas);
var
  R: TRect;
  X: integer;
begin
  PaintButton(ACanvas,FRect,False,FChecked);

  X := FRect.Left;

  PaintSmallImage(ACanvas,X);

  R := FRect;
  R.Left := X;

  if FStyle = arbsDropDown then begin
    R.Right := R.Right - FRibbon.FCharWidth div 2;

    FRibbon.DrawDownBmp(ACanvas,RevRGB(AXWR_COLOR_GROUP_CONT_IMAGE),R.Right - FRibbon.FCharWidth div 2 ,R.Top + (R.Bottom - R.Top) div 2);
  end;

  DoPaintText(ACanvas,R,[tfCenter]);
end;

procedure TAXWRibbonButton.PaintSmallSplit(ACanvas: TCanvas);
var
  R: TRect;
  X: integer;
begin
  PaintSplit(ACanvas);

  X := FRect.Left;

  PaintSmallImage(ACanvas,X);

  if FCaption <> '' then begin
    R := FRect;
    R.Left := X + FRibbon.FTextMargin;
    R.Right := R.Right - FRibbon.FSmallSplitSz;

    DoPaintText(ACanvas,R,[tfVerticalCenter]);
  end;

  FRibbon.DrawDownBmp(ACanvas,RevRGB(AXWR_COLOR_GROUP_CONT_IMAGE),FRect.Right - FRibbon.FSmallSplitSz div 2,FRect.Top + FRibbon.FRowHeight div 2);
end;

procedure TAXWRibbonButton.PaintSplit(ACanvas: TCanvas);
var
  R: TRect;
begin
  R := FRect;
  SubtractRect(R,FRect,FSplitBtn.FRect);

  PaintButton(ACanvas,R,FSplitBtn.FState > amsNormal,FChecked);
  FSplitBtn.PaintButton(ACanvas,FSplitBtn.FRect,FState > amsNormal,FChecked);

  if (FState > amsNormal) or (FSplitBtn.FState > amsNormal) then begin
    ACanvas.Pen.Color := RevRGB(AXWR_COLOR_BTN_SPLIT_BORDER);
    ACanvas.Pen.Width := AXWR_SIZE_BTN_BORDER;
    ACanvas.MoveTo(FRect.Left + AXWR_SIZE_BTN_BORDER - 1,FRect.Top + AXWR_SIZE_BTN_BORDER - 1);
    ACanvas.LineTo(FRect.Right - AXWR_SIZE_BTN_BORDER,FRect.Top + AXWR_SIZE_BTN_BORDER - 1);
    ACanvas.LineTo(FRect.Right - AXWR_SIZE_BTN_BORDER,FRect.Bottom - AXWR_SIZE_BTN_BORDER);
    ACanvas.LineTo(FRect.Left + AXWR_SIZE_BTN_BORDER - 1,FRect.Bottom - AXWR_SIZE_BTN_BORDER);
    ACanvas.LineTo(FRect.Left + AXWR_SIZE_BTN_BORDER - 1,FRect.Top + AXWR_SIZE_BTN_BORDER - 1);
    ACanvas.Pen.Width := 1;
  end;
end;

function TAXWRibbonButton.DoDropDown: boolean;
var
  Pt: TPoint;
begin
  Result := False;

  if (FAction <> Nil) and (FDropDownMenu = Nil) and (FAction is TAXWActionBase) then begin
    TAXWActionBase(FAction).Prepare;

    FDropDownMenu := TPopupMenu(TAXWActionBase(FAction).HasObject(TPopupMenu));
  end;

  if FDropDownMenu <> Nil then begin
    FChecked := True;

    Pt.X := FRect.Left;
    Pt.Y := FRect.Bottom + 1;

    Pt := FRibbon.ClientToScreen(Pt);
    FDropDownMenu.Popup(Pt.X,Pt.Y);

    FChecked := False;

    Paint(FRibbon.Canvas);

    Result := True;
  end;
end;

procedure TAXWRibbonButton.DoPaintText(ACanvas: TCanvas; ARect: TRect; AFormat: TTextFormat);
begin
  if (FShowCaption or (FStyle in [arbsComboBox,arbsFontComboBox])) and (FCaption <> '') then begin
    if FEnabled then
      ACanvas.Font.Color := RevRGB(AXWR_COLOR_FONT_BUTTON)
    else
      ACanvas.Font.Color := RevRGB(AXWR_COLOR_FONT_BUTTON_DISABLED);

    PaintText(ACanvas,FCaption,ARect,FState,AFormat);
  end;
end;

procedure TAXWRibbonButton.SetAction(const Value: TAction);
begin
  FAction := Value;
  if FAction <> Nil then begin
    TAXWRAction(FAction).OnChange := HandleActionChange;
    HandleActionChange(Value);
  end;

  // Not handled. AV if action is deleted while running.
//  FAction.FreeNotification(FRibbon);
end;

procedure TAXWRibbonButton.SetBtnColor(ACanvas: TCanvas);
begin
  if FChecked then
    ACanvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_DOWN)
  else begin
    case FState of
      amsNormal: ACanvas.Brush.Color := RevRGB(AXWR_COLOR_BTN);
      amsHoover: ACanvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_HOOVER);
      amsDown  : ACanvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_DOWN);
    end;
  end;
end;

procedure TAXWRibbonButton.SetCaption(const Value: string);
begin
  FCaption := Value;

  FRibbon.Invalidate;
end;

procedure TAXWRibbonButton.SetComboItems(const Value: TStringList);
begin
  FComboItems.Assign(Value);
end;

procedure TAXWRibbonButton.SetComboWidth(const Value: integer);
begin
  FComboWidth := Value;

  FRibbon.Invalidate;
end;

procedure TAXWRibbonButton.SetCheked(const Value: boolean);
begin
  FChecked := Value;

  Paint(FRibbon.Canvas);
end;

procedure TAXWRibbonButton.SetEnabled(const Value: boolean);
begin
  FEnabled := Value;

  Paint(FRibbon.Canvas);
end;

procedure TAXWRibbonButton.SetGroupIndex(const Value: integer);
begin
  FGroupIndex := Value;

  Paint(FRibbon.Canvas);
end;

procedure TAXWRibbonButton.SetImageIndex(const Value: TImageIndex);
begin
  FImageIndex := Value;

  Paint(FRibbon.Canvas);
end;

procedure TAXWRibbonButton.SetShowCaption(const Value: boolean);
begin
  FShowCaption := Value;

  FRibbon.Invalidate;
end;

procedure TAXWRibbonButton.SetSplitAction(const Value: TAction);
begin
  if FSplitBtn <> Nil then
    FSplitBtn.Action := Value;
end;

procedure TAXWRibbonButton.SetStyle(const Value: TAXWRibbonButtonStyle);
begin
  FStyle := Value;

  if (FStyle in [arbsSplit,arbsSplitThemeColors,arbsComboBox,arbsFontComboBox]) and (FSplitBtn = Nil) then begin
    FSplitBtn := TAXWRibbonSubButton.Create(Self);
    FSplitBtn.FPaintEvent := Paint;
  end
  else if (FStyle in [arbsSplit,arbsSplitThemeColors,arbsComboBox,arbsFontComboBox]) and (FSplitBtn <> Nil) then begin
    FSplitBtn.Free;
    FSplitBtn := Nil;
  end
  else if FStyle = arbsCheckBox then begin
    FShowCaption := True;
    FAutoCheck := True;
  end;

  FCapture := FStyle in [arbsComboBox,arbsFontComboBox];

  FRibbon.Invalidate;
end;

procedure TAXWRibbonButton.SplitClicked;
begin
//  FDown := True;

  case FStyle of
    arbsNormal          : ;
    arbsDropDown        : ;
    arbsSplit           : begin
      if not DoDropDown then begin
        if Assigned(FRibbon.FShowListBoxEv) then
          FRibbon.FShowListBoxEv(Self);

        if FComboItems.Count > 0 then begin
          FChecked := True;

          FRibbon.ShowListBox(Self,FRect.Left,FRect.Bottom + 1,FStyle = arbsFontComboBox);
        end;
      end;
    end;
    arbsSplitThemeColors: FRibbon.ShowColorPicker(Self,FRect.Left,FRect.Bottom + 1);
    arbsComboBox        : ;
    arbsFontComboBox    : ;
    arbsSeparator       : ;
    arbsBreak           : ;
  end;
end;

{ TAXWRibbonButtons }

function TAXWRibbonButtons.Add: TAXWRibbonButton;
begin
  Result := TAXWRibbonButton(inherited Add);

  FRibbon.Invalidate;
end;

function TAXWRibbonButtons.Add(ACaption: string; AImageIndex: integer; AAction: TAction): TAXWRibbonButton;
begin
  Result := Add;
  Result.Caption := ACaption;
  Result.ImageIndex := AImageIndex;
  Result.Action := AAction;
end;

procedure TAXWRibbonButtons.Calc(ACanvas: TCanvas; var AX, AY: integer);
begin
  if FLarge then
    CalcLarge(ACanvas,AX,AY)
  else
    CalcSmall(ACanvas,AX,AY);
end;

procedure TAXWRibbonButtons.CalcLarge(ACanvas: TCanvas; var AX, AY: integer);
var
  i  : integer;
  X,Y: integer;
begin
  X := AX;
  Y := AY;

  for i := 0 to Count - 1 do
    Items[i].Calc(ACanvas,X,Y);

  FRect.Left := AX;
  FRect.Top := AY;
  FRect.Right := X;
  FRect.Bottom := FRect.Top + FRibbon.FRowHeight * 3;

  AX := X;
end;

procedure TAXWRibbonButtons.CalcSmall(ACanvas: TCanvas; var AX, AY: integer);
var
  i   : integer;
  X,Y : integer;
  D   : integer;
  MaxX: integer;
begin
  X := AX;
  Y := AY;

  FBreakCount := 0;

  for i := 0 to Count - 1 do begin
    if Items[i].FStyle = arbsBreak then
      Inc(FBreakCount);
  end;

  FBreakCount := Min(2,FBreakCount);

  case FBreakCount of
    0: D := FRibbon.FRowHeight;
    1: D := FRibbon.FRowHeight div 3;
    else D := 0;
  end;

  Inc(Y,D);

  MaxX := AX;

  for i := 0 to Count - 1 do begin
    case Items[i].FStyle of
//      arbsSeparator: Inc(X,FRibbon.FSmallImgSz div 4);
      arbsBreak    : begin
        MaxX := Max(MaxX,X);
        X := AX;
        Y := Y + FRibbon.FRowHeight + D;
      end;
      else           Items[i].Calc(ACanvas,X,Y);
    end;
  end;

  MaxX := Max(MaxX,X);

  FRect.Left := AX;
  FRect.Top := AY;
  FRect.Right := MaxX;
  FRect.Bottom := FRect.Top + FRibbon.FRowHeight;

  AX := MaxX;
end;

constructor TAXWRibbonButtons.Create(ARibbon: TAXWRibbon; AGroup: TAXWRibbonGroup; ALarge: boolean);
begin
  inherited Create(TAXWRibbonButton);

  FRibbon := ARibbon;
  FGroup := AGroup;
  FLarge := ALarge;
end;

destructor TAXWRibbonButtons.Destroy;
begin

  inherited;
end;

procedure TAXWRibbonButtons.DoGroup(AButton: TAXWRibbonButton);
var
  i: integer;
begin
  for i := 0 to Count - 1 do begin
    if (Items[i].GroupIndex = AButton.GroupIndex) and (Items[i] <> AButton) then
      Items[i].FChecked := False;
  end;

  AButton.FChecked := True;

  FRibbon.Invalidate;
end;

function TAXWRibbonButtons.GetItems(Index: Integer): TAXWRibbonButton;
begin
  Result := TAXWRibbonButton(inherited Items[Index]);
end;

function TAXWRibbonButtons.GetOwner: TPersistent;
begin
  Result := FGroup;
end;

procedure TAXWRibbonButtons.Paint(ACanvas: TCanvas);
var
  i  : integer;
begin
  for i := 0 to Count - 1 do
    Items[i].Paint(ACanvas);
end;

procedure TAXWRibbonButtons.SetItems(Index: Integer; const Value: TAXWRibbonButton);
begin
  inherited SetItem(Index, Value);
end;

{ TAXWRListBox }

constructor TAXWRListBox.Create(AOwner: TComponent; ARibbon: TAXWRibbon);
begin
  inherited Create(AOwner,ARibbon);

  OnClick := DoClick;
end;

procedure TAXWRListBox.DoClick;
begin
  FButton.ListBoxClicked;

  Hide;
end;

procedure TAXWRListBox.Hide;
begin
  Visible := False;

  FButton.Checked := False;
end;

procedure TAXWRListBox.WMKillFocus(var Message: TWMKillFocus);
begin
  Hide;
end;

procedure TAXWRListBox.WndProc(var Message: TMessage);
begin
  inherited;

  if Showing and (Message.Msg = CM_CANCELMODE) and (Message.LParam <> NativeInt(Self)) then begin
    Hide;
  end;
end;

{ TAXWRMultiColorPicker }

procedure TAXWRMultiColorPicker.Hide;
begin
  Visible := False;

  FButton.Checked := False;
end;

procedure TAXWRMultiColorPicker.WMKillFocus(var Message: TWMKillFocus);
begin
  Hide;
end;

procedure TAXWRMultiColorPicker.WndProc(var Message: TMessage);
begin
  inherited;

  if Showing and (Message.Msg = CM_CANCELMODE) and (Message.LParam <> NativeInt(Self)) then
    Hide;
end;

{ TAXWRibbonAppMenu }

constructor TAXWRibbonAppMenu.Create(AOwner: TComponent; ARibbon: TAXWRibbon);
begin
  inherited Create(AOwner);

  FRibbon := ARibbon;

  FMRUIndex := -1;

  Visible := False;

  Width := 500;
  Height := 500;

  FLbMenu := TAXWRHooverListBox.Create(Self,FRibbon);
  InsertControl(FLbMenu);
  FLbMenu.Width := 150;
  FLbMenu.Align := alLeft;
  FLbMenu.Style := lbOwnerDrawFixed;
  FLbMenu.AXWStyle := ahlbsMenu;
  FLbMenu.ItemHeight := FRibbon.FRowHeight * 2;
  FLbMenu.Name := 'Menu';
  FLbMenu.FTimer.OnTimer := HandleSelectPage;
  FLbMenu.OnClick := HandleMenuClick;

  FLbPage := TAXWRHooverListBox.Create(Self,FRibbon);
  InsertControl(FLbPage);
  FLbPage.Align := alClient;
  FLbPage.Style := lbOwnerDrawFixed;
  FLbPage.AXWStyle := ahlbsMenuPage;
  FLbPage.ItemHeight := FRibbon.FRowHeight;
  FLbPage.Name := 'Page';
  FLbPage.OnClick := HandlePageClick;
end;

destructor TAXWRibbonAppMenu.Destroy;
begin
  inherited;
end;

procedure TAXWRibbonAppMenu.HandleMenuClick(ASender: TObject);
var
  MI: TAXWRibbonAppMenuItem;
begin
  if FlbMenu.FHooverItem >= 0 then begin
    MI := TAXWRibbonAppMenuItem(FlbMenu.Items.Objects[FlbMenu.FHooverItem]);

    if MI.Action <> Nil then
      MI.Action.Execute;
  end;

  PostMessage(FRibbon.Handle,WM_AXWR_CLOSEWINDOW,0,0);
end;

procedure TAXWRibbonAppMenu.HandlePageClick(ASender: TObject);
var
  MI: TAXWRibbonAppSubMenuItem;
begin
  FMRUIndex := -1;

  if FlbPage.FHooverItem >= 0 then begin
    MI := TAXWRibbonAppSubMenuItem(FlbPage.Items.Objects[FlbPage.FHooverItem]);

    if MI = Nil then begin
      if FRibbon.FAppMenuItems.FMRU.Count > 0 then begin
        FMRUIndex := FlbPage.FHooverItem;

        if FRibbon.FMRUAction <> Nil then begin
          FRibbon.MRUAction.Execute;

          FRibbon.FAppMenuItems.FMRU.Exchange(FMRUIndex,0);
        end;
      end;
    end
    else if MI.Action <> Nil then
      MI.Action.Execute;
  end;

  PostMessage(FRibbon.Handle,WM_AXWR_CLOSEWINDOW,0,0);
end;

procedure TAXWRibbonAppMenu.HandleSelectPage(ASender: TObject);
var
  i : integer;
  MI: TAXWRibbonAppMenuItem;
begin
  FLbMenu.FTimer.Enabled := False;

  FMRUIndex := -1;

  if FLbMenu.FHooverItem >= 0 then begin
    MI := TAXWRibbonAppMenuItem(FLbMenu.Items.Objects[FLbMenu.FHooverItem]);
    if (MI <> Nil) and (MI.SubMenu.Count > 0) then begin
      FLbPage.ItemHeight := FRibbon.FRowHeight * 2;

      FLbPage.Items.BeginUpdate;

      FLbPage.Items.Clear;

      for i := 0 to MI.SubMenu.Count - 1 do
        FLbPage.Items.AddObject(MI.SubMenu[i].Caption,MI.SubMenu[i]);

      FLbPage.Items.EndUpdate;
    end
    else begin
      FLbPage.ItemHeight := FRibbon.FRowHeight;

      FLbPage.Items.Assign(FRibbon.FAppMenuItems.MRU);
    end;
  end;
end;

{ TAXWRibbonAppMenuItem }

procedure TAXWRibbonAppMenuItem.AssignTo(Dest: TPersistent);
begin
  inherited;

end;

constructor TAXWRibbonAppMenuItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);

  FSubMenu := TAXWRibbonAppSubMenuItems.Create(Self);

  FCaption := 'Menu item';
  FImageIndex := -1;
end;

destructor TAXWRibbonAppMenuItem.Destroy;
begin
  FSubMenu.Free;

  inherited;
end;

procedure TAXWRibbonAppMenuItem.HandleActionChange(ASender: TObject);
begin
  FCaption := TCustomAction(ASender).Caption;

  if FImageIndex < 0 then
    FImageIndex := TCustomAction(ASender).ImageIndex;
end;

procedure TAXWRibbonAppMenuItem.SetAction(const Value: TAction);
begin
  FAction := Value;

  TAXWRAction(FAction).OnChange := HandleActionChange;
  HandleActionChange(Value);
end;

procedure TAXWRibbonAppMenuItem.SetSubMenu(const Value: TAXWRibbonAppSubMenuItems);
begin
  FSubMenu.Assign(Value);
end;

{ TAXWRibbonAppMenuItems }

function TAXWRibbonAppMenuItems.Add: TAXWRibbonAppMenuItem;
begin
  Result := TAXWRibbonAppMenuItem(inherited Add);
end;

procedure TAXWRibbonAppMenuItems.AddMRU(AText: string);
var
  i: integer;
begin
  i := FMRU.IndexOf(AText);
  if i >= 0 then
    FMRU.Exchange(i,0)
  else begin
    FMRU.Insert(0,AText);

    while FMRU.Count > FMRUCount do
      FMRU.Delete(FMRU.Count - 1);
  end;
end;

constructor TAXWRibbonAppMenuItems.Create(ARibbon: TAXWRibbon);
begin
  inherited Create(TAXWRibbonAppMenuItem);

  FRibbon := ARibbon;

  FMRU := TStringList.Create;

  FMRUCount := 15;
end;

destructor TAXWRibbonAppMenuItems.Destroy;
begin
  FMRU.Free;

  inherited;
end;

function TAXWRibbonAppMenuItems.GetItem(Index: Integer): TAXWRibbonAppMenuItem;
begin
  Result := TAXWRibbonAppMenuItem(inherited Items[Index]);
end;

function TAXWRibbonAppMenuItems.GetOwner: TPersistent;
begin
  Result := FRibbon
end;

procedure TAXWRibbonAppMenuItems.SetItem(Index: Integer; const Value: TAXWRibbonAppMenuItem);
begin
  inherited SetItem(Index, Value);
end;

{ TAXWRibbonAppSubMenuItem }

procedure TAXWRibbonAppSubMenuItem.AssignTo(Dest: TPersistent);
begin
  inherited;

end;

constructor TAXWRibbonAppSubMenuItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);

  FCaption := 'Sub menu item';
  FImageIndex := -1;
end;

destructor TAXWRibbonAppSubMenuItem.Destroy;
begin

  inherited;
end;

procedure TAXWRibbonAppSubMenuItem.HandleActionChange(ASender: TObject);
begin
  FCaption := TCustomAction(ASender).Caption;

{$ifndef AXWRIBBON}
  if ASender is TAXWAction then
    FImageIndex := TAXWAction(ASender).LargeImageIndex
  else
{$endif}
    FImageIndex := TCustomAction(ASender).ImageIndex;
end;

procedure TAXWRibbonAppSubMenuItem.SetAction(const Value: TAction);
begin
  FAction := Value;
  TAXWRAction(FAction).OnChange := HandleActionChange;
  HandleActionChange(Value);
end;

{ TAXWRibbonAppSubMenuItems }

function TAXWRibbonAppSubMenuItems.Add: TAXWRibbonAppSubMenuItem;
begin
  Result := TAXWRibbonAppSubMenuItem(inherited Add);
end;

constructor TAXWRibbonAppSubMenuItems.Create(AAppMenuItem: TAXWRibbonAppMenuItem);
begin
  inherited Create(TAXWRibbonAppSubMenuItem);

  FAppMenuItem := AAppMenuItem;
end;

destructor TAXWRibbonAppSubMenuItems.Destroy;
begin

  inherited;
end;

function TAXWRibbonAppSubMenuItems.GetItem(Index: Integer): TAXWRibbonAppSubMenuItem;
begin
  Result := TAXWRibbonAppSubMenuItem(inherited Items[Index]);
end;

function TAXWRibbonAppSubMenuItems.GetOwner: TPersistent;
begin
  Result := FAppMenuItem;
end;

procedure TAXWRibbonAppSubMenuItems.SetItem(Index: Integer; const Value: TAXWRibbonAppSubMenuItem);
begin
  inherited SetItem(Index, Value);
end;

{ TAXWRMenuListBox }


procedure TAXWRHooverListBox.CMMouseLeave(var Message: TMessage);
begin
  FHooverItem := -1;

  Repaint;
end;

constructor TAXWRHooverListBox.Create(AOwner: TComponent; ARibbon: TAXWRibbon);
begin
  inherited Create(AOwner);

  FRibbon := ARibbon;

  FHooverItem := -1;

  DoubleBuffered := True;

  FTimer := TTimer.Create(Self);
  FTimer.Enabled := False;
  FTimer.Interval := 300;
  FTimer.OnTimer := HandleTimer;
end;

destructor TAXWRHooverListBox.Destroy;
begin
  FTimer.Free;

  inherited;
end;

procedure TAXWRHooverListBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  case FAXWStyle of
    ahlbsNormal  : DrawItemNormal(Index,Rect,State);
    ahlbsMenu    : DrawItemMenu(Index,Rect,State);
    ahlbsMenuPage: DrawItemMenuPage(Index,Rect,State);
    ahlbsFonts   : DrawItemFonts(Index,Rect,State);
  end;
end;

procedure TAXWRHooverListBox.DrawItemFonts(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  S: string;
begin
  Canvas.Font.Color := clBlack;
  Canvas.Font.Name := Items[Index];

  if (odFocused in State) or (odSelected in State) then
    Canvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_DOWN)
  else if Index = FHooverItem then
    Canvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_HOOVER);

  Canvas.FillRect(Rect);

  S := Items[Index];

{$ifdef DELPHI_2007_OR_LATER}
  Canvas.TextRect(Rect,S,[tfSingleLine,tfLeft,tfVerticalCenter]);
{$else}
  TextRect(Canvas.Handle,Rect,S,[tfSingleLine,tfLeft,tfVerticalCenter]);
{$endif}
end;

procedure TAXWRHooverListBox.DrawItemMenu(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  S : string;
  R : TRect;
  W : integer;
  MI: TAXWRibbonAppMenuItem;
begin
  MI := TAXWRibbonAppMenuItem(Items.Objects[Index]);

  Canvas.Font.Color := clBlack;

  if Index = FHooverItem then
    Canvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_HOOVER)
  else
    Canvas.Brush.Color := Color;

  Canvas.FillRect(Rect);

  R := Rect;

  R.Left := R.Left + ItemHeight;

  if MI.Separator then begin
    Canvas.Pen.Color := AXWR_COLOR_LINE;
    Canvas.MoveTo(R.Left,R.Bottom - 1);
    Canvas.LineTo(R.Right,R.Bottom - 1);
  end;

  S := MI.Caption;

{$ifdef DELPHI_2007_OR_LATER}
  Canvas.TextRect(R,S,[tfSingleLine,tfLeft,tfVerticalCenter]);
{$else}
  TextRect(Canvas.Handle,R,S,[tfSingleLine,tfLeft,tfVerticalCenter]);
{$endif}

  R.Left := R.Right - FRibbon.FRowHeight;

  if not MI.ShowMRU then begin
    if MI.IsSplit and (Index = FHooverItem) and FHooverSplit then begin
      Canvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_SPLIT_HOOVER);

      Canvas.FillRect(R);
    end;

    FRibbon.DrawRightBmp(Canvas,RevRGB(AXWR_COLOR_GROUP_CONT_IMAGE),R);
  end;

  if (MI.ImageIndex >= 0) and (FRibbon.LargeImages <> Nil) then begin
    W := Rect.Bottom - Rect.Top;

    DrawImageListImage(FRibbon.LargeImages,Canvas,Rect.Left + (W - FRibbon.FLargeImgSz) div 2,Rect.Top + (W - FRibbon.FLargeImgSz) div 2,MI.ImageIndex,True);
  end;
end;

procedure TAXWRHooverListBox.DrawItemMenuPage(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  S : string;
  R : TRect;
  W : integer;
  MI: TAXWRibbonAppSubMenuItem;
begin
  Canvas.Font.Color := clBlack;

  if Index = FHooverItem then
    Canvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_HOOVER)
  else
    Canvas.Brush.Color := Color;

  Canvas.FillRect(Rect);

  MI := TAXWRibbonAppSubMenuItem(Items.Objects[Index]);

  if MI <> Nil then begin
    R := Rect;

    R.Left := R.Left + ItemHeight;

    R.Top := R.Top + FRibbon.FTextHeight div 2;

    S := MI.Caption;

    Canvas.Font.Style := [fsBold];

{$ifdef DELPHI_2007_OR_LATER}
    Canvas.TextRect(R,S,[tfSingleLine,tfLeft,tfTop]);
{$else}
    TextRect(Canvas.Handle,R,S,[tfSingleLine,tfLeft,tfTop]);
{$endif}

    R.Top := R.Top + FRibbon.FTextHeight;
    R.Right := R.Right - FRibbon.FTextMargin;

    S := Mi.Description;

    Canvas.Font.Style := [];

{$ifdef DELPHI_2007_OR_LATER}
    Canvas.TextRect(R,S,[tfWordBreak,tfLeft,tfTop]);
{$else}
    TextRect(Canvas.Handle,R,S,[tfWordBreak,tfLeft,tfTop]);
{$endif}

    if (MI.ImageIndex >= 0) and (FRibbon.LargeImages <> Nil) then begin
      W := Rect.Bottom - Rect.Top;

      DrawImageListImage(FRibbon.LargeImages,Canvas,Rect.Left + (W - FRibbon.FLargeImgSz) div 2,Rect.Top + (W - FRibbon.FLargeImgSz) div 2,MI.ImageIndex,True);
    end;
  end
  else begin
    S := Items[Index];

{$ifdef DELPHI_2007_OR_LATER}
    Canvas.TextRect(Rect,S,[tfSingleLine,tfLeft,tfVerticalCenter]);
{$else}
    TextRect(Canvas.Handle,Rect,S,[tfSingleLine,tfLeft,tfVerticalCenter]);
{$endif}
  end;
end;

procedure TAXWRHooverListBox.DrawItemNormal(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  S: string;
begin
  Canvas.Font.Color := clBlack;

  if (odFocused in State) or (odSelected in State) then
    Canvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_DOWN)
  else if Index = FHooverItem then
    Canvas.Brush.Color := RevRGB(AXWR_COLOR_BTN_HOOVER);

  Canvas.FillRect(Rect);

  S := Items[Index];

{$ifdef DELPHI_2007_OR_LATER}
  Canvas.TextRect(Rect,S,[tfSingleLine,tfLeft,tfVerticalCenter]);
{$else}
  TextRect(Canvas.Handle,Rect,S,[tfSingleLine,tfLeft,tfVerticalCenter]);
{$endif}
end;

procedure TAXWRHooverListBox.HandleTimer(ASender: TObject);
begin

  FTimer.Enabled := False;
end;

procedure TAXWRHooverListBox.MeasureItem(Index: Integer; var Height: Integer);
var
  TM: TTEXTMETRIC;
begin
  inherited;

  if FAXWStyle = ahlbsFonts then begin
    Canvas.Font.Size := 12;
    Canvas.Font.Name := Items[Index];

    GetTextMetrics(Canvas.Handle,TM);

    Height := Round(TM.tmHeight * 1.2);
  end;
end;

procedure TAXWRHooverListBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;

  if Assigned(FClickEvent) then
    FClickEvent(Self);
end;

procedure TAXWRHooverListBox.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  i : integer;
  Pt: TPoint;
  R : TRect;
  HS: boolean;
begin
  Pt.X := X;
  Pt.Y := Y;

  i := ItemAtPos(Pt,False);

  // i can be greater than max index if the mouse is outside the items.
  if (i >= 0) and (i < Items.Count) then begin
    R := ItemRect(i);

    HS := (X >= (R.Right - FRibbon.FRowHeight)) and (X <= R.Right);

    if (i <> FHooverItem) then begin
      FTimer.Enabled := False;

      FHooverSplit := HS;

      FHooverItem := i;

      Repaint;

      if FAXWStyle = ahlbsMenu then
        FTimer.Enabled := True;
    end
    else if HS <> FHooverSplit then begin
      FHooverSplit := HS;

      Repaint;
    end;
  end
  else if i > 0 then begin
    FTimer.Enabled := False;

    FHooverItem := -1;

    Repaint;
  end;
end;

procedure TAXWRHooverListBox.WMKillFocus(var Message: TWMKillFocus);
begin
  if (FRibbon <> Nil) and FRibbon.CloseThisWindow(Message.FocusedWnd) then begin
    PostMessage(FRibbon.Handle,WM_AXWR_CLOSEWINDOW,0,0);

    FTimer.Enabled := False;
  end;
end;

procedure TAXWRHooverListBox.WndProc(var Message: TMessage);
begin
  inherited;

  if (FRibbon <> Nil) and Showing and (Message.Msg = CM_CANCELMODE) and (FRibbon.CloseThisWindow(TWinControl(Message.LParam))) then begin
    PostMessage(FRibbon.Handle,WM_AXWR_CLOSEWINDOW,0,0);

    FTimer.Enabled := False;
  end;
end;

end.
