unit AXWRibbon;

interface

uses Classes, SysUtils, Controls, Graphics, Dialogs;

type TAXWRibbon = class;
     TAXWRibbonPage = class;
     TAXWRibbonGroup = class;

     TAXWRibbonButton = class(TCollectionItem)
protected
public
     constructor Create(Collection: TCollection); override;
     procedure AssignTo(Dest: TPersistent); override;
published
     end;

     TAXWRibbonButtons = class(TCollection)
private
     function GetItems(Index: integer): TAXWRibbonButton;
protected
     FRibbon: TAXWRibbon;
     FGroup : TAXWRibbonGroup;

     function GetOwner: TPersistent; override;
public
     constructor Create(ARibbon: TAXWRibbon; AGroup: TAXWRibbonGroup);
     destructor Destroy; override;

     function Add: TAXWRibbonButton;

     property Items[Index: integer]: TAXWRibbonButton read GetItems; default;
published
     end;

     TAXWRibbonGroup = class(TCollectionItem)
private
     procedure SetLarge(const Value: TAXWRibbonButtons);
     procedure SetSmall(const Value: TAXWRibbonButtons);
protected
     FSmall: TAXWRibbonButtons;
     FLarge: TAXWRibbonButtons;
public
     constructor Create(Collection: TCollection); override;
     destructor Destroy; override;

     procedure AssignTo(Dest: TPersistent); override;
published
     property Small: TAXWRibbonButtons read FSmall write SetSmall;
     property Large: TAXWRibbonButtons read FLarge write SetLarge;
     end;

     TAXWRibbonGroups = class(TCollection)
private
     function GetItems(Index: integer): TAXWRibbonGroup;
protected
     FRibbon: TAXWRibbon;
     FPage  : TAXWRibbonPage;

     function GetOwner: TPersistent; override;
public
     constructor Create(ARibbon: TAXWRibbon; APage: TAXWRibbonPage);
     destructor Destroy; override;

     function Add: TAXWRibbonGroup;

     property Items[Index: integer]: TAXWRibbonGroup read GetItems; default;
published
     end;

     TAXWRibbonPage = class(TCollectionItem)
private
     procedure SetGroups(const Value: TAXWRibbonGroups);
protected
     FGroups: TAXWRibbonGroups;
public
     constructor Create(Collection: TCollection); override;
     destructor Destroy; override;

     procedure AssignTo(Dest: TPersistent); override;
published
     property Groups: TAXWRibbonGroups read FGroups write SetGroups;
     end;

     TAXWRibbonPages = class(TCollection)
private
     function GetItems(Index: integer): TAXWRibbonPage;
protected
     FRibbon: TAXWRibbon;

     function GetOwner: TPersistent; override;
public
     constructor Create(ARibbon: TAXWRibbon);
     destructor Destroy; override;

     function Add: TAXWRibbonPage;

     property Items[Index: integer]: TAXWRibbonPage read GetItems; default;
published
     end;

     TAXWRibbon = class(TCustomControl)
private
     procedure SetPages(const Value: TAXWRibbonPages);
     procedure SetActivePageIndex(const Value: integer);
protected
     FPages          : TAXWRibbonPages;
     FActivePageIndex: integer;
public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;

     procedure Paint; override;
     procedure KeyDown(var Key: Word; Shift: TShiftState); override;
published
     property Pages          : TAXWRibbonPages read FPages write SetPages;
     property ActivePageIndex: integer read FActivePageIndex write SetActivePageIndex;
     end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('AXW', [TAXWRibbon]);
end;

{ TAXWRibbon }

constructor TAXWRibbon.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FPages := TAXWRibbonPages.Create(Self);

  Width := 400;
  Height := 100;

  FActivePageIndex := -1;
end;

destructor TAXWRibbon.Destroy;
begin
  FPages.Free;

  inherited;
end;

procedure TAXWRibbon.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;

end;

procedure TAXWRibbon.Paint;
var
  i: integer;
  j: integer;
  k: integer;
  X,X2,X3: integer;
  P: TAXWRibbonPage;
begin
  inherited;

  Canvas.Brush.Color := clYellow;
  Canvas.Rectangle(0,0,Width,Height);

  Canvas.Pen.Color := clWhite;
  Canvas.Pen.Width := 2;

  if FActivePageIndex >= 0 then begin
    P := FPages[FActivePageIndex];

    X := 0;

    Canvas.Brush.Color := clBlue;

    Canvas.Rectangle(X,0,X + 300,Height);

    X2 := X;
    for j := 0 to P.Groups.Count - 1 do begin
      Canvas.Brush.Color := clGreen;

      Canvas.Rectangle(X2,0,X2 + 100,Height);

      X3 := X2;

      for k := 0 to P.Groups[j].Large.Count - 1 do begin
        Canvas.Brush.Color := clGray;

        Canvas.Rectangle(X3,0,X3 + 20,20);

        Inc(X3,20);
      end;

      for k := 0 to P.Groups[j].Small.Count - 1 do begin
        Canvas.Brush.Color := clGray;

        Canvas.Rectangle(X3,0,X3 + 10,10);

        Inc(X3,10);
      end;

      Inc(X2,100);
    end;
  end;
end;

procedure TAXWRibbon.SetActivePageIndex(const Value: integer);
begin
  if (Value >= 0) and (Value < FPages.Count) then begin
    FActivePageIndex := Value;

    Invalidate;
  end;
end;

procedure TAXWRibbon.SetPages(const Value: TAXWRibbonPages);
begin
  FPages.Assign(Value);
end;

{ TAXWRibbonPages }

function TAXWRibbonPages.Add: TAXWRibbonPage;
begin
  Result := TAXWRibbonPage(inherited Add);
end;

constructor TAXWRibbonPages.Create(ARibbon: TAXWRibbon);
begin
  inherited Create(TAXWRibbonPage);

  FRibbon := ARibbon;
end;

destructor TAXWRibbonPages.Destroy;
begin

  inherited;
end;

function TAXWRibbonPages.GetItems(Index: integer): TAXWRibbonPage;
begin
  Result := TAXWRibbonPage(inherited Items[Index]);
end;

function TAXWRibbonPages.GetOwner: TPersistent;
begin
  Result := FRibbon;
end;

{ TAXWRibbonPage }

procedure TAXWRibbonPage.AssignTo(Dest: TPersistent);
begin
  inherited;

end;

constructor TAXWRibbonPage.Create(Collection: TCollection);
begin
  inherited Create(Collection);

  FGroups := TAXWRibbonGroups.Create(TAXWRibbonPages(Collection).FRibbon,Self);

  TAXWRibbonPages(Collection).FRibbon.SetActivePageIndex(TAXWRibbonPages(Collection).Count);
end;

destructor TAXWRibbonPage.Destroy;
begin
  FGroups.Free;

  inherited;
end;

procedure TAXWRibbonPage.SetGroups(const Value: TAXWRibbonGroups);
begin
  FGroups.Assign(Value);
end;

{ TAXWRibbonGroups }

function TAXWRibbonGroups.Add: TAXWRibbonGroup;
begin
  Result := TAXWRibbonGroup(inherited Add);
end;

constructor TAXWRibbonGroups.Create(ARibbon: TAXWRibbon; APage: TAXWRibbonPage);
begin
  inherited Create(TAXWRibbonGroup);

  FRibbon := ARibbon;
  FPage := APage;
end;

destructor TAXWRibbonGroups.Destroy;
begin

  inherited;
end;

function TAXWRibbonGroups.GetItems(Index: integer): TAXWRibbonGroup;
begin
  Result := TAXWRibbonGroup(inherited Items[Index]);
end;

function TAXWRibbonGroups.GetOwner: TPersistent;
begin
  Result := FPage;
end;

{ TAXWRibbonGroup }

procedure TAXWRibbonGroup.AssignTo(Dest: TPersistent);
begin
  inherited;

end;

constructor TAXWRibbonGroup.Create(Collection: TCollection);
begin
  inherited Create(Collection);

  FSmall := TAXWRibbonButtons.Create(TAXWRibbonGroups(Collection).FRibbon,Self);
  FLarge := TAXWRibbonButtons.Create(TAXWRibbonGroups(Collection).FRibbon,Self);
end;

destructor TAXWRibbonGroup.Destroy;
begin
  FSmall.Free;
  FLarge.Free;

  inherited;
end;

procedure TAXWRibbonGroup.SetLarge(const Value: TAXWRibbonButtons);
begin
  FLarge.Assign(Value);
end;

procedure TAXWRibbonGroup.SetSmall(const Value: TAXWRibbonButtons);
begin
  FSmall.Assign(Value);
end;

{ TAXWRibbonButtons }

function TAXWRibbonButtons.Add: TAXWRibbonButton;
begin
  Result := TAXWRibbonButton(inherited Add);
end;

constructor TAXWRibbonButtons.Create(ARibbon: TAXWRibbon; AGroup: TAXWRibbonGroup);
begin
  inherited Create(TAXWRibbonButton);

  FRibbon := ARibbon;
  FGroup := AGroup;
end;

destructor TAXWRibbonButtons.Destroy;
begin

  inherited;
end;

function TAXWRibbonButtons.GetItems(Index: integer): TAXWRibbonButton;
begin
  Result := TAXWRibbonButton(inherited Items[Index]);
end;

function TAXWRibbonButtons.GetOwner: TPersistent;
begin
  Result := FGroup;
end;

{ TAXWRibbonButton }

procedure TAXWRibbonButton.AssignTo(Dest: TPersistent);
begin
  inherited;

end;

constructor TAXWRibbonButton.Create(Collection: TCollection);
begin
  inherited Create(Collection);

end;

end.
